@@ -21,19 +21,24 @@
 #include "config.h"
 #include "bytestream-socks5.h"
 
-#include <arpa/inet.h>
-#include <errno.h>
-#include <fcntl.h>
-#include <netdb.h>
-#include <net/if.h>
-#include <netinet/in.h>
-#include <string.h>
-#include <sys/ioctl.h>
-#include <sys/socket.h>
-#include <sys/types.h>
-#include <unistd.h>
+#include <gibber/gibber-sockets.h>
 
-#ifdef HAVE_GETIFADDRS
+#include <errno.h>
+
+/* on Darwin, net/if.h requires sys/sockets.h, which is included by
+ * gibber-sockets.h; so this must come after that header */
+#ifdef HAVE_NET_IF_H
+# include <net/if.h>
+#endif
+
+#include <string.h>
+#include <sys/types.h>
+
+#ifdef HAVE_UNISTD_H
+# include <unistd.h>
+#endif
+
+#ifdef HAVE_IFADDRS_H
  #include <ifaddrs.h>
 #endif
 
@@ -120,14 +125,14 @@ struct _Streamhost
 {
   gchar *jid;
   gchar *host;
-  gchar *port;
+  guint16 port;
 };
 typedef struct _Streamhost Streamhost;
 
 static Streamhost *
 streamhost_new (const gchar *jid,
                 const gchar *host,
-                const gchar *port)
+                guint16 port)
 {
   Streamhost *streamhost;
 
@@ -137,7 +142,7 @@ streamhost_new (const gchar *jid,
   streamhost = g_slice_new0 (Streamhost);
   streamhost->jid = g_strdup (jid);
   streamhost->host = g_strdup (host);
-  streamhost->port = g_strdup (port);
+  streamhost->port = port;
 
   return streamhost;
 }
@@ -150,7 +155,7 @@ streamhost_free (Streamhost *streamhost)
 
   g_free (streamhost->jid);
   g_free (streamhost->host);
-  g_free (streamhost->port);
+
   g_slice_free (Streamhost, streamhost);
 }
 
@@ -754,7 +759,8 @@ target_got_connect_reply (GabbleBytestreamSocks5 *self)
       LmMessageNode *node;
       Streamhost *current_streamhost;
 
-      node = lm_message_node_add_child (iq_result->node, "query", "");
+      node = lm_message_node_add_child (
+        wocky_stanza_get_top_node (iq_result), "query", "");
       lm_message_node_set_attribute (node, "xmlns", NS_BYTESTREAMS);
 
       /* streamhost-used informs the other end of the streamhost we
@@ -846,13 +852,14 @@ initiator_got_connect_reply (GabbleBytestreamSocks5 *self)
 
       g_signal_emit_by_name (self, "connection-error");
       g_object_set (self, "state", GABBLE_BYTESTREAM_STATE_CLOSED, NULL);
-      return;
     }
+
+  lm_message_unref (iq);
 }
 
 /* Process the received data and returns the number of bytes that have been
  * used */
-static gsize
+static gssize
 socks5_handle_received_data (GabbleBytestreamSocks5 *self,
                              GString *string)
 {
@@ -881,7 +888,7 @@ socks5_handle_received_data (GabbleBytestreamSocks5 *self,
             DEBUG ("Authentication failed");
 
             socks5_error (self);
-            return string->len;
+            return -1;
           }
 
         /* We have been authorized, let's send a CONNECT command */
@@ -944,7 +951,7 @@ socks5_handle_received_data (GabbleBytestreamSocks5 *self,
             DEBUG ("Connection refused");
 
             socks5_error (self);
-            return string->len;
+            return -1;
           }
 
         if (string->str[3] == SOCKS5_ATYP_DOMAIN)
@@ -965,7 +972,7 @@ socks5_handle_received_data (GabbleBytestreamSocks5 *self,
             DEBUG ("Wrong domain");
 
             socks5_error (self);
-            return string->len;
+            return -1;
           }
 
         if ((guint8) string->len < SOCKS5_MIN_LENGTH + addr_len)
@@ -983,7 +990,7 @@ socks5_handle_received_data (GabbleBytestreamSocks5 *self,
             DEBUG ("Connection refused");
 
             socks5_error (self);
-            return string->len;
+            return -1;
           }
 
         if (priv->socks5_state == SOCKS5_STATE_TARGET_CONNECT_REQUESTED)
@@ -1028,7 +1035,7 @@ socks5_handle_received_data (GabbleBytestreamSocks5 *self,
             DEBUG ("Authentication failed");
 
             socks5_error (self);
-            return string->len;
+            return -1;
           }
 
         /* The auth request string is SOCKS5_VERSION + # of methods + methods */
@@ -1058,7 +1065,7 @@ socks5_handle_received_data (GabbleBytestreamSocks5 *self,
 
         socks5_error (self);
 
-        return auth_len;
+        return -1;
 
       case SOCKS5_STATE_INITIATOR_AWAITING_COMMAND:
         /* The client has been authorized and we are waiting for a command,
@@ -1091,7 +1098,7 @@ socks5_handle_received_data (GabbleBytestreamSocks5 *self,
             DEBUG ("Invalid SOCKS5 connect message");
 
             socks5_error (self);
-            return string->len;
+            return -1;
           }
 
         domain = compute_domain (priv->stream_id, priv->self_full_jid,
@@ -1102,7 +1109,8 @@ socks5_handle_received_data (GabbleBytestreamSocks5 *self,
             DEBUG ("Reject connection to prevent spoofing");
             socks5_close_transport (self);
             socks5_error (self);
-            return string->len;
+            g_free (domain);
+            return -1;
           }
 
         msg[0] = SOCKS5_VERSION;
@@ -1185,7 +1193,7 @@ transport_handler (GibberTransport *transport,
   GabbleBytestreamSocks5 *self = GABBLE_BYTESTREAM_SOCKS5 (user_data);
   GabbleBytestreamSocks5Private *priv =
       GABBLE_BYTESTREAM_SOCKS5_GET_PRIVATE (self);
-  gsize used_bytes;
+  gssize used_bytes;
 
   g_assert (priv->read_buffer != NULL);
   g_string_append_len (priv->read_buffer, (const gchar *) data->data,
@@ -1237,7 +1245,7 @@ socks5_connect (GabbleBytestreamSocks5 *self)
       return;
     }
 
-  DEBUG ("Trying streamhost %s on port %s", streamhost->host,
+  DEBUG ("Trying streamhost %s on port %d", streamhost->host,
       streamhost->port);
 
   transport = gibber_tcp_transport_new ();
@@ -1267,7 +1275,8 @@ gabble_bytestream_socks5_add_streamhost (GabbleBytestreamSocks5 *self,
   const gchar *zeroconf;
   const gchar *jid;
   const gchar *host;
-  const gchar *port;
+  const gchar *portstr;
+  gint64 port;
   Streamhost *streamhost;
 
   g_return_if_fail (!tp_strdiff (streamhost_node->name, "streamhost"));
@@ -1294,22 +1303,29 @@ gabble_bytestream_socks5_add_streamhost (GabbleBytestreamSocks5 *self,
       return;
     }
 
-  port = lm_message_node_get_attribute (streamhost_node, "port");
-  if (port == NULL)
+  portstr = lm_message_node_get_attribute (streamhost_node, "port");
+  if (portstr == NULL)
     {
       DEBUG ("streamhost doesn't contain a port");
       return;
     }
 
-  if (tp_strdiff (jid, priv->peer_jid) && priv->muc_contact)
+  port = g_ascii_strtoll (portstr, NULL, 10);
+  if (port <= 0 || port > G_MAXUINT16)
     {
-      DEBUG ("skip streamhost %s (%s:%s); we don't support relay with muc "
-          "contact", jid, host, port);
+      DEBUG ("Invalid port: %s", portstr);
       return;
     }
 
-  DEBUG ("streamhost with jid %s, host %s and port %s added", jid, host,
-      port);
+  if (tp_strdiff (jid, priv->peer_jid) && priv->muc_contact)
+    {
+      DEBUG ("skip streamhost %s (%s:%"G_GINT64_FORMAT
+          "); we don't support relay with muc contact", jid, host, port);
+      return;
+    }
+
+  DEBUG ("streamhost with jid %s, host %s and port %"G_GINT64_FORMAT" added",
+      jid, host, port);
 
   streamhost = streamhost_new (jid, host, port);
   priv->streamhosts = g_slist_append (priv->streamhosts, streamhost);
@@ -1414,7 +1430,8 @@ gabble_bytestream_socks5_accept (GabbleBytestreamIface *iface,
 
   msg = gabble_bytestream_factory_make_accept_iq (priv->peer_jid,
       priv->stream_init_id, NS_BYTESTREAMS);
-  si = lm_message_node_get_child_with_namespace (msg->node, "si", NS_SI);
+  si = lm_message_node_get_child_with_namespace (
+    wocky_stanza_get_top_node (msg), "si", NS_SI);
   g_assert (si != NULL);
 
   if (func != NULL)
@@ -1451,11 +1468,13 @@ gabble_bytestream_socks5_decline (GabbleBytestreamSocks5 *self,
 
   if (error != NULL && error->domain == GABBLE_XMPP_ERROR)
     {
-      gabble_xmpp_error_to_node (error->code, msg->node, error->message);
+      gabble_xmpp_error_to_node (error->code,
+        wocky_stanza_get_top_node (msg), error->message);
     }
   else
     {
-      gabble_xmpp_error_to_node (XMPP_ERROR_FORBIDDEN, msg->node,
+      gabble_xmpp_error_to_node (XMPP_ERROR_FORBIDDEN,
+          wocky_stanza_get_top_node (msg),
           "Offer Declined");
     }
 
@@ -1534,7 +1553,7 @@ initiator_connected_to_proxy (GabbleBytestreamSocks5 *self)
       return;
     }
 
-  DEBUG ("connect to proxy: %s (%s:%s)", proxy->jid, proxy->host, proxy->port);
+  DEBUG ("connect to proxy: %s (%s:%d)", proxy->jid, proxy->host, proxy->port);
   priv->socks5_state = SOCKS5_STATE_INITIATOR_TRYING_CONNECT;
 
   transport = gibber_tcp_transport_new ();
@@ -1561,8 +1580,8 @@ socks5_init_reply_cb (GabbleConnection *conn,
       LmMessageNode *query, *streamhost = NULL;
       const gchar *jid;
 
-      query = lm_message_node_get_child_with_namespace (reply_msg->node,
-          "query", NS_BYTESTREAMS);
+      query = lm_message_node_get_child_with_namespace (
+        wocky_stanza_get_top_node (reply_msg), "query", NS_BYTESTREAMS);
 
       if (query != NULL)
         streamhost = lm_message_node_get_child (query, "streamhost-used");
@@ -1624,6 +1643,19 @@ socks5_init_error:
   return LM_HANDLER_RESULT_REMOVE_MESSAGE;
 }
 
+#ifdef G_OS_WIN32
+
+static GSList *
+get_local_interfaces_ips (void)
+{
+  /* FIXME: fd.o#24775: please implement this using ioctlsocket() and
+   * SIO_GET_INTERFACE_LIST, if you care about doing SOCKS5 bytestreams on
+   * Windows */
+  return NULL;
+}
+
+#else
+
 /* get_local_interfaces_ips original code from Farsight 2 (function
  * fs_interfaces_get_local_ips in /gst-libs/gst/farsight/fs-interfaces.c).
  *   Copyright (C) 2006 Youness Alaoui <kakaroto@kakaroto.homelinux.net>
@@ -1774,6 +1806,8 @@ get_local_interfaces_ips (void)
 
 #endif /* ! HAVE_GETIFADDRS */
 
+#endif /* ! G_OS_WIN32 */
+
 static void
 new_connection_cb (GibberListener *listener,
                    GibberTransport *transport,
@@ -1814,6 +1848,13 @@ gabble_bytestream_socks5_initiate (GabbleBytestreamIface *iface)
       return FALSE;
     }
 
+  ips = get_local_interfaces_ips ();
+  if (ips == NULL)
+    {
+      DEBUG ("Can't get IP addresses");
+      return FALSE;
+    }
+
   g_assert (priv->listener == NULL);
   priv->listener = gibber_listener_new ();
 
@@ -1837,17 +1878,10 @@ gabble_bytestream_socks5_initiate (GabbleBytestreamIface *iface)
         '@', "mode", "tcp",
       ')', NULL);
 
-  ips = get_local_interfaces_ips ();
-  if (ips == NULL)
-    {
-      DEBUG ("Can't get IP addresses");
-      return FALSE;
-    }
-
   for (ip = ips; ip != NULL; ip = g_slist_next (ip))
     {
       LmMessageNode *node;
-      NodeIter i = node_iter (msg->node);
+      NodeIter i = node_iter (wocky_stanza_get_top_node (msg));
 
       node = lm_message_node_add_child (node_iter_data (i),
           "streamhost", "");
@@ -1874,17 +1908,21 @@ gabble_bytestream_socks5_initiate (GabbleBytestreamIface *iface)
       for (l = proxies; l != NULL; l = g_slist_next (l))
         {
           LmMessageNode *node;
+          gchar *portstr;
           GabbleSocks5Proxy *proxy = (GabbleSocks5Proxy *) l->data;
-          NodeIter i = node_iter (msg->node);
+          NodeIter i = node_iter (wocky_stanza_get_top_node (msg));
 
           node = lm_message_node_add_child (node_iter_data (i),
               "streamhost", "");
 
+          portstr = g_strdup_printf ("%d", proxy->port);
+
           lm_message_node_set_attributes (node,
               "jid", proxy->jid,
               "host", proxy->host,
-              "port", proxy->port,
+              "port", portstr,
               NULL);
+          g_free (portstr);
         }
       g_slist_free (proxies);
     }
