@@ -30,9 +30,6 @@ G_BEGIN_DECLS
 void conn_aliasing_init (GabbleConnection *conn);
 void conn_aliasing_iface_init (gpointer g_iface, gpointer iface_data);
 
-gboolean gabble_conn_aliasing_pep_nick_event_handler (GabbleConnection *conn,
-    LmMessage *msg, TpHandle handle);
-
 void gabble_conn_aliasing_nickname_updated (GObject *object,
     TpHandle handle, gpointer user_data);
 
