@@ -31,7 +31,8 @@
 #include "caps-cache.h"
 #include "connection.h"
 #include "debug.h"
-#include "debugger.h"
+
+#include "extensions/extensions.h"
 
 G_DEFINE_TYPE(GabbleConnectionManager,
     gabble_connection_manager,
@@ -53,7 +54,8 @@ gabble_connection_manager_finalize (GObject *object)
 {
   gabble_caps_cache_free_shared ();
   gabble_debug_free ();
-  gabble_debugger_free_singleton ();
+
+  G_OBJECT_CLASS (gabble_connection_manager_parent_class)->finalize (object);
 }
 
 static void
@@ -95,6 +97,7 @@ struct _GabbleParams {
   gchar *alias;
   GStrv fallback_socks5_proxies;
   guint keepalive_interval;
+  gboolean decloak_automatically;
 };
 
 enum {
@@ -119,6 +122,7 @@ enum {
     JABBER_PARAM_ALIAS,
     JABBER_PARAM_FALLBACK_SOCKS5_PROXIES,
     JABBER_PARAM_KEEPALIVE_INTERVAL,
+    JABBER_PARAM_DECLOAK_AUTOMATICALLY,
 
     LAST_JABBER_PARAM
 };
@@ -130,8 +134,7 @@ static TpCMParamSpec jabber_params[] = {
     /* FIXME: validate the JID according to the RFC */
     tp_cm_param_filter_string_nonempty, NULL },
   { "password", DBUS_TYPE_STRING_AS_STRING, G_TYPE_STRING,
-    TP_CONN_MGR_PARAM_FLAG_REQUIRED | TP_CONN_MGR_PARAM_FLAG_REGISTER |
-      TP_CONN_MGR_PARAM_FLAG_SECRET,
+    TP_CONN_MGR_PARAM_FLAG_REGISTER | TP_CONN_MGR_PARAM_FLAG_SECRET,
     NULL,
     G_STRUCT_OFFSET(GabbleParams, password), NULL, NULL },
 
@@ -225,6 +228,11 @@ static TpCMParamSpec jabber_params[] = {
     TP_CONN_MGR_PARAM_FLAG_HAS_DEFAULT, GUINT_TO_POINTER (30),
     G_STRUCT_OFFSET (GabbleParams, keepalive_interval), NULL, NULL },
 
+  { GABBLE_PROP_CONNECTION_INTERFACE_GABBLE_DECLOAK_DECLOAK_AUTOMATICALLY,
+    DBUS_TYPE_BOOLEAN_AS_STRING, G_TYPE_BOOLEAN,
+    TP_CONN_MGR_PARAM_FLAG_HAS_DEFAULT, GINT_TO_POINTER (FALSE),
+    G_STRUCT_OFFSET (GabbleParams, decloak_automatically), NULL, NULL },
+
   { NULL, NULL, 0, 0, NULL, 0 }
 };
 
@@ -246,6 +254,7 @@ free_params (void *p)
   g_free (params->https_proxy_server);
   g_free (params->fallback_conference_server);
   g_free (params->stun_server);
+  g_free (params->fallback_stun_server);
   g_free (params->alias);
   g_strfreev (params->fallback_socks5_proxies);
 
@@ -341,6 +350,8 @@ _gabble_connection_manager_new_connection (TpBaseConnectionManager *self,
       JABBER_PARAM_FALLBACK_SOCKS5_PROXIES, params->fallback_socks5_proxies);
   SET_PROPERTY_IF_PARAM_SET ("keepalive-interval",
       JABBER_PARAM_KEEPALIVE_INTERVAL, params->keepalive_interval);
+  SET_PROPERTY_IF_PARAM_SET ("decloak-automatically",
+      JABBER_PARAM_DECLOAK_AUTOMATICALLY, params->decloak_automatically);
 
 out:
   return (TpBaseConnection *) conn;
