@@ -28,12 +28,19 @@
 #include <telepathy-glib/contacts-mixin.h>
 #include <telepathy-glib/presence-mixin.h>
 #include <telepathy-glib/dbus-properties-mixin.h>
+#include <telepathy-glib/dbus.h>
 
+#include <wocky/wocky-session.h>
+#include <wocky/wocky-pep-service.h>
+
+#include "gabble/connection.h"
+#include "capabilities.h"
 #include "error.h"
 #include "ft-manager.h"
 #include "jingle-factory.h"
 #include "muc-factory.h"
 #include "olpc-gadget-manager.h"
+#include "slacker.h"
 #include "types.h"
 
 G_BEGIN_DECLS
@@ -70,10 +77,11 @@ typedef enum
   GABBLE_CONNECTION_FEATURES_PRESENCE_INVISIBLE = 1 << 2,
   GABBLE_CONNECTION_FEATURES_PRIVACY = 1 << 3,
   GABBLE_CONNECTION_FEATURES_PEP = 1 << 4,
+  GABBLE_CONNECTION_FEATURES_GOOGLE_MAIL_NOTIFY = 1 << 5,
 } GabbleConnectionFeatures;
 
 typedef struct _GabbleConnectionPrivate GabbleConnectionPrivate;
-typedef struct _GabbleConnectionClass GabbleConnectionClass;
+typedef struct _GabbleConnectionMailNotificationPrivate GabbleConnectionMailNotificationPrivate;
 
 typedef LmHandlerResult (*GabbleConnectionMsgReplyFunc) (GabbleConnection *conn,
                                                          LmMessage *sent_msg,
@@ -85,9 +93,9 @@ typedef LmHandlerResult (*GabbleConnectionMsgReplyFunc) (GabbleConnection *conn,
  * connection.c */
 typedef enum
 {
-  GABBLE_LIST_HANDLE_PUBLISH = 1,
+  GABBLE_LIST_HANDLE_STORED = 1,
+  GABBLE_LIST_HANDLE_PUBLISH,
   GABBLE_LIST_HANDLE_SUBSCRIBE,
-  GABBLE_LIST_HANDLE_STORED,
   GABBLE_LIST_HANDLE_DENY
 } GabbleListHandle;
 
@@ -116,8 +124,12 @@ struct _GabbleConnection {
     TpPresenceMixin presence;
     TpContactsMixin contacts;
 
+    /* DBus daemon instance */
+    TpDBusDaemon *daemon;
+
     /* loudmouth connection */
     LmConnection *lmconn;
+    WockySession *session;
 
     /* channel factories borrowed from TpBaseConnection's list */
     GabbleRoster *roster;
@@ -161,17 +173,38 @@ struct _GabbleConnection {
     /* outstanding avatar requests */
     GHashTable *avatar_requests;
 
+    /* outstanding vcard requests */
+    GHashTable *vcard_requests;
+
     /* jingle factory */
     GabbleJingleFactory *jingle_factory;
 
     /* file transfer manager */
     GabbleFtManager *ft_manager;
 
-    /* temporary, for requestotron support */
-    GPtrArray *channel_factories;
-    GPtrArray *channel_managers;
-    GPtrArray *channel_requests;
-    gboolean has_tried_connection;
+    /* PEP */
+    WockyPepService *pep_nick;
+    WockyPepService *pep_location;
+    WockyPepService *pep_olpc_buddy_props;
+    WockyPepService *pep_olpc_activities;
+    WockyPepService *pep_olpc_current_act;
+    WockyPepService *pep_olpc_act_props;
+
+    /* Sidecars */
+    /* gchar *interface → GabbleSidecar */
+    GHashTable *sidecars;
+
+    /* gchar *interface → GList<DBusGMethodInvocation> */
+    GHashTable *pending_sidecars;
+
+    /* Mail Notification */
+    GabbleConnectionMailNotificationPrivate *mail_priv;
+
+    /* ContactInfo.SupportedFields, or NULL to use the generic one */
+    GPtrArray *contact_info_fields;
+
+    GabbleSlacker *slacker;
+    guint slacker_inactivity_changed_id;
 
     GabbleConnectionPrivate *priv;
 };
@@ -186,26 +219,10 @@ typedef enum {
     GABBLE_CONNECTION_ALIAS_FROM_ROSTER
 } GabbleConnectionAliasSource;
 
-GType gabble_connection_get_type (void);
-
-/* TYPE MACROS */
-#define GABBLE_TYPE_CONNECTION \
-  (gabble_connection_get_type ())
-#define GABBLE_CONNECTION(obj) \
-  (G_TYPE_CHECK_INSTANCE_CAST((obj), GABBLE_TYPE_CONNECTION, GabbleConnection))
-#define GABBLE_CONNECTION_CLASS(klass) \
-  (G_TYPE_CHECK_CLASS_CAST((klass), GABBLE_TYPE_CONNECTION, \
-                           GabbleConnectionClass))
-#define GABBLE_IS_CONNECTION(obj) \
-  (G_TYPE_CHECK_INSTANCE_TYPE((obj), GABBLE_TYPE_CONNECTION))
-#define GABBLE_IS_CONNECTION_CLASS(klass) \
-  (G_TYPE_CHECK_CLASS_TYPE((klass), GABBLE_TYPE_CONNECTION))
-#define GABBLE_CONNECTION_GET_CLASS(obj) \
-  (G_TYPE_INSTANCE_GET_CLASS ((obj), GABBLE_TYPE_CONNECTION, \
-                              GabbleConnectionClass))
-
 gchar *gabble_connection_get_full_jid (GabbleConnection *conn);
 
+WockyPorter *gabble_connection_get_porter (GabbleConnection *conn);
+
 gboolean _gabble_connection_set_properties_from_account (
     GabbleConnection *conn, const gchar *account, GError **error);
 gboolean _gabble_connection_send (GabbleConnection *conn, LmMessage *msg,
@@ -221,22 +238,28 @@ void _gabble_connection_send_iq_error (GabbleConnection *conn,
 const char *_gabble_connection_find_conference_server (GabbleConnection *);
 gchar *gabble_connection_get_canonical_room_name (GabbleConnection *conn,
     const gchar *jid);
-gboolean _gabble_connection_signal_own_presence (GabbleConnection *,
-    GError **);
 
-void gabble_connection_ensure_capabilities (GabbleConnection *conn,
-    GabblePresenceCapabilities caps);
+void gabble_connection_ensure_capabilities (GabbleConnection *self,
+    const GabbleCapabilitySet *ensured);
 
 gboolean gabble_connection_send_presence (GabbleConnection *conn,
     LmMessageSubType sub_type, const gchar *contact, const gchar *status,
     GError **error);
 
+gboolean gabble_connection_send_capabilities (GabbleConnection *self,
+    const gchar *recipient, GError **error);
+
+gboolean gabble_connection_request_decloak (GabbleConnection *self,
+    const gchar *to, const gchar *reason, GError **error);
+
+void gabble_connection_fill_in_caps (GabbleConnection *self,
+    LmMessage *presence_message);
+
 /* extern only for the benefit of the unit tests */
 void _gabble_connection_create_handle_repos (TpBaseConnection *conn,
     TpHandleRepoIface *repos[NUM_TP_HANDLE_TYPES]);
 
 /* For unit tests only */
-void gabble_connection_set_connect_timeout (guint timeout);
 void gabble_connection_set_disco_reply_timeout (guint timeout);
 
 G_END_DECLS
