@@ -48,6 +48,7 @@
 enum
 {
   ITEM_FOUND,
+  DONE,
   LAST_SIGNAL
 };
 
@@ -139,6 +140,16 @@ gabble_disco_class_init (GabbleDiscoClass *gabble_disco_class)
                   NULL, NULL,
                   gabble_marshal_VOID__POINTER,
                   G_TYPE_NONE, 1, G_TYPE_POINTER);
+
+  signals[DONE] =
+    g_signal_new ("done",
+                  G_OBJECT_CLASS_TYPE (gabble_disco_class),
+                  G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
+                  0,
+                  NULL, NULL,
+                  gabble_marshal_VOID__VOID,
+                  G_TYPE_NONE, 0);
+
 }
 
 static void
@@ -388,7 +399,8 @@ request_reply_cb (GabbleConnection *conn, LmMessage *sent_msg,
   if (!g_list_find (priv->requests, request))
     return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
 
-  query_node = lm_message_node_get_child_with_namespace (reply_msg->node,
+  query_node = lm_message_node_get_child_with_namespace (
+      wocky_stanza_get_top_node (reply_msg),
       "query", disco_type_to_xmlns (request->type));
 
   if (lm_message_get_sub_type (reply_msg) == LM_MESSAGE_SUB_TYPE_ERROR)
@@ -499,7 +511,8 @@ gabble_disco_request_with_timeout (GabbleDisco *self, GabbleDiscoType type,
   priv->requests = g_list_prepend (priv->requests, request);
   msg = lm_message_new_with_sub_type (jid, LM_MESSAGE_TYPE_IQ,
                                            LM_MESSAGE_SUB_TYPE_GET);
-  lm_node = lm_message_node_add_child (msg->node, "query", NULL);
+  lm_node = lm_message_node_add_child (
+      wocky_stanza_get_top_node (msg), "query", NULL);
 
   lm_message_node_set_attribute (lm_node, "xmlns", disco_type_to_xmlns (type));
 
@@ -884,7 +897,7 @@ end_cb (gpointer pipeline, gpointer user_data)
   gabble_disco_pipeline_destroy (pipeline);
   priv->service_cache = g_slist_reverse (priv->service_cache);
 
-  /* FIXME - service discovery done - signal that somehow */
+  g_signal_emit (G_OBJECT (disco), signals[DONE], 0);
 }
 
 static void
@@ -936,12 +949,9 @@ gabble_disco_service_find (GabbleDisco *disco,
       if (type != NULL && tp_strdiff (type, item->type))
         continue;
 
-      if (feature != NULL)
-        {
-          gpointer k, v;
-          if (!g_hash_table_lookup_extended (item->features, feature, &k, &v))
-            continue;
-        }
+      if (feature != NULL &&
+          !g_hash_table_lookup_extended (item->features, feature, NULL, NULL))
+        continue;
 
       return item;
     }
