@@ -1,6 +1,6 @@
 /*
  * ft-manager.c - Source for GabbleFtManager
- * Copyright (C) 2009 Collabora Ltd.
+ * Copyright (C) 2009-2010 Collabora Ltd.
  *   @author: Guillaume Desmottes <guillaume.desmottes@collabora.co.uk>
  *
  * This library is free software; you can redistribute it and/or
@@ -18,8 +18,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <config.h>
-
 #define _BSD_SOURCE
 #define _XOPEN_SOURCE /* glibc2 needs this */
 #include <time.h>
@@ -30,8 +28,9 @@
 #include <string.h>
 #include <glib/gstdio.h>
 
+#include "jingle-session.h"
+#include "jingle-share.h"
 #include "caps-channel-manager.h"
-#include "capabilities.h"
 #include "connection.h"
 #include "ft-manager.h"
 #include "error.h"
@@ -59,6 +58,7 @@ channel_manager_iface_init (gpointer, gpointer);
 static void gabble_ft_manager_channel_created (GabbleFtManager *mgr,
     GabbleFileTransferChannel *chan, gpointer request_token);
 
+
 static void caps_channel_manager_iface_init (gpointer g_iface,
     gpointer iface_data);
 
@@ -83,6 +83,7 @@ struct _GabbleFtManagerPrivate
   GList *channels;
   /* path of the temporary directory used to store UNIX sockets */
   gchar *tmp_dir;
+  gulong status_changed_id;
 };
 
 static void
@@ -96,8 +97,13 @@ gabble_ft_manager_init (GabbleFtManager *obj)
   obj->priv->channels = NULL;
 }
 
+static void gabble_ft_manager_constructed (GObject *object);
 static void gabble_ft_manager_dispose (GObject *object);
 static void gabble_ft_manager_finalize (GObject *object);
+static void connection_status_changed_cb (GabbleConnection *conn,
+                                          guint status,
+                                          guint reason,
+                                          GabbleFtManager *self);
 
 static void
 gabble_ft_manager_get_property (GObject *object,
@@ -146,6 +152,7 @@ gabble_ft_manager_class_init (GabbleFtManagerClass *gabble_ft_manager_class)
   g_type_class_add_private (gabble_ft_manager_class,
                             sizeof (GabbleFtManagerPrivate));
 
+  object_class->constructed = gabble_ft_manager_constructed;
   object_class->get_property = gabble_ft_manager_get_property;
   object_class->set_property = gabble_ft_manager_set_property;
 
@@ -161,26 +168,46 @@ gabble_ft_manager_class_init (GabbleFtManagerClass *gabble_ft_manager_class)
   g_object_class_install_property (object_class, PROP_CONNECTION, param_spec);
 }
 
+
+static void
+gabble_ft_manager_constructed (GObject *object)
+{
+  void (*chain_up) (GObject *) =
+      G_OBJECT_CLASS (gabble_ft_manager_parent_class)->constructed;
+  GabbleFtManager *self = GABBLE_FT_MANAGER (object);
+
+  if (chain_up != NULL)
+    chain_up (object);
+
+  self->priv->status_changed_id = g_signal_connect (self->priv->connection,
+      "status-changed", (GCallback) connection_status_changed_cb, object);
+}
+
+static void
+ft_manager_close_all (GabbleFtManager *self)
+{
+  GList *l;
+
+  while ((l = self->priv->channels) != NULL)
+    {
+      gabble_file_transfer_channel_do_close (l->data);
+      /* Channels should have closed and disappeared from the list */
+      g_assert (l != self->priv->channels);
+    }
+}
+
+
 void
 gabble_ft_manager_dispose (GObject *object)
 {
   GabbleFtManager *self = GABBLE_FT_MANAGER (object);
-  GList *tmp, *l;
 
   if (self->priv->dispose_has_run)
     return;
 
   self->priv->dispose_has_run = TRUE;
 
-  tmp = self->priv->channels;
-  self->priv->channels = NULL;
-
-  for (l = tmp; l != NULL; l = g_list_next (l))
-    {
-      g_object_unref (l->data);
-    }
-
-  g_list_free (tmp);
+  g_assert (self->priv->channels == NULL);
 
   if (G_OBJECT_CLASS (gabble_ft_manager_parent_class)->dispose)
     G_OBJECT_CLASS (gabble_ft_manager_parent_class)->dispose (object);
@@ -259,6 +286,31 @@ file_channel_closed_cb (GabbleFileTransferChannel *chan,
     }
 }
 
+static void
+gabble_ft_manager_channels_created (GabbleFtManager *self, GList *channels)
+{
+  GList *i;
+  GHashTable *new_channels = g_hash_table_new_full (g_direct_hash,
+      g_direct_equal, NULL, NULL);
+
+  for (i = channels; i ; i = i->next)
+    {
+      GabbleFileTransferChannel *chan = i->data;
+
+      gabble_signal_connect_weak (chan, "closed",
+          G_CALLBACK (file_channel_closed_cb), G_OBJECT (self));
+
+      self->priv->channels = g_list_append (self->priv->channels, chan);
+      /* The channels can't satisfy a request because this will always be called
+         when we receive an incoming jingle-share session */
+      g_hash_table_insert (new_channels, chan, NULL);
+    }
+
+  tp_channel_manager_emit_new_channels (self, new_channels);
+
+  g_hash_table_destroy (new_channels);
+}
+
 static void
 gabble_ft_manager_channel_created (GabbleFtManager *self,
                                    GabbleFileTransferChannel *chan,
@@ -280,6 +332,104 @@ gabble_ft_manager_channel_created (GabbleFtManager *self,
   g_slist_free (requests);
 }
 
+
+static void
+new_jingle_session_cb (GabbleJingleFactory *jf,
+    GabbleJingleSession *sess,
+    gpointer data)
+{
+  GabbleFtManager *self = GABBLE_FT_MANAGER (data);
+  GTalkFileCollection *gtalk_fc = NULL;
+  GabbleJingleContent *content = NULL;
+  GabbleJingleShareManifest *manifest = NULL;
+  GList *channels = NULL;
+  GList *cs, *i;
+
+  if (gabble_jingle_session_get_content_type (sess) ==
+      GABBLE_TYPE_JINGLE_SHARE)
+    {
+      cs = gabble_jingle_session_get_contents (sess);
+
+      if (cs != NULL)
+        {
+          content = GABBLE_JINGLE_CONTENT (cs->data);
+          g_list_free (cs);
+        }
+
+      if (content == NULL)
+          return;
+
+      gtalk_fc = gtalk_file_collection_new_from_session (jf, sess);
+
+      if (gtalk_fc)
+        {
+          gchar *token = NULL;
+
+          g_object_get (gtalk_fc,
+              "token", &token,
+              NULL);
+
+          manifest = gabble_jingle_share_get_manifest (
+              GABBLE_JINGLE_SHARE (content));
+          for (i = manifest->entries; i; i = i->next)
+            {
+              GabbleJingleShareManifestEntry *entry = i->data;
+              GabbleFileTransferChannel *channel = NULL;
+              gchar *filename = NULL;
+
+              filename = g_strdup_printf ("%s%s",
+                  entry->name, entry->folder? ".tar":"");
+              channel = gabble_file_transfer_channel_new (self->priv->connection,
+                  sess->peer, sess->peer, TP_FILE_TRANSFER_STATE_PENDING,
+                  NULL, filename, entry->size, TP_FILE_HASH_TYPE_NONE, NULL,
+                  NULL, 0, 0, FALSE, NULL, gtalk_fc, token);
+              g_free (filename);
+
+              gtalk_file_collection_add_channel (gtalk_fc, channel);
+              channels = g_list_prepend (channels, channel);
+            }
+
+          if (channels != NULL)
+            gabble_ft_manager_channels_created (self, channels);
+
+          g_list_free (channels);
+
+          /* Channels will hold the reference to the gtalk file collection,
+             so we can drop ours already. If no channels were created,
+             then we need to destroy it anyways */
+          g_object_unref (gtalk_fc);
+        }
+    }
+}
+
+
+static void
+connection_status_changed_cb (GabbleConnection *conn,
+                              guint status,
+                              guint reason,
+                              GabbleFtManager *self)
+{
+
+  switch (status)
+    {
+      case TP_CONNECTION_STATUS_CONNECTING:
+        g_signal_connect (self->priv->connection->jingle_factory,
+            "new-session",
+            G_CALLBACK (new_jingle_session_cb), self);
+        break;
+
+      case TP_CONNECTION_STATUS_DISCONNECTED:
+        ft_manager_close_all (self);
+        if (self->priv->status_changed_id != 0)
+          {
+            g_signal_handler_disconnect (self->priv->connection,
+                self->priv->status_changed_id);
+            self->priv->status_changed_id = 0;
+          }
+        break;
+    }
+}
+
 static gboolean
 gabble_ft_manager_handle_request (TpChannelManager *manager,
                                  gpointer request_token,
@@ -402,7 +552,7 @@ gabble_ft_manager_handle_request (TpChannelManager *manager,
   chan = gabble_file_transfer_channel_new (self->priv->connection,
       handle, base_connection->self_handle, TP_FILE_TRANSFER_STATE_PENDING,
       content_type, filename, size, content_hash_type, content_hash,
-      description, date, initial_offset, NULL, TRUE);
+      description, date, initial_offset, TRUE, NULL, NULL, NULL);
 
   if (!gabble_file_transfer_channel_offer_file (chan, &error))
     {
@@ -475,6 +625,33 @@ gabble_ft_manager_foreach_channel_class (TpChannelManager *manager,
   g_hash_table_destroy (table);
 }
 
+static LmMessageNode *
+hyvaa_vappua (
+    LmMessageNode *si_node,
+    const gchar **filename,
+    const gchar **size_str,
+    GError **error)
+{
+#define die_if_null(var, msg) \
+  if ((var) == NULL) \
+    { \
+      g_set_error (error, GABBLE_XMPP_ERROR, XMPP_ERROR_BAD_REQUEST, msg); \
+      return NULL; \
+    }
+
+  LmMessageNode *file_node = lm_message_node_get_child_with_namespace (si_node,
+      "file", NS_FILE_TRANSFER);
+
+  die_if_null (file_node, "Invalid file transfer SI request: no <file>")
+  die_if_null (*filename = lm_message_node_get_attribute (file_node, "name"),
+      "Invalid file transfer SI request: missing file name")
+  die_if_null (*size_str = lm_message_node_get_attribute (file_node, "size"),
+      "Invalid file transfer SI request: missing file size")
+
+  return file_node;
+#undef die_if_null
+}
+
 void gabble_ft_manager_handle_si_request (GabbleFtManager *self,
                                           GabbleBytestreamIface *bytestream,
                                           TpHandle handle,
@@ -489,48 +666,29 @@ void gabble_ft_manager_handle_si_request (GabbleFtManager *self,
   TpFileHashType content_hash_type;
   GabbleFileTransferChannel *chan;
   gboolean resume_supported;
+  GError *error = NULL;
 
-  si_node = lm_message_node_get_child_with_namespace (msg->node, "si", NS_SI);
+  si_node = lm_message_node_get_child_with_namespace (
+      wocky_stanza_get_top_node (msg), "si", NS_SI);
   g_assert (si_node != NULL);
 
-  file_node = lm_message_node_get_child_with_namespace (si_node, "file",
-      NS_FILE_TRANSFER);
+  file_node = hyvaa_vappua (si_node, &filename, &size_str, &error);
+
   if (file_node == NULL)
     {
-      GError e = { GABBLE_XMPP_ERROR, XMPP_ERROR_BAD_REQUEST,
-          "Invalid file transfer SI request: no <file>" };
-
-      DEBUG ("%s", e.message);
-      gabble_bytestream_iface_close (bytestream, &e);
+      DEBUG ("%s", error->message);
+      gabble_bytestream_iface_close (bytestream, error);
+      g_clear_error (&error);
+      return;
     }
 
-  filename = lm_message_node_get_attribute (file_node, "name");
-  if (filename == NULL)
-    {
-      GError e = { GABBLE_XMPP_ERROR, XMPP_ERROR_BAD_REQUEST,
-          "Invalid file transfer SI request: missing file name" };
-
-      DEBUG ("%s", e.message);
-      gabble_bytestream_iface_close (bytestream, &e);
-    }
-
-  size_str = lm_message_node_get_attribute (file_node, "size");
-  if (size_str == NULL)
-    {
-      GError e = { GABBLE_XMPP_ERROR, XMPP_ERROR_BAD_REQUEST,
-          "Invalid file transfer SI request: missing file size" };
-
-      DEBUG ("%s", e.message);
-      gabble_bytestream_iface_close (bytestream, &e);
-    }
   size = g_ascii_strtoull (size_str, NULL, 0);
 
   content_type = lm_message_node_get_attribute (file_node, "mime-type");
   if (content_type == NULL)
-    {
-      content_type = "application/octet-stream";
-    }
+    content_type = "application/octet-stream";
 
+  /* The hash is always an MD5-sum, if present. */
   content_hash = lm_message_node_get_attribute (file_node, "hash");
   if (content_hash != NULL)
     content_hash_type = TP_FILE_HASH_TYPE_MD5;
@@ -558,7 +716,7 @@ void gabble_ft_manager_handle_si_request (GabbleFtManager *self,
   chan = gabble_file_transfer_channel_new (self->priv->connection,
       handle, handle, TP_FILE_TRANSFER_STATE_PENDING,
       content_type, filename, size, content_hash_type, content_hash,
-      description, date, 0, bytestream, resume_supported);
+      description, date, 0, resume_supported, bytestream, NULL, NULL);
 
   gabble_ft_manager_channel_created (self, chan, NULL);
 }
@@ -586,6 +744,8 @@ gabble_ft_manager_new (GabbleConnection *connection)
       NULL);
 }
 
+#ifdef G_OS_UNIX
+/* We assume that all Unixes have mkdtemp */
 const gchar *
 gabble_ft_manager_get_tmp_dir (GabbleFtManager *self)
 {
@@ -600,18 +760,16 @@ gabble_ft_manager_get_tmp_dir (GabbleFtManager *self)
 
   return self->priv->tmp_dir;
 }
+#endif
 
 static void
-add_file_transfer_channel_class (GPtrArray *arr,
-                                 TpHandle handle)
+add_file_transfer_channel_class (GPtrArray *arr)
 {
   GValue monster = {0, };
   GHashTable *fixed_properties;
   GValue *channel_type_value;
   GValue *target_handle_type_value;
 
-  g_assert (handle != 0);
-
   g_value_init (&monster, TP_STRUCT_TYPE_REQUESTABLE_CHANNEL_CLASS);
   g_value_take_boxed (&monster,
       dbus_g_type_specialized_construct (
@@ -641,141 +799,51 @@ add_file_transfer_channel_class (GPtrArray *arr,
   g_ptr_array_add (arr, g_value_get_boxed (&monster));
 }
 
-/* The channel-manager-specific representation of capabilities for this manager
- * is: NULL = no file transfer, non-NULL = file transfer.
- */
-#define SOME_NON_NULL_POINTER (((char *) NULL) + 1)
-
 static void
-gabble_ft_manager_get_contact_caps (GabbleCapsChannelManager *manager,
-                                    GabbleConnection *conn,
-                                    TpHandle handle,
-                                    GPtrArray *arr)
+gabble_ft_manager_get_contact_caps (
+    GabbleCapsChannelManager *manager G_GNUC_UNUSED,
+    TpHandle handle G_GNUC_UNUSED,
+    const GabbleCapabilitySet *caps,
+    GPtrArray *arr)
 {
-  TpBaseConnection *base = (TpBaseConnection *) conn;
-  GabblePresence *presence;
-
-  g_assert (handle != 0);
-
-  if (handle == base->self_handle)
-    {
-#ifdef ENABLE_ASSUMED_FT_CAP
-      /* For backwards compatibility, the default behaviour of the stable
-       * branch is to say that we can receive file transfers, even if no
-       * client seems to be able to. Disable with
-       * ./configure --disable-assumed-ft-cap */
-      add_file_transfer_channel_class (arr, handle);
-      return;
-#else
-      presence = conn->self_presence;
-#endif
-    }
-  else
-    {
-      presence = gabble_presence_cache_get (conn->presence_cache, handle);
-    }
-
- if (presence == NULL)
-   return;
-
- if (presence->per_channel_manager_caps == NULL)
-   return;
-
- if (g_hash_table_lookup (presence->per_channel_manager_caps, manager) == NULL)
-   return;
-
-  /* FT is supported */
-  add_file_transfer_channel_class (arr, handle);
+  if (gabble_capability_set_has (caps, NS_FILE_TRANSFER) ||
+      gabble_capability_set_has (caps, NS_GOOGLE_FEAT_SHARE))
+    add_file_transfer_channel_class (arr);
 }
 
 static void
-gabble_ft_manager_get_feature_list (
-    GabbleCapsChannelManager *manager,
-    gpointer specific_caps,
-    GSList **features)
+gabble_ft_manager_represent_client (
+    GabbleCapsChannelManager *manager G_GNUC_UNUSED,
+    const gchar *client_name,
+    const GPtrArray *filters,
+    const gchar * const *cap_tokens G_GNUC_UNUSED,
+    GabbleCapabilitySet *cap_set)
 {
-  static const Feature ft = { FEATURE_OPTIONAL, NS_FILE_TRANSFER,
-      PRESENCE_CAP_SI_FILE_TRANSFER };
+  guint i;
 
-  if (specific_caps != NULL)
+  for (i = 0; i < filters->len; i++)
     {
-      *features = g_slist_prepend (*features, (gpointer) &ft);
+      GHashTable *channel_class = g_ptr_array_index (filters, i);
+
+      if (tp_strdiff (tp_asv_get_string (channel_class,
+              TP_IFACE_CHANNEL ".ChannelType"),
+            TP_IFACE_CHANNEL_TYPE_FILE_TRANSFER))
+        continue;
+
+      if (tp_asv_get_uint32 (channel_class,
+            TP_IFACE_CHANNEL ".TargetHandleType", NULL)
+          != TP_HANDLE_TYPE_CONTACT)
+        continue;
+
+      DEBUG ("client %s supports file transfer", client_name);
+      gabble_capability_set_add (cap_set, NS_FILE_TRANSFER);
+      gabble_capability_set_add (cap_set, NS_GOOGLE_FEAT_SHARE);
+      /* there's no point in looking at the subsequent filters if we've
+       * already added the FT capability */
+      break;
     }
 }
 
-static gpointer
-gabble_ft_manager_parse_caps (GabbleCapsChannelManager *manager,
-                              gchar **uris)
-{
-  gchar **i;
-
-  for (i = uris; *i; i++)
-    {
-      if (!tp_strdiff (*i, NS_FILE_TRANSFER))
-        return SOME_NON_NULL_POINTER;
-    }
-
-  return NULL;
-}
-
-static void
-gabble_ft_manager_copy_caps (GabbleCapsChannelManager *manager,
-                             gpointer *specific_caps_out,
-                             gpointer specific_caps_in)
-{
-  *specific_caps_out = specific_caps_in;
-}
-
-static gboolean
-gabble_ft_manager_caps_diff (GabbleCapsChannelManager *manager,
-                             TpHandle handle,
-                             gpointer specific_old_caps,
-                             gpointer specific_new_caps)
-{
-  return specific_old_caps != specific_new_caps;
-}
-
-static void
-gabble_ft_manager_update_caps (GabbleCapsChannelManager *manager,
-    gpointer specific_caps_out,
-    gpointer specific_caps_in G_GNUC_UNUSED)
-{
-  /* We don't need to do anything. If @out is NULL, we won't be called, and
-   * @in will be copied instead; if @out is non-NULL, it means FT is supported,
-   * so it doesn't matter what @in was. */
-  g_return_if_fail (specific_caps_out != NULL);
-}
-
-static void
-gabble_ft_manager_add_self_capability (GabbleCapsChannelManager *manager,
-    GabbleConnection *conn,
-    GHashTable *channel_class)
-{
-  GabblePresence *presence;
-
-  if (tp_strdiff (tp_asv_get_string (channel_class,
-          TP_IFACE_CHANNEL ".ChannelType"),
-        TP_IFACE_CHANNEL_TYPE_FILE_TRANSFER))
-    return;
-
-  if (tp_asv_get_uint32 (channel_class, TP_IFACE_CHANNEL ".TargetHandleType",
-        NULL) != TP_HANDLE_TYPE_CONTACT)
-    {
-      return;
-    }
-
-  presence = conn->self_presence;
-  g_assert (presence != NULL);
-
-  if (presence->per_channel_manager_caps == NULL)
-    presence->per_channel_manager_caps = g_hash_table_new (NULL, NULL);
-
-  /* it doesn't matter whether we already had this capability - this either
-   * changes it from FALSE to TRUE, or from TRUE to TRUE */
-  g_hash_table_insert (presence->per_channel_manager_caps,
-      manager, SOME_NON_NULL_POINTER);
-}
-
 static void
 caps_channel_manager_iface_init (gpointer g_iface,
                                  gpointer iface_data)
@@ -783,10 +851,5 @@ caps_channel_manager_iface_init (gpointer g_iface,
   GabbleCapsChannelManagerIface *iface = g_iface;
 
   iface->get_contact_caps = gabble_ft_manager_get_contact_caps;
-  iface->get_feature_list = gabble_ft_manager_get_feature_list;
-  iface->parse_caps = gabble_ft_manager_parse_caps;
-  iface->copy_caps = gabble_ft_manager_copy_caps;
-  iface->caps_diff = gabble_ft_manager_caps_diff;
-  iface->update_caps = gabble_ft_manager_update_caps;
-  iface->add_self_capability = gabble_ft_manager_add_self_capability;
+  iface->represent_client = gabble_ft_manager_represent_client;
 }
