@@ -25,7 +25,6 @@
 #include <string.h>
 #include <glib.h>
 
-#include <lib/gibber/gibber-resolver.h>
 #include <loudmouth/loudmouth.h>
 #include <libsoup/soup.h>
 
@@ -33,6 +32,7 @@
 
 #include "connection.h"
 #include "debug.h"
+#include "jingle-share.h"
 #include "jingle-media-rtp.h"
 #include "jingle-session.h"
 #include "jingle-transport-google.h"
@@ -59,11 +59,6 @@ enum
   LAST_PROPERTY
 };
 
-/* The 'session' map is keyed by:
- * "<peer's handle>\n<peer's resource>\n<session id>"
- */
-#define SESSION_MAP_KEY_FORMAT "%u\n%s\n%s"
-
 struct _GabbleJingleFactoryPrivate
 {
   GabbleConnection *conn;
@@ -74,7 +69,6 @@ struct _GabbleJingleFactoryPrivate
 
   /* instances of SESSION_MAP_KEY_FORMAT => GabbleJingleSession. */
   GHashTable *sessions;
-  GibberResolver *resolver;
   SoupSession *soup;
 
   gchar *stun_server;
@@ -97,7 +91,7 @@ static LmHandlerResult jingle_cb (LmMessageHandler *handler,
 static GabbleJingleSession *create_session (GabbleJingleFactory *fac,
     const gchar *sid,
     TpHandle peer,
-    const gchar *peer_resource,
+    const gchar *jid,
     gboolean local_hold);
 
 static void session_terminated_cb (GabbleJingleSession *sess,
@@ -140,14 +134,15 @@ gabble_jingle_factory_init (GabbleJingleFactory *obj)
 
   priv->conn = NULL;
   priv->dispose_has_run = FALSE;
-  priv->resolver = gibber_resolver_get_resolver ();
   priv->relay_http_port = 80;
 }
 
 typedef struct {
+    GabbleJingleFactory *factory;
     gchar *stun_server;
     guint16 stun_port;
     gboolean fallback;
+    GCancellable *cancellable;
 } PendingStunServer;
 
 static void
@@ -155,51 +150,53 @@ pending_stun_server_free (gpointer p)
 {
   PendingStunServer *data = p;
 
+  if (data->factory != NULL)
+    g_object_remove_weak_pointer (G_OBJECT (data->factory),
+        (gpointer)&data->factory);
+
+  g_object_unref (data->cancellable);
   g_free (data->stun_server);
   g_slice_free (PendingStunServer, p);
 }
 
 static void
-stun_server_resolved_cb (GibberResolver *resolver,
-                         GList *entries,
-                         GError *error,
-                         gpointer user_data,
-                         GObject *object)
+stun_server_resolved_cb (GObject *resolver,
+                         GAsyncResult *result,
+                         gpointer user_data)
 {
-  GabbleJingleFactory *self = GABBLE_JINGLE_FACTORY (object);
   PendingStunServer *data = user_data;
-  GibberResolverAddrInfo *info;
+  GabbleJingleFactory *self = data->factory;
   GError *e = NULL;
   gchar *stun_server;
+  GList *entries;
 
-  if (error != NULL)
-    {
-      DEBUG ("Failed to resolve STUN server %s:%u: %s",
-          data->stun_server, data->stun_port, error->message);
-      return;
-    }
+  if (self != NULL)
+      g_object_weak_unref (G_OBJECT (self),
+          (GWeakNotify)g_cancellable_cancel, data->cancellable);
+
+  entries = g_resolver_lookup_by_name_finish (
+      G_RESOLVER (resolver), result, &e);
 
   if (entries == NULL)
     {
-      DEBUG ("No results for STUN server %s:%u",
-          data->stun_server, data->stun_port);
-      return;
-    }
-
-  info = entries->data;
-
-  if (!gibber_resolver_sockaddr_to_str ((struct sockaddr *) &(info->sockaddr),
-        info->sockaddr_len, &stun_server, NULL, &e))
-    {
-      DEBUG ("Couldn't convert resolved address of %s to string: %s",
-          data->stun_server, e->message);
+      DEBUG ("Failed to resolve STUN server %s:%u: %s",
+          data->stun_server, data->stun_port, e->message);
       g_error_free (e);
-      return;
+      goto out;
     }
 
+  stun_server = g_inet_address_to_string (entries->data);
+  g_resolver_free_addresses (entries);
+
   DEBUG ("Resolved STUN server %s:%u to %s:%u", data->stun_server,
       data->stun_port, stun_server, data->stun_port);
 
+  if (self == NULL)
+    {
+      g_free (stun_server);
+      goto out;
+    }
+
   if (data->fallback)
     {
       g_free (self->priv->fallback_stun_server);
@@ -212,6 +209,10 @@ stun_server_resolved_cb (GibberResolver *resolver,
       self->priv->stun_server = stun_server;
       self->priv->stun_port = data->stun_port;
     }
+
+out:
+  pending_stun_server_free (data);
+  g_object_unref (resolver);
 }
 
 static void
@@ -220,21 +221,29 @@ take_stun_server (GabbleJingleFactory *self,
                   guint16 stun_port,
                   gboolean fallback)
 {
-  PendingStunServer *data = g_slice_new0 (PendingStunServer);
+  GResolver *resolver;
+  PendingStunServer *data;
 
   if (stun_server == NULL)
     return;
 
+  resolver = g_resolver_get_default ();
+  data = g_slice_new0 (PendingStunServer);
+
   DEBUG ("Resolving %s STUN server %s:%u",
       fallback ? "fallback" : "primary", stun_server, stun_port);
+  data->factory = self;
+  g_object_add_weak_pointer (G_OBJECT (self), (gpointer *) &data->factory);
   data->stun_server = stun_server;
   data->stun_port = stun_port;
   data->fallback = fallback;
 
-  gibber_resolver_addrinfo (self->priv->resolver, stun_server, NULL,
-      AF_INET, SOCK_DGRAM, IPPROTO_UDP, 0,
-      stun_server_resolved_cb, data, pending_stun_server_free,
-      G_OBJECT (self));
+  data->cancellable = g_cancellable_new ();
+  g_object_weak_ref (G_OBJECT (self), (GWeakNotify)g_cancellable_cancel,
+      data->cancellable);
+
+  g_resolver_lookup_by_name_async (resolver, stun_server,
+      data->cancellable, stun_server_resolved_cb, data);
 }
 
 
@@ -246,8 +255,8 @@ got_jingle_info_stanza (GabbleJingleFactory *fac,
   LmMessageSubType sub_type;
   LmMessageNode *query_node, *node;
 
-  query_node = lm_message_node_get_child_with_namespace (message->node,
-      "query", NS_GOOGLE_JINGLE_INFO);
+  query_node = lm_message_node_get_child_with_namespace (
+      wocky_stanza_get_top_node (message), "query", NS_GOOGLE_JINGLE_INFO);
 
   if (query_node == NULL)
     return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
@@ -258,7 +267,8 @@ got_jingle_info_stanza (GabbleJingleFactory *fac,
     {
       GabbleXmppError xmpp_error = XMPP_ERROR_UNDEFINED_CONDITION;
 
-      node = lm_message_node_get_child (message->node, "error");
+      node = lm_message_node_get_child (wocky_stanza_get_top_node (message),
+          "error");
       if (node != NULL)
         {
           xmpp_error = gabble_xmpp_error_from_node (node, NULL);
@@ -443,7 +453,8 @@ jingle_info_send_request (GabbleJingleFactory *fac)
   msg = lm_message_new_with_sub_type (jid, LM_MESSAGE_TYPE_IQ,
       LM_MESSAGE_SUB_TYPE_GET);
 
-  node = lm_message_node_add_child (msg->node, "query", NULL);
+  node = lm_message_node_add_child (
+      wocky_stanza_get_top_node (msg), "query", NULL);
   lm_message_node_set_attribute (node, "xmlns", NS_GOOGLE_JINGLE_INFO);
 
   if (!_gabble_connection_send_with_reply (priv->conn, msg,
@@ -547,9 +558,10 @@ gabble_jingle_factory_constructor (GType type,
   priv = self->priv;
 
   /* FIXME: why was this in _constructed in media factory? */
-  g_signal_connect (priv->conn,
-      "status-changed", (GCallback) connection_status_changed_cb, self);
+  gabble_signal_connect_weak (priv->conn, "status-changed",
+      (GCallback) connection_status_changed_cb, G_OBJECT (self));
 
+  jingle_share_register (self);
   jingle_media_rtp_register (self);
   jingle_transport_google_register (self);
   jingle_transport_rawudp_register (self);
@@ -671,31 +683,37 @@ connection_status_changed_cb (GabbleConnection *conn,
     }
 }
 
+/* The 'session' map is keyed by:
+ * "<peer's handle>\n<peer's jid>\n<session id>"
+ */
+#define SESSION_MAP_KEY_FORMAT "%u\n%s\n%s"
+
 static gchar *
 make_session_map_key (TpHandle peer,
-    const gchar *resource,
+    const gchar *jid,
     const gchar *sid)
 {
-  return g_strdup_printf (SESSION_MAP_KEY_FORMAT, peer, resource, sid);
+  return g_strdup_printf (SESSION_MAP_KEY_FORMAT, peer, jid, sid);
 }
 
 static gchar *
 get_unique_sid_for (GabbleJingleFactory *factory,
     TpHandle peer,
-    const gchar *resource,
+    const gchar *jid,
     gchar **key)
 {
   guint32 val;
   gchar *sid = NULL;
-  gchar *key_;
+  gchar *key_ = NULL;
 
   do
     {
       val = g_random_int_range (1000000, G_MAXINT);
 
       g_free (sid);
+      g_free (key_);
       sid = g_strdup_printf ("%u", val);
-      key_ = make_session_map_key (peer, resource, sid);
+      key_ = make_session_map_key (peer, jid, sid);
     }
   while (g_hash_table_lookup (factory->priv->sessions, key_) != NULL);
 
@@ -715,22 +733,10 @@ ensure_session (GabbleJingleFactory *self,
   GabbleJingleFactoryPrivate *priv = self->priv;
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
       (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
-  const gchar *resource;
   gchar *key;
   GabbleJingleSession *sess;
   TpHandle peer;
 
-  resource = strchr (from, '/');
-
-  if (resource == NULL || *resource == '\0')
-    {
-      g_set_error (error, GABBLE_XMPP_ERROR,
-          XMPP_ERROR_BAD_REQUEST, "IQ sender '%s' has no resource", from);
-      return NULL;
-    }
-
-  resource++;
-
   peer = tp_handle_ensure (contact_repo, from, NULL, error);
 
   if (peer == 0)
@@ -739,7 +745,8 @@ ensure_session (GabbleJingleFactory *self,
       return NULL;
     }
 
-  key = make_session_map_key (peer, resource, sid);
+  /* If we can ensure the handle, we can decode the jid */
+  key = make_session_map_key (peer, from, sid);
   sess = g_hash_table_lookup (priv->sessions, key);
   g_free (key);
 
@@ -747,7 +754,7 @@ ensure_session (GabbleJingleFactory *self,
     {
       if (action == JINGLE_ACTION_SESSION_INITIATE)
         {
-          sess = create_session (self, sid, peer, resource, FALSE);
+          sess = create_session (self, sid, peer, from, FALSE);
           g_object_set (sess, "dialect", dialect, NULL);
           *new_session = TRUE;
         }
@@ -832,7 +839,7 @@ static GabbleJingleSession *
 create_session (GabbleJingleFactory *fac,
     const gchar *sid,
     TpHandle peer,
-    const gchar *peer_resource,
+    const gchar *jid,
     gboolean local_hold)
 {
   GabbleJingleFactoryPrivate *priv = fac->priv;
@@ -840,19 +847,18 @@ create_session (GabbleJingleFactory *fac,
   gboolean local_initiator;
   gchar *sid_, *key;
 
-  g_assert (peer != 0);
-  g_assert (peer_resource != NULL);
+  g_assert (jid != NULL);
 
   if (sid != NULL)
     {
-      key = make_session_map_key (peer, peer_resource, sid);
+      key = make_session_map_key (peer, jid, sid);
       sid_ = g_strdup (sid);
 
       local_initiator = FALSE;
     }
   else
     {
-      sid_ = get_unique_sid_for (fac, peer, peer_resource, &key);
+      sid_ = get_unique_sid_for (fac, peer, jid, &key);
 
       local_initiator = TRUE;
     }
@@ -861,14 +867,15 @@ create_session (GabbleJingleFactory *fac,
    * get_unique_sid_for should have ensured the key is fresh. */
   g_assert (NULL == g_hash_table_lookup (priv->sessions, key));
 
-  sess = gabble_jingle_session_new (priv->conn, sid_, local_initiator, peer,
-      peer_resource, local_hold);
-  g_signal_connect (sess, "terminated", (GCallback) session_terminated_cb, fac);
+  sess = gabble_jingle_session_new (priv->conn, sid_, local_initiator, jid,
+      local_hold);
+  g_signal_connect (sess, "terminated",
+    (GCallback) session_terminated_cb, fac);
 
   /* Takes ownership of key */
   g_hash_table_insert (priv->sessions, key, sess);
 
-  DEBUG ("new session (%u, %s, %s) @ %p", peer, peer_resource, sid_, sess);
+  DEBUG ("new session (%s, %s) @ %p", jid, sid_, sess);
 
   g_free (sid_);
 
@@ -878,10 +885,10 @@ create_session (GabbleJingleFactory *fac,
 GabbleJingleSession *
 gabble_jingle_factory_create_session (GabbleJingleFactory *fac,
     TpHandle peer,
-    const gchar *peer_resource,
+    const gchar *jid,
     gboolean local_hold)
 {
-  return create_session (fac, NULL, peer, peer_resource, local_hold);
+  return create_session (fac, NULL, peer, jid, local_hold);
 }
 
 void
@@ -931,7 +938,7 @@ session_terminated_cb (GabbleJingleSession *session,
                        GabbleJingleFactory *factory)
 {
   gchar *key = make_session_map_key (session->peer,
-      gabble_jingle_session_get_peer_resource (session),
+      gabble_jingle_session_get_peer_jid (session),
       gabble_jingle_session_get_sid (session));
 
   DEBUG ("removing terminated session with key %s", key);
