@@ -74,9 +74,18 @@ void jingle_media_rtp_register (GabbleJingleFactory *factory);
 gboolean jingle_media_rtp_set_local_codecs (GabbleJingleMediaRtp *self,
     GList *codecs, gboolean ready, GError **error);
 GList *gabble_jingle_media_rtp_get_remote_codecs (GabbleJingleMediaRtp *self);
+GList *gabble_jingle_media_rtp_get_local_codecs (GabbleJingleMediaRtp *self);
 
 JingleCodec * jingle_media_rtp_codec_new (guint id, const gchar *name,
     guint clockrate, guint channels, GHashTable *params);
+void jingle_media_rtp_codec_free (JingleCodec *p);
+void jingle_media_rtp_free_codecs (GList *codecs);
+GList * jingle_media_rtp_copy_codecs (GList *codecs);
+
+gboolean jingle_media_rtp_compare_codecs (GList *old,
+    GList *new,
+    GList **changed,
+    GError **e);
 
 #endif /* __JINGLE_MEDIA_RTP_H__ */
 
