@@ -25,10 +25,13 @@
 #include <glib.h>
 
 #include <loudmouth/loudmouth.h>
+#include <telepathy-glib/handle-repo-dynamic.h>
 
 #define DEBUG_FLAG GABBLE_DEBUG_MEDIA
 
+#include "capabilities.h"
 #include "connection.h"
+#include "conn-presence.h"
 #include "debug.h"
 #include "gabble-signals-marshal.h"
 #include "jingle-content.h"
@@ -59,8 +62,8 @@ enum
 {
   PROP_CONNECTION = 1,
   PROP_SESSION_ID,
+  PROP_PEER_JID,
   PROP_PEER,
-  PROP_PEER_RESOURCE,
   PROP_LOCAL_INITIATOR,
   PROP_STATE,
   PROP_DIALECT,
@@ -105,7 +108,7 @@ typedef struct {
 } JingleStateActions;
 
 /* gcc should be able to figure this out from the table below, but.. */
-#define MAX_ACTIONS_PER_STATE 11
+#define MAX_ACTIONS_PER_STATE 12
 
 /* NB: JINGLE_ACTION_UNKNOWN is used as a terminator here. */
 static JingleAction allowed_actions[MAX_JINGLE_STATES][MAX_ACTIONS_PER_STATE] = {
@@ -115,24 +118,27 @@ static JingleAction allowed_actions[MAX_JINGLE_STATES][MAX_ACTIONS_PER_STATE] =
   { JINGLE_ACTION_SESSION_TERMINATE, JINGLE_ACTION_SESSION_ACCEPT,
     JINGLE_ACTION_TRANSPORT_ACCEPT, /* required for GTalk4 */
     JINGLE_ACTION_DESCRIPTION_INFO, JINGLE_ACTION_SESSION_INFO,
-    JINGLE_ACTION_TRANSPORT_INFO, JINGLE_ACTION_UNKNOWN },
+    JINGLE_ACTION_TRANSPORT_INFO, JINGLE_ACTION_INFO,
+    JINGLE_ACTION_UNKNOWN },
   /* JS_STATE_PENDING_INITIATED */
   { JINGLE_ACTION_SESSION_ACCEPT, JINGLE_ACTION_SESSION_TERMINATE,
     JINGLE_ACTION_TRANSPORT_INFO, JINGLE_ACTION_CONTENT_REJECT,
     JINGLE_ACTION_CONTENT_MODIFY, JINGLE_ACTION_CONTENT_ACCEPT,
     JINGLE_ACTION_CONTENT_REMOVE,  JINGLE_ACTION_DESCRIPTION_INFO,
     JINGLE_ACTION_TRANSPORT_ACCEPT, JINGLE_ACTION_SESSION_INFO,
+    JINGLE_ACTION_INFO,
     JINGLE_ACTION_UNKNOWN },
   /* JS_STATE_PENDING_ACCEPT_SENT */
   { JINGLE_ACTION_TRANSPORT_INFO, JINGLE_ACTION_DESCRIPTION_INFO,
     JINGLE_ACTION_SESSION_TERMINATE, JINGLE_ACTION_SESSION_INFO,
+    JINGLE_ACTION_INFO,
     JINGLE_ACTION_UNKNOWN },
   /* JS_STATE_ACTIVE */
   { JINGLE_ACTION_CONTENT_MODIFY, JINGLE_ACTION_CONTENT_ADD,
     JINGLE_ACTION_CONTENT_REMOVE, JINGLE_ACTION_CONTENT_REPLACE,
     JINGLE_ACTION_CONTENT_ACCEPT, JINGLE_ACTION_CONTENT_REJECT,
     JINGLE_ACTION_SESSION_INFO, JINGLE_ACTION_TRANSPORT_INFO,
-    JINGLE_ACTION_DESCRIPTION_INFO,
+    JINGLE_ACTION_DESCRIPTION_INFO, JINGLE_ACTION_INFO,
     JINGLE_ACTION_SESSION_TERMINATE, JINGLE_ACTION_UNKNOWN },
   /* JS_STATE_ENDED */
   { JINGLE_ACTION_UNKNOWN }
@@ -155,13 +161,15 @@ gabble_jingle_session_defines_action (GabbleJingleSession *sess,
         return (a != JINGLE_ACTION_DESCRIPTION_INFO &&
             a != JINGLE_ACTION_SESSION_INFO);
       case JINGLE_DIALECT_GTALK4:
-        if (a == JINGLE_ACTION_TRANSPORT_ACCEPT)
+        if (a == JINGLE_ACTION_TRANSPORT_ACCEPT ||
+            a == JINGLE_ACTION_INFO )
           return TRUE;
       case JINGLE_DIALECT_GTALK3:
         return (a == JINGLE_ACTION_SESSION_ACCEPT ||
             a == JINGLE_ACTION_SESSION_INITIATE ||
             a == JINGLE_ACTION_SESSION_TERMINATE ||
-            a == JINGLE_ACTION_TRANSPORT_INFO);
+            a == JINGLE_ACTION_TRANSPORT_INFO ||
+            a == JINGLE_ACTION_INFO);
       default:
         return FALSE;
     }
@@ -177,6 +185,8 @@ gabble_jingle_session_init (GabbleJingleSession *obj)
          GabbleJingleSessionPrivate);
   obj->priv = priv;
 
+  DEBUG ("Initializing the jingle session %p", obj);
+
   priv->initiator_contents = g_hash_table_new_full (g_str_hash, g_str_equal,
       g_free, g_object_unref);
   priv->responder_contents = g_hash_table_new_full (g_str_hash, g_str_equal,
@@ -252,9 +262,6 @@ gabble_jingle_session_get_property (GObject *object,
     case PROP_PEER:
       g_value_set_uint (value, sess->peer);
       break;
-    case PROP_PEER_RESOURCE:
-      g_value_set_string (value, priv->peer_resource);
-      break;
     case PROP_STATE:
       g_value_set_uint (value, priv->state);
       break;
@@ -300,13 +307,8 @@ gabble_jingle_session_set_property (GObject *object,
     case PROP_DIALECT:
       priv->dialect = g_value_get_uint (value);
       break;
-    case PROP_PEER:
-      sess->peer = g_value_get_uint (value);
-      /* priv->conn might not yet be set, so we'll ref this in constructed */
-      break;
-    case PROP_PEER_RESOURCE:
-      g_free (priv->peer_resource);
-      priv->peer_resource = g_value_dup_string (value);
+    case PROP_PEER_JID:
+      priv->peer_jid = g_value_dup_string (value);
       break;
     case PROP_LOCAL_HOLD:
       {
@@ -347,41 +349,42 @@ gabble_jingle_session_constructed (GObject *object)
     chain_up (object);
 
   g_assert (priv->conn != NULL);
-  g_assert (self->peer != 0);
-  g_assert (priv->peer_resource != NULL);
+  g_assert (priv->peer_jid != NULL);
   g_assert (priv->sid != NULL);
 
-  tp_handle_ref (contact_repo, self->peer);
-
-  /* It's a tad silly that, for incoming calls, the caller of
-   * gabble_jingle_session_new() has just deconstructed the jid into a handle
-   * and a resource, only for us to stitch it back together here. Perhaps there
-   * should be two variants: one taking handle + resource, and another taking a
-   * full JID; the other fields could be filled in here.
-   */
-  priv->peer_jid = g_strdup_printf ("%s/%s",
-      tp_handle_inspect (contact_repo, self->peer), priv->peer_resource);
+  self->peer = tp_handle_ensure (contact_repo, priv->peer_jid, NULL, NULL);
 
   if (priv->local_initiator)
     priv->initiator = gabble_connection_get_full_jid (priv->conn);
   else
     priv->initiator = g_strdup (priv->peer_jid);
+
+  /* FIXME Gabble resource part handling falls apart when it comes to mucs, so
+   * jump though some hoops here */
+  if (tp_dynamic_handle_repo_lookup_exact (contact_repo, priv->peer_jid) == 0)
+    {
+      /* The peer jid isn't exactly what is in the contact repo so it will have
+       * a resource */
+      if (gabble_decode_jid (priv->peer_jid, NULL, NULL,
+          &priv->peer_resource))
+        {
+          /* fake for gcc */;
+        }
+    }
 }
 
 GabbleJingleSession *
 gabble_jingle_session_new (GabbleConnection *connection,
                            const gchar *session_id,
                            gboolean local_initiator,
-                           TpHandle peer,
-                           const gchar *peer_resource,
+                           const gchar *jid,
                            gboolean local_hold)
 {
   return g_object_new (GABBLE_TYPE_JINGLE_SESSION,
       "session-id", session_id,
       "connection", connection,
       "local-initiator", local_initiator,
-      "peer", peer,
-      "peer-resource", peer_resource,
+      "peer-jid", jid,
       "local-hold", local_hold,
       NULL);
 }
@@ -416,19 +419,20 @@ gabble_jingle_session_class_init (GabbleJingleSessionClass *cls)
       "Specifies if local end initiated the session.",
       TRUE,
       G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
-  g_object_class_install_property (object_class, PROP_LOCAL_INITIATOR, param_spec);
+  g_object_class_install_property (object_class, PROP_LOCAL_INITIATOR,
+      param_spec);
 
   param_spec = g_param_spec_uint ("peer", "Session peer",
       "The TpHandle representing the other party in the session.",
       0, G_MAXUINT32, 0,
-      G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_PEER, param_spec);
 
-  param_spec = g_param_spec_string ("peer-resource", "Session peer's resource",
-      "The resource of the contact with whom this session communicates.",
+  param_spec = g_param_spec_string ("peer-jid", "Session peer's jid",
+      "The full jid of the contact with whom this session communicates",
       NULL,
       G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
-  g_object_class_install_property (object_class, PROP_PEER_RESOURCE,
+  g_object_class_install_property (object_class, PROP_PEER_JID,
       param_spec);
 
   param_spec = g_param_spec_uint ("state", "Session state",
@@ -520,6 +524,8 @@ parse_action (const gchar *txt)
       return JINGLE_ACTION_TRANSPORT_ACCEPT;
   else if (!tp_strdiff (txt, "description-info"))
       return JINGLE_ACTION_DESCRIPTION_INFO;
+  else if (!tp_strdiff (txt, "info"))
+      return JINGLE_ACTION_INFO;
 
   return JINGLE_ACTION_UNKNOWN;
 }
@@ -560,6 +566,8 @@ produce_action (JingleAction action, JingleDialect dialect)
       return "transport-accept";
     case JINGLE_ACTION_DESCRIPTION_INFO:
       return "description-info";
+    case JINGLE_ACTION_INFO:
+      return "info";
     default:
       /* only reached if g_return_val_if_fail is disabled */
       DEBUG ("unknown action %u", action);
@@ -630,8 +638,10 @@ lookup_content (GabbleJingleSession *sess,
           priv->conn->presence_cache, sess->peer);
 
       if (creator == NULL && presence != NULL &&
+          priv->peer_resource != NULL &&
           gabble_presence_resource_has_caps (presence, priv->peer_resource,
-              PRESENCE_CAP_JINGLE_OMITS_CONTENT_CREATOR))
+              gabble_capability_set_predicate_has,
+              QUIRK_OMITS_CONTENT_CREATORS))
         {
           DEBUG ("working around missing 'creator' attribute");
 
@@ -757,7 +767,8 @@ create_content (GabbleJingleSession *sess, GType content_type,
   GabbleJingleContent *c;
   GHashTable *contents;
 
-  DEBUG ("session creating new content type, conn == %p, jf == %p", priv->conn, priv->conn->jingle_factory);
+  DEBUG ("session creating new content name %s, type %d, conn %p, jf %p",
+    name, type, priv->conn, priv->conn->jingle_factory);
 
   /* FIXME: media-type is introduced by GabbleJingleMediaRTP, not by the
    * superclass, so this call is unsafe in the general case */
@@ -1330,6 +1341,7 @@ on_transport_info (GabbleJingleSession *sess, LmMessageNode *node,
   if (JINGLE_IS_GOOGLE_DIALECT (priv->dialect))
     {
       GHashTableIter iter;
+      gpointer value;
 
       if (priv->dialect == JINGLE_DIALECT_GTALK4)
         {
@@ -1361,8 +1373,9 @@ on_transport_info (GabbleJingleSession *sess, LmMessageNode *node,
         }
 
         g_hash_table_iter_init (&iter, priv->initiator_contents);
-        while (g_hash_table_iter_next (&iter, NULL, (gpointer) &c))
+        while (g_hash_table_iter_next (&iter, NULL, &value))
           {
+            c = value;
             gabble_jingle_content_parse_transport_info (c, node, error);
             if (error != NULL && *error != NULL)
               break;
@@ -1399,6 +1412,26 @@ on_description_info (GabbleJingleSession *sess, LmMessageNode *node,
   _foreach_content (sess, node, TRUE, _each_description_info, error);
 }
 
+static void
+on_info (GabbleJingleSession *sess, LmMessageNode *node,
+    GError **error)
+{
+  GabbleJingleSessionPrivate *priv = sess->priv;
+  GabbleJingleContent *c = NULL;
+
+  DEBUG ("received info ");
+  if (JINGLE_IS_GOOGLE_DIALECT (priv->dialect))
+    {
+      GHashTableIter iter;
+      g_hash_table_iter_init (&iter, priv->initiator_contents);
+      while (g_hash_table_iter_next (&iter, NULL, (gpointer) &c))
+        {
+          gabble_jingle_content_parse_info (c, node, error);
+          if (error != NULL && *error != NULL)
+            break;
+        }
+    }
+}
 
 static HandlerFunc handlers[] = {
   NULL, /* for unknown action */
@@ -1414,7 +1447,8 @@ static HandlerFunc handlers[] = {
   on_session_terminate, /* jingle_on_session_terminate */
   on_transport_info, /* jingle_on_transport_info */
   on_transport_accept,
-  on_description_info
+  on_description_info,
+  on_info
 };
 
 static void
@@ -1810,6 +1844,8 @@ static void
 try_session_initiate_or_accept (GabbleJingleSession *sess)
 {
   GabbleJingleSessionPrivate *priv = sess->priv;
+  TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
+      (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
   LmMessage *msg;
   LmMessageNode *sess_node;
   gboolean contents_ready = TRUE;
@@ -1817,6 +1853,8 @@ try_session_initiate_or_accept (GabbleJingleSession *sess)
   JingleSessionState new_state;
   JingleReplyHandler handler;
 
+  DEBUG ("Trying initiate or accept");
+
   /* If there are no contents yet, we shouldn't have been called at all. */
   g_assert (g_hash_table_size (priv->initiator_contents) +
       g_hash_table_size (priv->responder_contents) > 0);
@@ -1829,6 +1867,18 @@ try_session_initiate_or_accept (GabbleJingleSession *sess)
           return;
         }
 
+      if (!priv->locally_accepted)
+        {
+          DEBUG ("session not locally accepted yet, not initiating");
+          return;
+        }
+
+      /* send directed presence (including our own caps, avatar etc.) to
+       * the peer, if we aren't already visible to them */
+      if (!conn_presence_visible_to (priv->conn, sess->peer))
+        conn_presence_signal_own_presence (priv->conn,
+            tp_handle_inspect (contact_repo, sess->peer), NULL);
+
       action = JINGLE_ACTION_SESSION_INITIATE;
       new_state = JS_STATE_PENDING_INITIATE_SENT;
       handler = _on_initiate_reply;
@@ -1854,14 +1904,20 @@ try_session_initiate_or_accept (GabbleJingleSession *sess)
 
   _map_initial_contents (sess, _check_content_ready, &contents_ready);
 
+  DEBUG ("Contents are ready: %s", contents_ready ? "yes" : "no");
+
   if (!contents_ready)
+    {
+      DEBUG ("Contents not yet ready, not initiating/accepting now..");
       return;
+    }
 
   msg = gabble_jingle_session_new_message (sess, action, &sess_node);
 
   if (priv->dialect == JINGLE_DIALECT_GTALK3)
     {
       gboolean has_video = FALSE;
+      gboolean has_audio = FALSE;
       GHashTableIter iter;
       gpointer value;
 
@@ -1875,19 +1931,25 @@ try_session_initiate_or_accept (GabbleJingleSession *sess)
           if (type == JINGLE_MEDIA_TYPE_VIDEO)
             {
               has_video = TRUE;
-              break;
+            }
+          else if (type == JINGLE_MEDIA_TYPE_AUDIO)
+            {
+              has_audio = TRUE;
             }
         }
 
-      sess_node = lm_message_node_add_child (sess_node, "description",
-        NULL);
+      if (has_video || has_audio)
+        {
+          sess_node = lm_message_node_add_child (sess_node, "description",
+              NULL);
 
-      if (has_video)
-        lm_message_node_set_attribute (sess_node, "xmlns",
-          NS_GOOGLE_SESSION_VIDEO);
-      else
-        lm_message_node_set_attribute (sess_node, "xmlns",
-          NS_GOOGLE_SESSION_PHONE);
+          if (has_video)
+            lm_message_node_set_attribute (sess_node, "xmlns",
+                NS_GOOGLE_SESSION_VIDEO);
+          else
+            lm_message_node_set_attribute (sess_node, "xmlns",
+                NS_GOOGLE_SESSION_PHONE);
+        }
     }
 
 
@@ -2012,7 +2074,7 @@ gabble_jingle_session_terminate (GabbleJingleSession *sess,
 
           lm_message_node_add_child (r, reason_elt, NULL);
 
-          if (text != NULL && *text != '\0')
+          if (!CHECK_STR_EMPTY(text))
             lm_message_node_add_child (r, "text", text);
         }
 
@@ -2106,24 +2168,31 @@ gabble_jingle_session_remove_content (GabbleJingleSession *sess,
 }
 
 GabbleJingleContent *
-gabble_jingle_session_add_content (GabbleJingleSession *sess, JingleMediaType mtype,
-    const gchar *content_ns, const gchar *transport_ns)
+gabble_jingle_session_add_content (GabbleJingleSession *sess,
+    JingleMediaType mtype,
+    const gchar *name,
+    const gchar *content_ns,
+    const gchar *transport_ns)
 {
   GabbleJingleSessionPrivate *priv = sess->priv;
   GabbleJingleContent *c;
   GType content_type;
-  gchar *name = NULL;
   GHashTable *contents = priv->local_initiator ? priv->initiator_contents
       : priv->responder_contents;
   guint id = g_hash_table_size (contents) + 1;
+  gchar *cname = NULL;
 
-  do
+  if (name == NULL || *name == '\0')
+    name = (mtype == JINGLE_MEDIA_TYPE_AUDIO ?  "Audio" : "Video");
+
+  cname = g_strdup (name);
+
+  while (g_hash_table_lookup (priv->initiator_contents, cname) != NULL
+      || g_hash_table_lookup (priv->responder_contents, cname) != NULL)
     {
-      g_free (name);
-      name = g_strdup_printf ("stream%d", id++);
+      g_free (cname);
+      cname = g_strdup_printf ("%s_%d", name, id++);
     }
-  while (g_hash_table_lookup (priv->initiator_contents, name) != NULL
-      && g_hash_table_lookup (priv->responder_contents, name) != NULL);
 
   content_type = gabble_jingle_factory_lookup_content_type (
       priv->conn->jingle_factory, content_ns);
@@ -2131,12 +2200,12 @@ gabble_jingle_session_add_content (GabbleJingleSession *sess, JingleMediaType mt
   g_assert (content_type != 0);
 
   c = create_content (sess, content_type, mtype,
-      content_ns, transport_ns, name, NULL, NULL);
+      content_ns, transport_ns, cname, NULL, NULL);
 
   /* The new content better have ended up in the set we thought it would... */
-  g_assert (g_hash_table_lookup (contents, name) != NULL);
+  g_assert (g_hash_table_lookup (contents, cname) != NULL);
 
-  g_free (name);
+  g_free (cname);
 
   return c;
 }
@@ -2191,6 +2260,12 @@ gabble_jingle_session_get_peer_resource (GabbleJingleSession *sess)
   return sess->priv->peer_resource;
 }
 
+const gchar *
+gabble_jingle_session_get_initiator (GabbleJingleSession *sess)
+{
+  return sess->priv->initiator;
+}
+
 const gchar *
 gabble_jingle_session_get_sid (GabbleJingleSession *sess)
 {
@@ -2281,3 +2356,8 @@ gabble_jingle_session_get_dialect (GabbleJingleSession *sess)
   return sess->priv->dialect;
 }
 
+const gchar *
+gabble_jingle_session_get_peer_jid (GabbleJingleSession *sess)
+{
+  return sess->priv->peer_jid;
+}
