@@ -82,8 +82,7 @@ struct _GabbleJingleSession {
 GabbleJingleSession *gabble_jingle_session_new (GabbleConnection *connection,
     const gchar *session_id,
     gboolean local_initiator,
-    TpHandle peer,
-    const gchar *peer_resource,
+    const gchar *jid,
     gboolean local_hold);
 
 const gchar * gabble_jingle_session_detect (LmMessage *message,
@@ -102,13 +101,18 @@ void gabble_jingle_session_remove_content (GabbleJingleSession *sess,
     GabbleJingleContent *c);
 
 GabbleJingleContent *
-gabble_jingle_session_add_content (GabbleJingleSession *sess, JingleMediaType mtype,
-    const gchar *content_ns, const gchar *transport_ns);
+gabble_jingle_session_add_content (GabbleJingleSession *sess,
+    JingleMediaType mtype,
+    const char *name,
+    const gchar *content_ns,
+    const gchar *transport_ns);
 
 GType gabble_jingle_session_get_content_type (GabbleJingleSession *);
 GList *gabble_jingle_session_get_contents (GabbleJingleSession *sess);
 const gchar *gabble_jingle_session_get_peer_resource (
     GabbleJingleSession *sess);
+const gchar *gabble_jingle_session_get_initiator (
+    GabbleJingleSession *sess);
 const gchar *gabble_jingle_session_get_sid (GabbleJingleSession *sess);
 JingleDialect gabble_jingle_session_get_dialect (GabbleJingleSession *sess);
 
@@ -131,5 +135,7 @@ gboolean gabble_jingle_session_get_remote_ringing (GabbleJingleSession *sess);
 gboolean gabble_jingle_session_defines_action (GabbleJingleSession *sess,
     JingleAction action);
 
+const gchar *gabble_jingle_session_get_peer_jid (GabbleJingleSession *sess);
+
 #endif /* __JINGLE_SESSION_H__ */
 
