@@ -23,8 +23,9 @@
 #include "media-channel.h"
 #include "media-channel-internal.h"
 
-
 #include <dbus/dbus-glib.h>
+#include <dbus/dbus-glib-lowlevel.h>
+
 #include <telepathy-glib/dbus.h>
 #include <telepathy-glib/errors.h>
 #include <telepathy-glib/exportable-channel.h>
@@ -71,8 +72,6 @@ G_DEFINE_TYPE_WITH_CODE (GabbleMediaChannel, gabble_media_channel,
       media_signalling_iface_init);
     G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_CHANNEL_TYPE_STREAMED_MEDIA,
       streamed_media_iface_init);
-    G_IMPLEMENT_INTERFACE (GABBLE_TYPE_SVC_CHANNEL_TYPE_STREAMED_MEDIA_FUTURE,
-      NULL);
     G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_PROPERTIES_INTERFACE,
       tp_properties_mixin_iface_init);
     G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_DBUS_PROPERTIES,
@@ -152,7 +151,7 @@ typedef struct {
 struct _delayed_request_streams_ctx {
   GabbleMediaChannel *chan;
   gulong caps_disco_id;
-  guint timeout_id;
+  gulong unsure_period_ended_id;
   guint contact_handle;
   GArray *types;
   GFunc succeeded_cb;
@@ -271,7 +270,7 @@ _latch_to_session (GabbleMediaChannel *chan)
 static void
 create_session (GabbleMediaChannel *chan,
     TpHandle peer,
-    const gchar *resource)
+    const gchar *jid)
 {
   GabbleMediaChannelPrivate *priv = chan->priv;
   gboolean local_hold = (priv->hold_state != TP_LOCAL_HOLD_STATE_UNHELD);
@@ -282,7 +281,7 @@ create_session (GabbleMediaChannel *chan,
 
   priv->session = g_object_ref (
       gabble_jingle_factory_create_session (priv->conn->jingle_factory,
-          peer, resource, local_hold));
+          peer, jid, local_hold));
 
   _latch_to_session (chan);
 }
@@ -436,7 +435,8 @@ gabble_media_channel_constructor (GType type, guint n_props,
       TpChannelMediaCapabilities flags = 0;
 
       if (presence != NULL)
-          _gabble_media_channel_caps_to_typeflags (presence->caps);
+        flags = _gabble_media_factory_caps_to_typeflags (
+            gabble_presence_peek_caps (presence));
 
       if (flags & TP_CHANNEL_MEDIA_CAPABILITY_IMMUTABLE_STREAMS)
         priv->immutable_streams = TRUE;
@@ -546,11 +546,9 @@ gabble_media_channel_get_property (GObject    *object,
               TP_IFACE_CHANNEL, "InitiatorID",
               TP_IFACE_CHANNEL, "Requested",
               TP_IFACE_CHANNEL, "Interfaces",
-              TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA, "ImmutableStreams",
               TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA, "InitialAudio",
               TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA, "InitialVideo",
-              GABBLE_IFACE_CHANNEL_TYPE_STREAMED_MEDIA_FUTURE, "InitialAudio",
-              GABBLE_IFACE_CHANNEL_TYPE_STREAMED_MEDIA_FUTURE, "InitialVideo",
+              TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA, "ImmutableStreams",
               NULL));
       break;
     case PROP_SESSION:
@@ -692,11 +690,6 @@ gabble_media_channel_class_init (GabbleMediaChannelClass *gabble_media_channel_c
       { "InitialVideo", "initial-video", NULL },
       { NULL }
   };
-  static TpDBusPropertiesMixinPropImpl streamed_media_future_props[] = {
-      { "InitialAudio", "initial-audio", NULL },
-      { "InitialVideo", "initial-video", NULL },
-      { NULL }
-  };
   static TpDBusPropertiesMixinIfaceImpl prop_interfaces[] = {
       { TP_IFACE_CHANNEL,
         tp_dbus_properties_mixin_getter_gobject_properties,
@@ -708,11 +701,6 @@ gabble_media_channel_class_init (GabbleMediaChannelClass *gabble_media_channel_c
         NULL,
         streamed_media_props,
       },
-      { GABBLE_IFACE_CHANNEL_TYPE_STREAMED_MEDIA_FUTURE,
-        tp_dbus_properties_mixin_getter_gobject_properties,
-        NULL,
-        streamed_media_future_props,
-      },
       { NULL }
   };
   GObjectClass *object_class = G_OBJECT_CLASS (gabble_media_channel_class);
@@ -908,6 +896,9 @@ gabble_media_channel_dispose (GObject *object)
       g_assert (d->content == NULL);
     }
 
+  g_list_free (priv->stream_creation_datas);
+  priv->stream_creation_datas = NULL;
+
   if (priv->delayed_request_streams != NULL)
     {
       g_ptr_array_foreach (priv->delayed_request_streams,
@@ -963,7 +954,14 @@ gabble_media_channel_close_async (TpSvcChannel *iface,
 {
   GabbleMediaChannel *self = GABBLE_MEDIA_CHANNEL (iface);
 
-  DEBUG ("called");
+  if (DEBUGGING)
+    {
+      gchar *caller = dbus_g_method_get_sender (context);
+
+      DEBUG ("called by %s", caller);
+      g_free (caller);
+    }
+
   gabble_media_channel_close (self);
   tp_svc_channel_return_from_close (context);
 }
@@ -1387,184 +1385,6 @@ gabble_media_channel_request_stream_direction (TpSvcChannelTypeStreamedMedia *if
     }
 }
 
-static const gchar *
-_pick_best_content_type (GabbleMediaChannel *chan, TpHandle peer,
-  const gchar *resource, JingleMediaType type)
-{
-  GabbleMediaChannelPrivate *priv = chan->priv;
-  GabblePresence *presence;
-
-  presence = gabble_presence_cache_get (priv->conn->presence_cache, peer);
-
-  if (presence == NULL)
-    {
-      DEBUG ("contact %d has no presence available", peer);
-      return NULL;
-    }
-
-  if (gabble_presence_resource_has_caps (presence, resource,
-          PRESENCE_CAP_JINGLE_RTP))
-    {
-      return NS_JINGLE_RTP;
-    }
-
-  if ((type == JINGLE_MEDIA_TYPE_VIDEO) &&
-      gabble_presence_resource_has_caps (presence, resource,
-          PRESENCE_CAP_JINGLE_DESCRIPTION_VIDEO))
-    {
-      return NS_JINGLE_DESCRIPTION_VIDEO;
-    }
-
-  if ((type == JINGLE_MEDIA_TYPE_AUDIO) &&
-      gabble_presence_resource_has_caps (presence, resource,
-          PRESENCE_CAP_JINGLE_DESCRIPTION_AUDIO))
-    {
-      return NS_JINGLE_DESCRIPTION_AUDIO;
-    }
-  if ((type == JINGLE_MEDIA_TYPE_AUDIO) &&
-      gabble_presence_resource_has_caps (presence, resource,
-          PRESENCE_CAP_GOOGLE_VOICE))
-    {
-      return NS_GOOGLE_SESSION_PHONE;
-    }
-  if ((type == JINGLE_MEDIA_TYPE_VIDEO) &&
-      gabble_presence_resource_has_caps (presence, resource,
-        PRESENCE_CAP_GOOGLE_VIDEO))
-    {
-      return NS_GOOGLE_SESSION_VIDEO;
-    }
-
-
-  return NULL;
-}
-
-
-static const gchar *
-_pick_best_resource (GabbleMediaChannel *chan,
-  TpHandle peer, gboolean want_audio, gboolean want_video,
-  const char **transport_ns, JingleDialect *dialect)
-{
-  GabbleMediaChannelPrivate *priv = chan->priv;
-  GabblePresence *presence;
-  GabblePresenceCapabilities caps;
-  const gchar *resource = NULL;
-
-  presence = gabble_presence_cache_get (priv->conn->presence_cache, peer);
-
-  if (presence == NULL)
-    {
-      DEBUG ("contact %d has no presence available", peer);
-      return NULL;
-    }
-
-  *dialect = JINGLE_DIALECT_ERROR;
-  *transport_ns = NULL;
-
-  g_return_val_if_fail (want_audio || want_video, NULL);
-
-  /* Try newest Jingle standard */
-  caps = PRESENCE_CAP_JINGLE_RTP;
-
-  if (want_audio)
-    caps |= PRESENCE_CAP_JINGLE_RTP_AUDIO;
-  if (want_video)
-    caps |= PRESENCE_CAP_JINGLE_RTP_VIDEO;
-
-  resource = gabble_presence_pick_resource_by_caps (presence, caps);
-
-  if (resource != NULL)
-    {
-      *dialect = JINGLE_DIALECT_V032;
-      goto CHOOSE_TRANSPORT;
-    }
-
-  /* Else try older Jingle draft */
-  caps = 0;
-
-  if (want_audio)
-    caps |= PRESENCE_CAP_JINGLE_DESCRIPTION_AUDIO;
-  if (want_video)
-    caps |= PRESENCE_CAP_JINGLE_DESCRIPTION_VIDEO;
-
-  resource = gabble_presence_pick_resource_by_caps (presence, caps);
-
-  if (resource != NULL)
-    {
-      *dialect = JINGLE_DIALECT_V015;
-      goto CHOOSE_TRANSPORT;
-    }
-
-  /* The Google dialects can't do video alone. */
-  if (!want_audio)
-    {
-      DEBUG ("No resource which supports video alone available");
-      return NULL;
-    }
-
-  /* Okay, let's try GTalk 0.3, possibly with video. */
-  caps = PRESENCE_CAP_GOOGLE_VOICE;
-
-  if (want_video)
-    caps |= PRESENCE_CAP_GOOGLE_VIDEO;
-
-  resource = gabble_presence_pick_resource_by_caps (presence, caps);
-
-  if (resource != NULL)
-    {
-      *dialect = JINGLE_DIALECT_GTALK3;
-      goto CHOOSE_TRANSPORT;
-    }
-
-  if (want_video)
-    {
-      DEBUG ("No resource which supports audio+video available");
-      return NULL;
-    }
-
-  /* Maybe GTalk 0.4 will save us all... ? */
-  caps = PRESENCE_CAP_GOOGLE_VOICE | PRESENCE_CAP_GOOGLE_TRANSPORT_P2P;
-  resource = gabble_presence_pick_resource_by_caps (presence, caps);
-
-  if (resource != NULL)
-    {
-      *dialect = JINGLE_DIALECT_GTALK4;
-      goto CHOOSE_TRANSPORT;
-    }
-
-  /* Nope, nothing we can do. */
-  return NULL;
-
-CHOOSE_TRANSPORT:
-  /* We prefer gtalk-p2p to ice, because it can use tcp and https relays (if
-   * available). */
-
-  if (*dialect == JINGLE_DIALECT_GTALK4 || *dialect == JINGLE_DIALECT_GTALK3)
-    {
-      /* the GTalk dialects only support google p2p as transport protocol. */
-      *transport_ns = NS_GOOGLE_TRANSPORT_P2P;
-    }
-  else if (gabble_presence_resource_has_caps (presence, resource,
-        PRESENCE_CAP_GOOGLE_TRANSPORT_P2P))
-    {
-      *transport_ns = NS_GOOGLE_TRANSPORT_P2P;
-    }
-  else if (gabble_presence_resource_has_caps (presence, resource,
-        PRESENCE_CAP_JINGLE_TRANSPORT_ICEUDP))
-    {
-      *transport_ns = NS_JINGLE_TRANSPORT_ICEUDP;
-    }
-  else if (gabble_presence_resource_has_caps (presence, resource,
-        PRESENCE_CAP_JINGLE_TRANSPORT_RAWUDP))
-    {
-      *transport_ns = NS_JINGLE_TRANSPORT_RAWUDP;
-    }
-
-  if (*transport_ns == NULL)
-      return NULL;
-
-  return resource;
-}
-
 typedef struct {
     /* number of streams requested == number of content objects */
     guint len;
@@ -1726,6 +1546,11 @@ _gabble_media_channel_request_contents (GabbleMediaChannel *chan,
     {
       peer_resource = gabble_jingle_session_get_peer_resource (priv->session);
 
+      if (peer_resource[0] != '\0')
+        DEBUG ("existing call, using peer resource %s", peer_resource);
+      else
+        DEBUG ("existing call, using bare JID");
+
       /* is a google call... we have no other option */
       if (!gabble_jingle_session_can_modify_contents (priv->session))
         {
@@ -1736,7 +1561,8 @@ _gabble_media_channel_request_contents (GabbleMediaChannel *chan,
 
       /* check if the resource supports it; FIXME - we assume only
        * one channel type (video or audio) will be added later */
-      if (NULL == _pick_best_content_type (chan, peer, peer_resource,
+      if (NULL == jingle_pick_best_content_type (priv->conn, peer,
+          peer_resource,
           want_audio ? JINGLE_MEDIA_TYPE_AUDIO : JINGLE_MEDIA_TYPE_VIDEO))
         {
           g_set_error (error, TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
@@ -1758,15 +1584,15 @@ _gabble_media_channel_request_contents (GabbleMediaChannel *chan,
   /* no existing call; we should choose a recipient and a mode */
   else
     {
+      gchar *jid;
+
       DEBUG ("picking the best resource (want audio: %u, want video: %u",
             want_audio, want_video);
 
       g_assert (priv->streams->len == 0);
 
-      peer_resource = _pick_best_resource (chan, peer, want_audio, want_video,
-          &transport_ns, &dialect);
-
-      if (peer_resource == NULL)
+      if (!jingle_pick_best_resource (priv->conn, peer,
+          want_audio, want_video, &transport_ns, &dialect, &peer_resource))
         {
           g_set_error (error, TP_ERRORS, TP_ERROR_NOT_CAPABLE,
               "member does not have the desired audio/video capabilities");
@@ -1774,9 +1600,12 @@ _gabble_media_channel_request_contents (GabbleMediaChannel *chan,
         }
 
       DEBUG ("Picking resource '%s' (transport: %s, dialect: %u)",
-          peer_resource, transport_ns, dialect);
+          peer_resource == NULL ? "(null)" : peer_resource,
+          transport_ns, dialect);
 
-      create_session (chan, peer, peer_resource);
+      jid = gabble_peer_to_jid (priv->conn, peer, peer_resource);
+      create_session (chan, peer, jid);
+      g_free (jid);
 
       g_object_set (priv->session, "dialect", dialect, NULL);
 
@@ -1811,7 +1640,8 @@ _gabble_media_channel_request_contents (GabbleMediaChannel *chan,
       GabbleJingleContent *c;
       const gchar *content_ns;
 
-      content_ns = _pick_best_content_type (chan, peer, peer_resource,
+      content_ns = jingle_pick_best_content_type (priv->conn, peer,
+          peer_resource,
           media_type == TP_MEDIA_STREAM_TYPE_AUDIO ?
             JINGLE_MEDIA_TYPE_AUDIO : JINGLE_MEDIA_TYPE_VIDEO);
 
@@ -1825,7 +1655,7 @@ _gabble_media_channel_request_contents (GabbleMediaChannel *chan,
       c = gabble_jingle_session_add_content (priv->session,
           media_type == TP_MEDIA_STREAM_TYPE_AUDIO ?
             JINGLE_MEDIA_TYPE_AUDIO : JINGLE_MEDIA_TYPE_VIDEO,
-            content_ns, transport_ns);
+            NULL, content_ns, transport_ns);
 
       /* The stream is created in "new-content" callback, and appended to
        * priv->streams. This is now guaranteed to happen asynchronously (adding
@@ -1845,8 +1675,9 @@ destroy_request (struct _delayed_request_streams_ctx *ctx,
 {
   GabbleMediaChannelPrivate *priv = ctx->chan->priv;
 
-  if (ctx->timeout_id)
-    g_source_remove (ctx->timeout_id);
+  if (ctx->unsure_period_ended_id)
+    g_signal_handler_disconnect (priv->conn->presence_cache,
+        ctx->unsure_period_ended_id);
 
   if (ctx->caps_disco_id)
     g_signal_handler_disconnect (priv->conn->presence_cache,
@@ -1887,7 +1718,6 @@ repeat_request (struct _delayed_request_streams_ctx *ctx)
   media_channel_request_streams (ctx->chan, ctx->contact_handle, ctx->types,
       ctx->succeeded_cb, ctx->failed_cb, ctx->context);
 
-  ctx->timeout_id = 0;
   ctx->context = NULL;
   destroy_and_remove_request (ctx);
   return FALSE;
@@ -1902,8 +1732,9 @@ capabilities_discovered_cb (GabblePresenceCache *cache,
   if (ctx->contact_handle != handle)
     return;
 
-  /* If there are more cache caps pending for this contact, wait for them. */
-  if (gabble_presence_cache_caps_pending (cache, handle))
+  /* If we're still unsure about this contact (most likely because there are
+   * more cache caps pending), wait for them. */
+  if (gabble_presence_cache_is_unsure (cache, handle))
     return;
 
   repeat_request (ctx);
@@ -1915,8 +1746,7 @@ delay_stream_request (GabbleMediaChannel *chan,
                       const GArray *types,
                       GFunc succeeded_cb,
                       GFunc failed_cb,
-                      gpointer context,
-                      gboolean disco_in_progress)
+                      gpointer context)
 {
   GabbleMediaChannelPrivate *priv = chan->priv;
   struct _delayed_request_streams_ctx *ctx =
@@ -1930,19 +1760,12 @@ delay_stream_request (GabbleMediaChannel *chan,
   ctx->types = g_array_sized_new (FALSE, FALSE, sizeof (guint), types->len);
   g_array_append_vals (ctx->types, types->data, types->len);
 
-  if (disco_in_progress)
-    {
-      ctx->caps_disco_id = g_signal_connect (priv->conn->presence_cache,
-          "capabilities-discovered", G_CALLBACK (capabilities_discovered_cb),
-          ctx);
-      ctx->timeout_id = 0;
-    }
-  else
-    {
-      ctx->caps_disco_id = 0;
-      ctx->timeout_id = g_timeout_add_seconds (5,
-          (GSourceFunc) repeat_request, ctx);
-    }
+  ctx->caps_disco_id = g_signal_connect (priv->conn->presence_cache,
+      "capabilities-discovered", G_CALLBACK (capabilities_discovered_cb),
+      ctx);
+  ctx->unsure_period_ended_id = g_signal_connect_swapped (
+      priv->conn->presence_cache, "unsure-period-ended",
+      G_CALLBACK (repeat_request), ctx);
 
   g_ptr_array_add (priv->delayed_request_streams, ctx);
 }
@@ -1980,7 +1803,7 @@ media_channel_request_streams (GabbleMediaChannel *self,
         {
           DEBUG ("Delaying RequestStreams until we get all caps from contact");
           delay_stream_request (self, contact_handle, types,
-              succeeded_cb, failed_cb, context, TRUE);
+              succeeded_cb, failed_cb, context);
           g_error_free (error);
           return;
         }
@@ -2014,6 +1837,10 @@ media_channel_request_streams (GabbleMediaChannel *self,
   priv->pending_stream_requests = g_list_prepend (priv->pending_stream_requests,
       psr);
   g_ptr_array_free (contents, TRUE);
+
+  /* signal acceptance */
+  gabble_jingle_session_accept (priv->session);
+
   return;
 
 error:
@@ -2115,46 +1942,47 @@ gabble_media_channel_request_initial_streams (GabbleMediaChannel *chan,
 static gboolean
 contact_is_media_capable (GabbleMediaChannel *chan,
     TpHandle peer,
-    gboolean *wait,
+    gboolean *wait_ret,
     GError **error)
 {
   GabbleMediaChannelPrivate *priv = chan->priv;
   GabblePresence *presence;
-  GabblePresenceCapabilities caps;
   TpBaseConnection *conn = (TpBaseConnection *) priv->conn;
   TpHandleRepoIface *contact_handles = tp_base_connection_get_handles (
       conn, TP_HANDLE_TYPE_CONTACT);
-  gboolean wait_ = FALSE;
+  gboolean wait = FALSE;
 
   presence = gabble_presence_cache_get (priv->conn->presence_cache, peer);
 
-  caps = PRESENCE_CAP_GOOGLE_VOICE | PRESENCE_CAP_GOOGLE_VOICE |
-    PRESENCE_CAP_JINGLE015 |
-    PRESENCE_CAP_JINGLE_DESCRIPTION_AUDIO |
-    PRESENCE_CAP_JINGLE_DESCRIPTION_VIDEO |
-    PRESENCE_CAP_JINGLE_RTP |
-    PRESENCE_CAP_JINGLE_DESCRIPTION_AUDIO |
-    PRESENCE_CAP_JINGLE_DESCRIPTION_VIDEO;
+  if (presence != NULL)
+    {
+      const GabbleCapabilitySet *caps = gabble_presence_peek_caps (presence);
 
-  if (presence != NULL && (presence->caps & caps) != 0)
-    return TRUE;
+      if (gabble_capability_set_has_one (caps,
+            gabble_capabilities_get_any_audio_video ()))
+        return TRUE;
+    }
 
   /* Okay, they're not capable (yet). Let's figure out whether we should wait,
    * and return an appropriate error.
    */
-  if (gabble_presence_cache_caps_pending (priv->conn->presence_cache, peer))
+  if (gabble_presence_cache_is_unsure (priv->conn->presence_cache, peer))
     {
-      DEBUG ("caps are pending for peer %u", peer);
-      wait_ = TRUE;
+      DEBUG ("presence cache is still unsure about handle %u", peer);
+      wait = TRUE;
     }
-  else if (gabble_presence_cache_is_unsure (priv->conn->presence_cache))
+  else if (!priv->tried_decloaking &&
+      gabble_presence_cache_request_decloaking (priv->conn->presence_cache,
+        peer, "media"))
     {
-      DEBUG ("presence cache is still unsure (interested in handle %u)", peer);
-      wait_ = TRUE;
+      /* only ask to decloak at most once per call */
+      priv->tried_decloaking = TRUE;
+      DEBUG ("asked handle %u to decloak, let's see what they do", peer);
+      wait = TRUE;
     }
 
-  if (wait != NULL)
-    *wait = wait_;
+  if (wait_ret != NULL)
+    *wait_ret = wait;
 
   if (presence == NULL)
     g_set_error (error, TP_ERRORS, TP_ERROR_OFFLINE,
@@ -2561,64 +2389,6 @@ stream_direction_changed_cb (GabbleMediaStream *stream,
       chan, id, direction, pending_send);
 }
 
-#define GTALK_CAPS \
-  ( PRESENCE_CAP_GOOGLE_VOICE )
-
-#define GTALK_VIDEO_CAPS \
-   ( PRESENCE_CAP_GOOGLE_VIDEO )
-
-#define JINGLE_CAPS \
-  ( PRESENCE_CAP_JINGLE015 | PRESENCE_CAP_JINGLE032 \
-  | PRESENCE_CAP_JINGLE_TRANSPORT_RAWUDP )
-
-#define JINGLE_AUDIO_CAPS \
-  ( PRESENCE_CAP_JINGLE_RTP | PRESENCE_CAP_JINGLE_RTP_AUDIO \
-  | PRESENCE_CAP_JINGLE_DESCRIPTION_AUDIO )
-
-#define JINGLE_VIDEO_CAPS \
-  ( PRESENCE_CAP_JINGLE_RTP | PRESENCE_CAP_JINGLE_RTP_VIDEO \
-  | PRESENCE_CAP_JINGLE_DESCRIPTION_VIDEO )
-
-GabblePresenceCapabilities
-_gabble_media_channel_typeflags_to_caps (TpChannelMediaCapabilities flags)
-{
-  GabblePresenceCapabilities caps = 0;
-  gboolean gtalk_p2p;
-
-  DEBUG ("adding Jingle caps (%s, %s)",
-    flags & TP_CHANNEL_MEDIA_CAPABILITY_AUDIO ? "audio" : "no audio",
-    flags & TP_CHANNEL_MEDIA_CAPABILITY_VIDEO ? "video" : "no video");
-
-  /* We speak Jingle (old and new), and can always do raw UDP */
-  caps |= JINGLE_CAPS;
-
-  if (flags & TP_CHANNEL_MEDIA_CAPABILITY_NAT_TRAVERSAL_ICE_UDP)
-    caps |= PRESENCE_CAP_JINGLE_TRANSPORT_ICEUDP;
-
-  gtalk_p2p = flags & TP_CHANNEL_MEDIA_CAPABILITY_NAT_TRAVERSAL_GTALK_P2P;
-
-  if (gtalk_p2p)
-    caps |= PRESENCE_CAP_GOOGLE_TRANSPORT_P2P;
-
-  if (flags & TP_CHANNEL_MEDIA_CAPABILITY_AUDIO)
-    {
-      caps |= JINGLE_AUDIO_CAPS;
-
-      if (gtalk_p2p)
-        caps |= GTALK_CAPS;
-    }
-
-  if (flags & TP_CHANNEL_MEDIA_CAPABILITY_VIDEO)
-    {
-      caps |= JINGLE_VIDEO_CAPS;
-
-      if (gtalk_p2p)
-        caps |= GTALK_VIDEO_CAPS;
-    }
-
-  return caps;
-}
-
 static void
 construct_stream (GabbleMediaChannel *chan,
                   GabbleJingleContent *c,
@@ -2651,24 +2421,24 @@ construct_stream (GabbleMediaChannel *chan,
   /* if any RequestStreams call was waiting for a stream to be created for
    * that content, return from it successfully */
     {
-      GList *link = priv->pending_stream_requests;
+      GList *l = priv->pending_stream_requests;
 
-      while (link != NULL)
+      while (l != NULL)
         {
-          if (pending_stream_request_maybe_satisfy (link->data,
+          if (pending_stream_request_maybe_satisfy (l->data,
                 chan, c, stream))
             {
-              GList *dead = link;
+              GList *dead = l;
 
               pending_stream_request_free (dead->data);
 
-              link = dead->next;
+              l = dead->next;
               priv->pending_stream_requests = g_list_delete_link (
                   priv->pending_stream_requests, dead);
             }
           else
             {
-              link = link->next;
+              l = l->next;
             }
         }
     }
@@ -2796,26 +2566,26 @@ content_removed_cb (GabbleJingleContent *content,
 
   if (d->self != NULL)
     {
-      GList *link = d->self->priv->pending_stream_requests;
+      GList *l = d->self->priv->pending_stream_requests;
 
       /* if any RequestStreams call was waiting for a stream to be created for
        * that content, return from it unsuccessfully */
-      while (link != NULL)
+      while (l != NULL)
         {
-          if (pending_stream_request_maybe_fail (link->data,
+          if (pending_stream_request_maybe_fail (l->data,
                 d->self, d->content))
             {
-              GList *dead = link;
+              GList *dead = l;
 
               pending_stream_request_free (dead->data);
 
-              link = dead->next;
+              l = dead->next;
               d->self->priv->pending_stream_requests = g_list_delete_link (
                   d->self->priv->pending_stream_requests, dead);
             }
           else
             {
-              link = link->next;
+              l = l->next;
             }
         }
     }
@@ -2899,48 +2669,6 @@ session_new_content_cb (GabbleJingleSession *session,
   create_stream_from_content (chan, c, FALSE);
 }
 
-TpChannelMediaCapabilities
-_gabble_media_channel_caps_to_typeflags (GabblePresenceCapabilities caps)
-{
-  TpChannelMediaCapabilities typeflags = 0;
-  GabblePresenceCapabilities any_transport =
-      PRESENCE_CAP_GOOGLE_TRANSPORT_P2P |
-      PRESENCE_CAP_JINGLE_TRANSPORT_RAWUDP |
-      PRESENCE_CAP_JINGLE_TRANSPORT_ICEUDP;
-  GabblePresenceCapabilities jingle_audio =
-      PRESENCE_CAP_JINGLE_DESCRIPTION_AUDIO |
-      PRESENCE_CAP_JINGLE_RTP_AUDIO;
-  GabblePresenceCapabilities jingle_video =
-      PRESENCE_CAP_JINGLE_DESCRIPTION_VIDEO |
-      PRESENCE_CAP_JINGLE_RTP_VIDEO;
-  gboolean just_google, one_media_type;
-
-  /* this is intentionally asymmetric to the previous function - we don't
-   * require the other end to advertise the GTalk-P2P transport capability
-   * separately because old GTalk clients didn't do that - having Google voice
-   * implied Google session and GTalk-P2P */
-
-  if ((caps & PRESENCE_CAP_GOOGLE_VOICE) ||
-      ((caps & any_transport) && (caps & jingle_audio)))
-        typeflags |= TP_CHANNEL_MEDIA_CAPABILITY_AUDIO;
-
-  if ((caps & PRESENCE_CAP_GOOGLE_VIDEO) ||
-      ((caps & any_transport) && (caps & jingle_video)))
-        typeflags |= TP_CHANNEL_MEDIA_CAPABILITY_VIDEO;
-
-  /* If this contact only supports Google Talk, or supports exactly one media
-   * type, set Immutable_Streams. */
-  just_google = (caps & (PRESENCE_CAP_GOOGLE_VOICE | PRESENCE_CAP_GOOGLE_VIDEO))
-      && !(caps & (PRESENCE_CAP_JINGLE_RTP | PRESENCE_CAP_JINGLE015));
-  one_media_type = (typeflags == TP_CHANNEL_MEDIA_CAPABILITY_AUDIO)
-      || (typeflags == TP_CHANNEL_MEDIA_CAPABILITY_VIDEO);
-
-  if (just_google || one_media_type)
-    typeflags |= TP_CHANNEL_MEDIA_CAPABILITY_IMMUTABLE_STREAMS;
-
-  return typeflags;
-}
-
 static void
 _emit_new_stream (GabbleMediaChannel *chan,
                   GabbleMediaStream *stream)
