@@ -21,21 +21,23 @@
 #ifndef __GABBLE_NAMESPACES__H__
 #define __GABBLE_NAMESPACES__H__
 
+#include "gabble/namespaces.h"
+
 #define NS_AMP                  "http://jabber.org/protocol/amp"
 #define NS_BYTESTREAMS          "http://jabber.org/protocol/bytestreams"
-#define NS_CAPS                 "http://jabber.org/protocol/caps"
 #define NS_CHAT_STATES          "http://jabber.org/protocol/chatstates"
 #define NS_DISCO_INFO           "http://jabber.org/protocol/disco#info"
 #define NS_DISCO_ITEMS          "http://jabber.org/protocol/disco#items"
 #define NS_FEATURENEG           "http://jabber.org/protocol/feature-neg"
 #define NS_FILE_TRANSFER        "http://jabber.org/protocol/si/profile/file-transfer"
-#define NS_GABBLE_CAPS          "http://telepathy.freedesktop.org/caps"
 #define NS_GOOGLE_CAPS          "http://www.google.com/xmpp/client/caps"
 #define NS_GOOGLE_FEAT_SESSION  "http://www.google.com/xmpp/protocol/session"
+#define NS_GOOGLE_FEAT_SHARE    "http://google.com/xmpp/protocol/share/v1"
 #define NS_GOOGLE_FEAT_VOICE    "http://www.google.com/xmpp/protocol/voice/v1"
 #define NS_GOOGLE_FEAT_VIDEO    "http://www.google.com/xmpp/protocol/video/v1"
 #define NS_GOOGLE_JINGLE_INFO   "google:jingleinfo"
 #define NS_GOOGLE_ROSTER        "google:roster"
+#define NS_GOOGLE_QUEUE         "google:queue"
 #define NS_IBB                  "http://jabber.org/protocol/ibb"
 
 /* Namespaces for XEP-0166 draft v0.15, the most capable Jingle dialect
@@ -68,6 +70,8 @@
 #define NS_GOOGLE_SESSION_PHONE "http://www.google.com/session/phone"
 /* Video capability in Google's Jingle dialect */
 #define NS_GOOGLE_SESSION_VIDEO "http://www.google.com/session/video"
+/* File transfer capability in Google's Jingle dialect */
+#define NS_GOOGLE_SESSION_SHARE "http://www.google.com/session/share"
 
 /* google-p2p transport */
 #define NS_GOOGLE_TRANSPORT_P2P "http://www.google.com/transport/p2p"
@@ -94,9 +98,11 @@
 #define NS_PRIVACY              "jabber:iq:privacy"
 #define NS_REGISTER             "jabber:iq:register"
 #define NS_ROSTER               "jabber:iq:roster"
+#define NS_SEARCH               "jabber:iq:search"
 #define NS_SI                   "http://jabber.org/protocol/si"
 #define NS_SI_MULTIPLE          "http://telepathy.freedesktop.org/xmpp/si-multiple"
 #define NS_TUBES                "http://telepathy.freedesktop.org/xmpp/tubes"
+#define NS_MUJI                 "http://telepathy.freedesktop.org/xmpp/muji"
 #define NS_VCARD_TEMP           "vcard-temp"
 #define NS_VCARD_TEMP_UPDATE    "vcard-temp:x:update"
 #define NS_X_DATA               "jabber:x:data"
@@ -104,6 +110,9 @@
 #define NS_X_CONFERENCE         "jabber:x:conference"
 #define NS_XMPP_STANZAS         "urn:ietf:params:xml:ns:xmpp-stanzas"
 #define NS_GEOLOC               "http://jabber.org/protocol/geoloc"
+#define NS_GOOGLE_MAIL_NOTIFY   "google:mail:notify"
+
+#define NS_TEMPPRES "urn:xmpp:temppres:0"
 
 
 #endif /* __GABBLE_NAMESPACES__H__ */
