@@ -54,7 +54,6 @@ enum
   LAST_PROPERTY
 };
 
-typedef struct _GabbleOlpcBuddyViewPrivate GabbleOlpcBuddyViewPrivate;
 struct _GabbleOlpcBuddyViewPrivate
 {
   GHashTable *properties;
@@ -69,10 +68,6 @@ G_DEFINE_TYPE_WITH_CODE (
       NULL);
     );
 
-#define GABBLE_OLPC_BUDDY_VIEW_GET_PRIVATE(obj) \
-    ((GabbleOlpcBuddyViewPrivate *) obj->priv)
-
-
 static void
 gabble_olpc_buddy_view_init (GabbleOlpcBuddyView *self)
 {
@@ -88,7 +83,7 @@ static void
 gabble_olpc_buddy_view_dispose (GObject *object)
 {
   GabbleOlpcBuddyView *self = GABBLE_OLPC_BUDDY_VIEW (object);
-  GabbleOlpcBuddyViewPrivate *priv = GABBLE_OLPC_BUDDY_VIEW_GET_PRIVATE (self);
+  GabbleOlpcBuddyViewPrivate *priv = self->priv;
 
   if (priv->dispose_has_run)
     return;
@@ -103,7 +98,7 @@ static void
 gabble_olpc_buddy_view_finalize (GObject *object)
 {
   GabbleOlpcBuddyView *self = GABBLE_OLPC_BUDDY_VIEW (object);
-  GabbleOlpcBuddyViewPrivate *priv = GABBLE_OLPC_BUDDY_VIEW_GET_PRIVATE (self);
+  GabbleOlpcBuddyViewPrivate *priv = self->priv;
 
   g_hash_table_destroy (priv->properties);
   g_free (priv->alias);
@@ -119,7 +114,7 @@ gabble_olpc_buddy_view_get_property (GObject *object,
                                GParamSpec *pspec)
 {
   GabbleOlpcBuddyView *self = GABBLE_OLPC_BUDDY_VIEW (object);
-  GabbleOlpcBuddyViewPrivate *priv = GABBLE_OLPC_BUDDY_VIEW_GET_PRIVATE (self);
+  GabbleOlpcBuddyViewPrivate *priv = self->priv;
 
   switch (property_id)
     {
@@ -163,7 +158,7 @@ gabble_olpc_buddy_view_set_property (GObject *object,
                                      GParamSpec *pspec)
 {
   GabbleOlpcBuddyView *self = GABBLE_OLPC_BUDDY_VIEW (object);
-  GabbleOlpcBuddyViewPrivate *priv = GABBLE_OLPC_BUDDY_VIEW_GET_PRIVATE (self);
+  GabbleOlpcBuddyViewPrivate *priv = self->priv;
 
   switch (property_id)
     {
@@ -193,22 +188,18 @@ gabble_olpc_buddy_view_constructor (GType type,
   GabbleOlpcBuddyViewPrivate *priv;
   DBusGConnection *bus;
   TpBaseConnection *conn;
-  TpHandleRepoIface *contact_handles;
 
   obj = G_OBJECT_CLASS (gabble_olpc_buddy_view_parent_class)->
            constructor (type, n_props, props);
 
   view = GABBLE_OLPC_VIEW (obj);
 
-  priv = GABBLE_OLPC_BUDDY_VIEW_GET_PRIVATE (GABBLE_OLPC_BUDDY_VIEW (obj));
+  priv = GABBLE_OLPC_BUDDY_VIEW (obj)->priv;
   conn = (TpBaseConnection *) view->conn;
 
   bus = tp_get_bus ();
   dbus_g_connection_register_g_object (bus, view->object_path, obj);
 
-  contact_handles = tp_base_connection_get_handles (conn,
-      TP_HANDLE_TYPE_CONTACT);
-
   if (priv->properties == NULL)
     {
       priv->properties = g_hash_table_new (g_direct_hash, g_direct_equal);
@@ -247,7 +238,8 @@ buddy_view_query_result_cb (GabbleConnection *conn,
   LmMessageNode *view_node;
   GabbleOlpcView *self = GABBLE_OLPC_VIEW (_view);
 
-  view_node = lm_message_node_get_child_with_namespace (reply_msg->node,
+  view_node = lm_message_node_get_child_with_namespace (
+      wocky_stanza_get_top_node (reply_msg),
       "view", NS_OLPC_BUDDY);
   if (view_node == NULL)
     return LM_HANDLER_RESULT_REMOVE_MESSAGE;
@@ -262,7 +254,7 @@ gabble_olpc_buddy_view_send_request (GabbleOlpcView *view,
                                      GError **error)
 {
   GabbleOlpcBuddyView *self = GABBLE_OLPC_BUDDY_VIEW (view);
-  GabbleOlpcBuddyViewPrivate *priv = GABBLE_OLPC_BUDDY_VIEW_GET_PRIVATE (self);
+  GabbleOlpcBuddyViewPrivate *priv = self->priv;
   LmMessage *query;
   LmMessageNode *buddy_node;
   gchar *max_str, *id_str;
