@@ -54,6 +54,27 @@ G_BEGIN_DECLS
 #define JABBER_PRESENCE_SHOW_DND "dnd"
 #define JABBER_PRESENCE_SHOW_XA "xa"
 
+typedef struct _GabbleCapabilityInfo GabbleCapabilityInfo;
+
+struct _GabbleCapabilityInfo
+{
+  /* struct _GabbleCapabilityInfo can be allocated before receiving the contact's
+   * caps. In this case, cap_set is NULL. */
+  GabbleCapabilitySet *cap_set;
+
+  /* array of GabbleDiscoIdentity or NULL */
+  GPtrArray *identities;
+
+  TpIntSet *guys;
+  guint trust;
+
+  /* TRUE if this cache entry is one of our own, so between caps and
+   * per_channel_manager_caps it holds the complete set of features for the
+   * node.
+   */
+  gboolean complete;
+};
+
 typedef struct _GabblePresenceCachePrivate GabblePresenceCachePrivate;
 
 struct _GabblePresenceCache {
@@ -81,34 +102,22 @@ void gabble_presence_cache_update_many (GabblePresenceCache *cache,
 void gabble_presence_cache_maybe_remove (GabblePresenceCache *cache,
     TpHandle handle);
 void gabble_presence_cache_add_bundle_caps (GabblePresenceCache *cache,
-    const gchar *node, GabblePresenceCapabilities caps);
-void gabble_presence_cache_add_own_caps (
-    GabblePresenceCache *cache,
+    const gchar *node, const gchar *ns);
+void gabble_presence_cache_add_own_caps (GabblePresenceCache *cache,
     const gchar *ver,
-    GabblePresenceCapabilities caps,
-    GHashTable *contact_caps);
-gboolean gabble_presence_cache_peek_own_caps (
+    const GabbleCapabilitySet *cap_set,
+    const GPtrArray *identities);
+const GabbleCapabilityInfo *gabble_presence_cache_peek_own_caps (
     GabblePresenceCache *cache,
-    const gchar *ver,
-    GabblePresenceCapabilities *caps,
-    GHashTable **contact_caps);
+    const gchar *ver);
 void gabble_presence_cache_really_remove (GabblePresenceCache *cache,
     TpHandle handle);
 
-/* loop on CapabilityInfo::per_channel_manager_caps and call
- * gabble_caps_channel_manager_free_capabilities */
-void gabble_presence_cache_free_cache_entry (
-    GHashTable *per_channel_manager_caps);
-
-/* loop on CapabilityInfo::per_channel_manager_caps and call
- * gabble_caps_channel_manager_copy_capabilities */
-void gabble_presence_cache_copy_cache_entry (GHashTable **out,
-    GHashTable *in);
-
-/* loop on CapabilityInfo::per_channel_manager_caps and call
- * gabble_caps_channel_manager_update_capabilities */
-void gabble_presence_cache_update_cache_entry (GHashTable *out,
-    GHashTable *in);
+LmHandlerResult
+gabble_presence_parse_presence_message (GabblePresenceCache *cache,
+    TpHandle handle,
+    const gchar *from,
+    LmMessage *message);
 
 void gabble_presence_cache_contacts_added_to_olpc_view (
     GabblePresenceCache *cache, TpHandleSet *handles);
@@ -116,10 +125,11 @@ void gabble_presence_cache_contacts_added_to_olpc_view (
 void gabble_presence_cache_contacts_removed_from_olpc_view (
     GabblePresenceCache *cache, TpHandleSet *handles);
 
-gboolean gabble_presence_cache_caps_pending (GabblePresenceCache *cache,
+gboolean gabble_presence_cache_is_unsure (GabblePresenceCache *cache,
     TpHandle handle);
 
-gboolean gabble_presence_cache_is_unsure (GabblePresenceCache *cache);
+gboolean gabble_presence_cache_request_decloaking (GabblePresenceCache *self,
+    TpHandle handle, const gchar *reason);
 
 void gabble_presence_cache_update_location (GabblePresenceCache *cache,
     TpHandle handle, GHashTable *location);
