@@ -1,8 +1,8 @@
 /*
  * roster.h - Headers for Gabble roster helper
  *
- * Copyright (C) 2006 Collabora Ltd.
- * Copyright (C) 2006 Nokia Corporation
+ * Copyright © 2006–2010 Collabora Ltd.
+ * Copyright © 2006–2010 Nokia Corporation
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -79,6 +79,7 @@ gboolean gabble_roster_handle_set_name (GabbleRoster *, TpHandle,
 gboolean gabble_roster_handle_remove (GabbleRoster *, TpHandle, GError **);
 gboolean gabble_roster_handle_add (GabbleRoster *, TpHandle, GError **);
 gboolean gabble_roster_handle_has_entry (GabbleRoster *, TpHandle);
+
 gboolean gabble_roster_handle_add_to_group (GabbleRoster *roster,
                                             TpHandle handle,
                                             TpHandle group,
@@ -88,6 +89,28 @@ gboolean gabble_roster_handle_remove_from_group (GabbleRoster *roster,
                                                  TpHandle group,
                                                  GError **error);
 
+gboolean gabble_roster_handle_subscribe (
+    GabbleRoster *roster,
+    TpHandle handle,
+    const gchar *message,
+    GError **error);
+gboolean gabble_roster_handle_unsubscribe (
+    GabbleRoster *roster,
+    TpHandle handle,
+    const gchar *message,
+    GError **error);
+
+gboolean gabble_roster_handle_subscribed (
+    GabbleRoster *roster,
+    TpHandle handle,
+    const gchar *message,
+    GError **error);
+gboolean gabble_roster_handle_unsubscribed (
+    GabbleRoster *roster,
+    TpHandle handle,
+    const gchar *message,
+    GError **error);
+
 G_END_DECLS
 
 #endif /* __ROSTER_H__ */
