@@ -1271,7 +1271,8 @@ gabble_tube_dbus_offer (GabbleTubeDBus *tube,
         }
 
       resource = gabble_presence_pick_resource_by_caps (presence,
-          PRESENCE_CAP_SI_TUBES);
+          DEVICE_AGNOSTIC,
+          gabble_capability_set_predicate_has, NS_TUBES);
 
       if (resource == NULL)
         {
@@ -1284,8 +1285,8 @@ gabble_tube_dbus_offer (GabbleTubeDBus *tube,
       full_jid = g_strdup_printf ("%s/%s", jid, resource);
       msg = gabble_bytestream_factory_make_stream_init_iq (full_jid,
           priv->stream_id, NS_TUBES);
-      si_node = lm_message_node_get_child_with_namespace (msg->node, "si",
-          NS_SI);
+      si_node = lm_message_node_get_child_with_namespace (
+          wocky_stanza_get_top_node (msg), "si", NS_SI);
       g_assert (si_node != NULL);
 
       tube_node = lm_message_node_add_child (si_node, "tube", NULL);
