@@ -22,16 +22,21 @@
 
 #include "config.h"
 #include "util.h"
+#include "gabble/disco-identity.h"
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
-#include <telepathy-glib/handle-repo-dynamic.h>
+#include <gobject/gvaluecollector.h>
 
-#ifdef HAVE_UUID
-# include <uuid.h>
-#endif
+#include <wocky/wocky-utils.h>
+#include <telepathy-glib/handle-repo-dynamic.h>
+#include <telepathy-glib/dbus.h>
+
+#include <extensions/extensions.h>
+
+#include <uuid.h>
 
 #define DEBUG_FLAG GABBLE_DEBUG_JID
 
@@ -40,6 +45,7 @@
 #include "connection.h"
 #include "debug.h"
 #include "namespaces.h"
+#include "presence-cache.h"
 
 gchar *
 sha1_hex (const gchar *bytes,
@@ -77,7 +83,6 @@ sha1_bin (const gchar *bytes,
 gchar *
 gabble_generate_id (void)
 {
-#ifdef HAVE_UUID
   /* generate random UUIDs */
   uuid_t uu;
   gchar *str;
@@ -86,15 +91,6 @@ gabble_generate_id (void)
   uuid_generate_random (uu);
   uuid_unparse_lower (uu, str);
   return str;
-#else
-  /* generate from the time, a counter, and a random integer */
-  static gulong last = 0;
-  GTimeVal tv;
-
-  g_get_current_time (&tv);
-  return g_strdup_printf ("%lx.%lx/%lx/%x", tv.tv_sec, tv.tv_usec,
-      last++, g_random_int ());
-#endif
 }
 
 static void
@@ -123,24 +119,10 @@ lm_message_node_add_own_nick (LmMessageNode *node,
   g_free (nick);
 }
 
-void
-lm_message_node_unlink (LmMessageNode *orphan,
-    LmMessageNode *parent)
-{
-  if (parent && orphan == parent->children)
-    parent->children = orphan->next;
-  if (orphan->prev)
-    orphan->prev->next = orphan->next;
-  if (orphan->next)
-    orphan->next->prev = orphan->prev;
-}
-
 void
 lm_message_node_steal_children (LmMessageNode *snatcher,
                                 LmMessageNode *mum)
 {
-  NodeIter i;
-
   g_return_if_fail (snatcher->children == NULL);
 
   if (mum->children == NULL)
@@ -148,12 +130,6 @@ lm_message_node_steal_children (LmMessageNode *snatcher,
 
   snatcher->children = mum->children;
   mum->children = NULL;
-
-  for (i = node_iter (snatcher); i; i = node_iter_next (i))
-    {
-      LmMessageNode *baby = node_iter_data (i);
-      baby->parent = snatcher;
-    }
 }
 
 /* variant of lm_message_node_get_child() which ignores node namespace
@@ -174,67 +150,16 @@ lm_message_node_get_child_any_ns (LmMessageNode *node, const gchar *name)
   return NULL;
 }
 
-static const gchar *
-find_namespace_of_prefix (LmMessageNode *node,
-    const gchar *prefix)
-{
-  gchar *attr = g_strdup_printf ("xmlns:%s", prefix);
-  const gchar *node_ns = NULL;
-
-  /* find the namespace in this node or its parents */
-  for (; (node != NULL) && (node_ns == NULL); node = node->parent)
-    {
-      node_ns = lm_message_node_get_attribute (node, attr);
-    }
-
-  g_free (attr);
-  return node_ns;
-}
-
-static const gchar *
-get_node_namespace (LmMessageNode *node,
-    gboolean check_prefix)
-{
-  const gchar *node_ns = NULL;
-  gchar *x = strchr (node->name, ':');
-
-  if (check_prefix && x != NULL)
-    {
-      gchar *prefix = g_strndup (node->name, (x - node->name));
-
-      node_ns = find_namespace_of_prefix (node, prefix);
-      g_free (prefix);
-    }
-  else
-    {
-      node_ns = lm_message_node_get_attribute (node, "xmlns");
-
-      /* Chain up to the parent to get its namespace, as child nodes have the
-       * same namespace as their parent, if not explicitly set otherwise.
-       * However, make sure we don't check the parent's prefix as that doesn't
-       * get inherited by children. */
-      if (node_ns == NULL && node->parent != NULL)
-        node_ns = get_node_namespace (node->parent, FALSE);
-    }
-
-  return node_ns;
-}
-
 const gchar *
 lm_message_node_get_namespace (LmMessageNode *node)
 {
-  return get_node_namespace (node, TRUE);
+  return wocky_node_get_ns (node);
 }
 
 const gchar *
 lm_message_node_get_name (LmMessageNode *node)
 {
-  gchar *x = strchr (node->name, ':');
-
-  if (x != NULL)
-    return x + 1;
-  else
-    return node->name;
+  return node->name;
 }
 
 gboolean
@@ -250,37 +175,20 @@ lm_message_node_get_child_with_namespace (LmMessageNode *node,
                                           const gchar *name,
                                           const gchar *ns)
 {
+  LmMessageNode *found;
   NodeIter i;
 
+  found = wocky_node_get_child_ns (node, name, ns);
+  if (found != NULL)
+    return found;
+
   for (i = node_iter (node); i; i = node_iter_next (i))
     {
-      LmMessageNode *tmp = node_iter_data (i);
-      gchar *tag = NULL;
-      gboolean found;
+      LmMessageNode *child = node_iter_data (i);
 
-      if (tp_strdiff (tmp->name, name))
-        {
-          const gchar *suffix;
-
-          suffix = strchr (tmp->name, ':');
-
-          if (suffix == NULL)
-            continue;
-          else
-            suffix++;
-
-          if (tp_strdiff (suffix, name))
-            continue;
-
-          tag = g_strndup (tmp->name, suffix - tmp->name - 1);
-        }
-
-      found = lm_message_node_has_namespace (tmp, ns, tag);
-
-      g_free (tag);
-
-      if (found)
-        return tmp;
+      found = lm_message_node_get_child_with_namespace (child, name, ns);
+      if (found != NULL)
+        return found;
     }
 
   return NULL;
@@ -395,7 +303,8 @@ lm_message_build (const gchar *to, LmMessageType type, guint spec, ...)
 
   msg = lm_message_new (to, type);
   va_start (ap, spec);
-  lm_message_node_add_build_va (msg->node, spec, ap);
+  lm_message_node_add_build_va (
+      wocky_stanza_get_top_node (msg), spec, ap);
   va_end (ap);
   return msg;
 }
@@ -415,44 +324,12 @@ lm_message_build_with_sub_type (const gchar *to, LmMessageType type,
 
   msg = lm_message_new_with_sub_type (to, type, sub_type);
   va_start (ap, spec);
-  lm_message_node_add_build_va (msg->node, spec, ap);
+  lm_message_node_add_build_va (
+      wocky_stanza_get_top_node (msg), spec, ap);
   va_end (ap);
   return msg;
 }
 
-static gboolean
-validate_jid_node (const gchar *node)
-{
-  /* See RFC 3920 §3.3. */
-  const gchar *c;
-
-  for (c = node; *c; c++)
-    if (strchr ("\"&'/:<>@", *c))
-      /* RFC 3920 §A.5 */
-      return FALSE;
-
-  return TRUE;
-}
-
-static gboolean
-validate_jid_domain (const gchar *domain)
-{
-  /* XXX: This doesn't do proper validation, it just checks the character
-   * range. In theory, we check that the domain is a well-formed IDN or
-   * an IPv4/IPv6 address literal.
-   *
-   * See RFC 3920 §3.2.
-   */
-
-  const gchar *c;
-
-  for (c = domain; *c; c++)
-    if (!g_ascii_isalnum (*c) && !strchr (":-.", *c))
-      return FALSE;
-
-  return TRUE;
-}
-
 /**
  * gabble_decode_jid
  *
@@ -479,85 +356,7 @@ gabble_decode_jid (const gchar *jid,
                    gchar **domain,
                    gchar **resource)
 {
-  char *tmp_jid, *tmp_node, *tmp_domain, *tmp_resource;
-
-  g_assert (jid != NULL);
-
-  if (node != NULL)
-    *node = NULL;
-
-  if (domain != NULL)
-    *domain = NULL;
-
-  if (resource != NULL)
-    *resource = NULL;
-
-  /* Take a local copy so we don't modify the caller's string. */
-  tmp_jid = g_strdup (jid);
-
-  /* If there's a slash in tmp_jid, split it in two and take the second part as
-   * the resource.
-   */
-  tmp_resource = strchr (tmp_jid, '/');
-
-  if (tmp_resource)
-    {
-      *tmp_resource = '\0';
-      tmp_resource++;
-    }
-  else
-    {
-      tmp_resource = NULL;
-    }
-
-  /* If there's an at sign in tmp_jid, split it in two and set tmp_node and
-   * tmp_domain appropriately. Otherwise, tmp_node is NULL and the domain is
-   * the whole string.
-   */
-  tmp_domain = strchr (tmp_jid, '@');
-
-  if (tmp_domain)
-    {
-      *tmp_domain = '\0';
-      tmp_domain++;
-      tmp_node = tmp_jid;
-    }
-  else
-    {
-      tmp_domain = tmp_jid;
-      tmp_node = NULL;
-    }
-
-  /* Domain must be non-empty and not contain invalid characters. If the node
-   * or the resource exist, they must be non-empty and the node must not
-   * contain invalid characters.
-   */
-  if (*tmp_domain == '\0' ||
-      !validate_jid_domain (tmp_domain) ||
-      (tmp_node != NULL &&
-         (*tmp_node == '\0' || !validate_jid_node (tmp_node))) ||
-      (tmp_resource != NULL && *tmp_resource == '\0'))
-    {
-      g_free (tmp_jid);
-      return FALSE;
-    }
-
-  /* the server must be stored after we find the resource, in case we
-   * truncated a resource from it */
-  if (domain != NULL)
-    *domain = g_utf8_strdown (tmp_domain, -1);
-
-  /* store the username if the user provided a pointer */
-  if (tmp_node != NULL && node != NULL)
-    *node = g_utf8_strdown (tmp_node, -1);
-
-  /* store the resource if the user provided a pointer */
-  if (tmp_resource != NULL && resource != NULL)
-    *resource = g_strdup (tmp_resource);
-
-  /* free our working copy */
-  g_free (tmp_jid);
-  return TRUE;
+  return wocky_decode_jid (jid, node, domain, resource);
 }
 
 /**
@@ -1184,38 +983,7 @@ lm_message_node_get_attribute_with_namespace (LmMessageNode *node,
     const gchar *attribute,
     const gchar *ns)
 {
-  GSList *l;
-  const gchar *result = NULL;
-
-  g_return_val_if_fail (node != NULL, NULL);
-  g_return_val_if_fail (attribute != NULL, NULL);
-  g_return_val_if_fail (ns != NULL, NULL);
-
-  for (l = node->attributes; l != NULL && result == NULL; l = g_slist_next (l))
-    {
-      /* This is NOT part of loudmouth API; it depends LM internals */
-      Attribute *attr = (Attribute *) l->data;
-      gchar **pair;
-
-      pair = g_strsplit (attr->key, ":", 2);
-
-      if (tp_strdiff (pair[1], attribute))
-        /* no prefix (pair[1] == NULL) or the local-name is not the
-         * attribute we are looking for */
-        goto next_attribute;
-
-      if (tp_strdiff (find_namespace_of_prefix (node, pair[0]), ns))
-        /* wrong namespace */
-        goto next_attribute;
-
-      result = attr->value;
-
-next_attribute:
-      g_strfreev (pair);
-      continue;
-    }
-
-  return result;
+  return wocky_node_get_attribute_ns (node, attribute, ns);
 }
 
 GPtrArray *
@@ -1229,3 +997,412 @@ gabble_g_ptr_array_copy (GPtrArray *source)
 
   return ret;
 }
+
+WockyBareContact *
+ensure_bare_contact_from_jid (GabbleConnection *conn,
+    const gchar *jid)
+{
+  WockyContactFactory *contact_factory;
+
+  contact_factory = wocky_session_get_contact_factory (conn->session);
+  return wocky_contact_factory_ensure_bare_contact (contact_factory, jid);
+}
+
+#define TWICE(x) x, x
+
+static gboolean
+jingle_pick_resource_or_bare_jid (GabblePresence *presence,
+    GabbleCapabilitySet *caps, const gchar **resource)
+{
+  const gchar *ret;
+
+  if (gabble_presence_has_resources (presence))
+    {
+      ret = gabble_presence_pick_resource_by_caps (presence,
+          PREFER_PHONES,
+          gabble_capability_set_predicate_at_least, caps);
+
+      if (resource != NULL)
+        *resource = ret;
+
+      return (ret != NULL);
+    }
+  else if (gabble_capability_set_at_least (
+        gabble_presence_peek_caps (presence), caps))
+    {
+      if (resource != NULL)
+        *resource = NULL;
+
+      return TRUE;
+    }
+  else
+    {
+      return FALSE;
+    }
+}
+
+gboolean
+jingle_pick_best_resource (GabbleConnection *conn,
+    TpHandle peer,
+    gboolean want_audio,
+    gboolean want_video,
+    const char **transport_ns,
+    JingleDialect *dialect,
+    const gchar **resource_out)
+{
+  /* We prefer gtalk-p2p to ice, because it can use tcp and https relays (if
+   * available). */
+  static const GabbleFeatureFallback transports[] = {
+        { TRUE, TWICE (NS_GOOGLE_TRANSPORT_P2P) },
+        { TRUE, TWICE (NS_JINGLE_TRANSPORT_ICEUDP) },
+        { TRUE, TWICE (NS_JINGLE_TRANSPORT_RAWUDP) },
+        { FALSE, NULL, NULL }
+  };
+  GabblePresence *presence;
+  GabbleCapabilitySet *caps;
+  const gchar *resource = NULL;
+  gboolean success = FALSE;
+
+  presence = gabble_presence_cache_get (conn->presence_cache, peer);
+
+  if (presence == NULL)
+    {
+      DEBUG ("contact %d has no presence available", peer);
+      return FALSE;
+    }
+
+  *dialect = JINGLE_DIALECT_ERROR;
+  *transport_ns = NULL;
+
+  g_return_val_if_fail (want_audio || want_video, FALSE);
+
+  /* from here on, goto FINALLY to free this, instead of returning early */
+  caps = gabble_capability_set_new ();
+
+  /* Try newest Jingle standard */
+  gabble_capability_set_add (caps, NS_JINGLE_RTP);
+
+  if (want_audio)
+    gabble_capability_set_add (caps, NS_JINGLE_RTP_AUDIO);
+  if (want_video)
+    gabble_capability_set_add (caps, NS_JINGLE_RTP_VIDEO);
+
+  if (jingle_pick_resource_or_bare_jid (presence, caps, &resource))
+    {
+      *dialect = JINGLE_DIALECT_V032;
+      goto CHOOSE_TRANSPORT;
+    }
+
+  /* Else try older Jingle draft */
+  gabble_capability_set_clear (caps);
+
+  if (want_audio)
+    gabble_capability_set_add (caps, NS_JINGLE_DESCRIPTION_AUDIO);
+  if (want_video)
+    gabble_capability_set_add (caps, NS_JINGLE_DESCRIPTION_VIDEO);
+
+  if (jingle_pick_resource_or_bare_jid (presence, caps, &resource))
+    {
+      *dialect = JINGLE_DIALECT_V015;
+      goto CHOOSE_TRANSPORT;
+    }
+
+  /* The Google dialects can't do video alone. */
+  if (!want_audio)
+    {
+      DEBUG ("No resource which supports video alone available");
+      goto FINALLY;
+    }
+
+  /* Okay, let's try GTalk 0.3, possibly with video. */
+  gabble_capability_set_clear (caps);
+  gabble_capability_set_add (caps, NS_GOOGLE_FEAT_VOICE);
+
+  if (want_video)
+    gabble_capability_set_add (caps, NS_GOOGLE_FEAT_VIDEO);
+
+  if (jingle_pick_resource_or_bare_jid (presence, caps, &resource))
+    {
+      *dialect = JINGLE_DIALECT_GTALK3;
+      goto CHOOSE_TRANSPORT;
+    }
+
+  if (want_video)
+    {
+      DEBUG ("No resource which supports audio+video available");
+      goto FINALLY;
+    }
+
+  /* Maybe GTalk 0.4 will save us all... ? */
+  gabble_capability_set_clear (caps);
+  gabble_capability_set_add (caps, NS_GOOGLE_FEAT_VOICE);
+  gabble_capability_set_add (caps, NS_GOOGLE_TRANSPORT_P2P);
+
+  if (jingle_pick_resource_or_bare_jid (presence, caps, &resource))
+    {
+      *dialect = JINGLE_DIALECT_GTALK4;
+      goto CHOOSE_TRANSPORT;
+    }
+
+  /* Nope, nothing we can do. */
+  goto FINALLY;
+
+CHOOSE_TRANSPORT:
+
+  if (resource_out != NULL)
+    *resource_out = resource;
+
+  success = TRUE;
+
+  if (*dialect == JINGLE_DIALECT_GTALK4 || *dialect == JINGLE_DIALECT_GTALK3)
+    {
+      /* the GTalk dialects only support google p2p as transport protocol. */
+      *transport_ns = NS_GOOGLE_TRANSPORT_P2P;
+    }
+  else if (resource == NULL)
+    {
+      *transport_ns = gabble_presence_pick_best_feature (presence, transports,
+          gabble_capability_set_predicate_has);
+    }
+  else
+    {
+      *transport_ns = gabble_presence_resource_pick_best_feature (presence,
+        resource, transports, gabble_capability_set_predicate_has);
+    }
+
+  if (*transport_ns == NULL)
+    success = FALSE;
+
+FINALLY:
+  gabble_capability_set_free (caps);
+  return success;
+}
+
+const gchar *
+jingle_pick_best_content_type (GabbleConnection *conn,
+  TpHandle peer,
+  const gchar *resource,
+  JingleMediaType type)
+{
+  GabblePresence *presence;
+  const GabbleFeatureFallback content_types[] = {
+      /* if $thing is supported, then use it */
+        { TRUE, TWICE (NS_JINGLE_RTP) },
+        { type == JINGLE_MEDIA_TYPE_VIDEO,
+            TWICE (NS_JINGLE_DESCRIPTION_VIDEO) },
+        { type == JINGLE_MEDIA_TYPE_AUDIO,
+            TWICE (NS_JINGLE_DESCRIPTION_AUDIO) },
+      /* odd Google ones: if $thing is supported, use $other_thing */
+        { type == JINGLE_MEDIA_TYPE_AUDIO,
+          NS_GOOGLE_FEAT_VOICE, NS_GOOGLE_SESSION_PHONE },
+        { type == JINGLE_MEDIA_TYPE_VIDEO,
+          NS_GOOGLE_FEAT_VIDEO, NS_GOOGLE_SESSION_VIDEO },
+        { FALSE, NULL, NULL }
+  };
+
+  presence = gabble_presence_cache_get (conn->presence_cache, peer);
+
+  if (presence == NULL)
+    {
+      DEBUG ("contact %d has no presence available", peer);
+      return NULL;
+    }
+
+  if (resource == NULL)
+    {
+      return gabble_presence_pick_best_feature (presence, content_types,
+          gabble_capability_set_predicate_has);
+    }
+  else
+    {
+      return gabble_presence_resource_pick_best_feature (presence, resource,
+          content_types, gabble_capability_set_predicate_has);
+    }
+}
+
+GPtrArray *
+gabble_call_candidates_to_array (GList *candidates)
+{
+  GPtrArray *arr;
+  GList *c;
+
+  arr = g_ptr_array_sized_new (g_list_length (candidates));
+
+  for (c = candidates; c != NULL; c = g_list_next (c))
+    {
+        JingleCandidate *cand = (JingleCandidate *) c->data;
+        GValueArray *a;
+        GHashTable *info;
+
+        info = tp_asv_new (
+          "Protocol", G_TYPE_UINT, cand->protocol,
+          "Type", G_TYPE_UINT, cand->type,
+          "Foundation", G_TYPE_STRING, cand->id,
+          "Priority", G_TYPE_UINT,
+            (guint) cand->preference * 65536,
+          "Username", G_TYPE_STRING, cand->username,
+          "Password", G_TYPE_STRING, cand->password,
+          NULL);
+
+         a = tp_value_array_build (4,
+            G_TYPE_UINT, cand->component,
+            G_TYPE_STRING, cand->address,
+            G_TYPE_UINT, cand->port,
+            GABBLE_HASH_TYPE_CANDIDATE_INFO, info,
+            G_TYPE_INVALID);
+
+        g_ptr_array_add (arr, a);
+  }
+
+  return arr;
+}
+
+gchar *
+gabble_peer_to_jid (GabbleConnection *conn,
+    TpHandle peer,
+    const gchar *resource)
+{
+  TpHandleRepoIface *repo = tp_base_connection_get_handles (
+    TP_BASE_CONNECTION (conn), TP_HANDLE_TYPE_CONTACT);
+  const gchar *target = tp_handle_inspect (repo, peer);
+
+  if (resource == NULL)
+    return g_strdup (target);
+
+  return g_strdup_printf ("%s/%s", target, resource);
+}
+
+GabbleDiscoIdentity *
+gabble_disco_identity_new (const gchar *category,
+    const gchar *type,
+    const gchar *lang,
+    const gchar *name)
+{
+  GabbleDiscoIdentity *ret;
+
+  g_return_val_if_fail (category != NULL, NULL);
+  g_return_val_if_fail (type != NULL, NULL);
+
+  ret = g_slice_new (GabbleDiscoIdentity);
+  ret->category = g_strdup (category);
+  ret->type = g_strdup (type);
+  ret->lang = g_strdup (lang);
+  ret->name = g_strdup (name);
+  return ret;
+}
+
+GabbleDiscoIdentity *
+gabble_disco_identity_copy (const GabbleDiscoIdentity *source)
+{
+  GabbleDiscoIdentity *ret = g_new (GabbleDiscoIdentity, 1);
+
+  ret->category = g_strdup (source->category);
+  ret->type = g_strdup (source->type);
+  ret->lang = g_strdup (source->lang);
+  ret->name = g_strdup (source->name);
+  return ret;
+}
+
+const gchar *
+gabble_disco_identity_get_category (GabbleDiscoIdentity *identity)
+{
+  return identity->category;
+}
+
+const gchar *
+gabble_disco_identity_get_type (GabbleDiscoIdentity *identity)
+{
+  return identity->type;
+}
+
+const gchar *
+gabble_disco_identity_get_lang (GabbleDiscoIdentity *identity)
+{
+  return identity->lang;
+}
+
+const gchar *
+gabble_disco_identity_get_name (GabbleDiscoIdentity *identity)
+{
+  return identity->name;
+}
+
+void
+gabble_disco_identity_free (GabbleDiscoIdentity *identity)
+{
+  if (identity == NULL)
+    return;
+
+  g_free (identity->category);
+  g_free (identity->type);
+  g_free (identity->lang);
+  g_free (identity->name);
+  g_slice_free (GabbleDiscoIdentity, identity);
+}
+
+/**
+ * gabble_disco_identity_array_new:
+ *
+ * Creates a new array of GabbleDiscoIdentity objects.
+ *
+ * Returns: A newly instantiated array.
+ * See: gabble_disco_identity_array_free()
+ */
+GPtrArray *
+gabble_disco_identity_array_new (void)
+{
+  return g_ptr_array_new_with_free_func (
+      (GDestroyNotify) gabble_disco_identity_free);
+}
+
+/**
+ * gabble_disco_identity_array_copy():
+ * @source: The source array to be copied.
+ *
+ * Copies an array of GabbleDiscoIdentity objects. The returned array contains
+ * new copies of the contents of the source array.
+ *
+ * Returns: A newly instantiated array with new copies of the contents of the
+ *          source array.
+ * See: gabble_disco_identity_array_new()
+ */
+GPtrArray *
+gabble_disco_identity_array_copy (const GPtrArray *source)
+{
+  GPtrArray *ret;
+  guint i;
+
+  if (!source)
+    return NULL;
+
+  ret = g_ptr_array_sized_new (source->len);
+  g_ptr_array_set_free_func (ret, (GDestroyNotify) gabble_disco_identity_free);
+  for (i = 0; i < source->len; ++i)
+    {
+      g_ptr_array_add (ret,
+         gabble_disco_identity_copy (g_ptr_array_index (source, i)));
+    }
+  return ret;
+}
+
+/**
+ * gabble_disco_identity_array_free():
+ * @arr: Array to be freed.
+ *
+ * Frees an array of GabbleDiscoIdentity objects created with
+ * gabble_disco_identity_array_new() or returned by
+ * gabble_disco_identity_array_copy().
+ *
+ * Note that if this method is called with an array created with
+ * g_ptr_array_new, the caller should also free the array contents.
+ *
+ * See: gabble_disco_identity_array_new(), gabble_disco_identity_array_copy()
+ */
+void
+gabble_disco_identity_array_free (GPtrArray *arr)
+{
+  if (!arr)
+    return;
+
+  g_ptr_array_free (arr, TRUE);
+}
