@@ -28,13 +28,19 @@
 #include <telepathy-glib/handle-repo.h>
 #include <telepathy-glib/util.h>
 #include <loudmouth/loudmouth.h>
+#include <wocky/wocky-bare-contact.h>
+
+#include "jingle-factory.h"
+#include "jingle-content.h"
 
 #include "types.h"
 
-typedef LmMessageNode * NodeIter;
+#define CHECK_STR_EMPTY(x) ((x) == NULL || (x)[0] == '\0')
+
+typedef GSList * NodeIter;
 #define node_iter(node) (node->children)
-#define node_iter_next(i) (i->next)
-#define node_iter_data(i) (i)
+#define node_iter_next(i) (g_slist_next (i))
+#define node_iter_data(i) ((LmMessageNode *) i->data)
 
 /* Guarantees that the resulting hash is in lower-case */
 gchar *sha1_hex (const gchar *bytes, guint len);
@@ -47,8 +53,6 @@ gchar *gabble_generate_id (void);
 
 void lm_message_node_add_own_nick (LmMessageNode *node,
     GabbleConnection *conn);
-void lm_message_node_unlink (LmMessageNode *orphan,
-    LmMessageNode *parent);
 void lm_message_node_steal_children (LmMessageNode *snatcher,
     LmMessageNode *mum);
 gboolean lm_message_node_has_namespace (LmMessageNode *node, const gchar *ns,
@@ -97,4 +101,26 @@ const gchar * lm_message_node_get_attribute_with_namespace (LmMessageNode *node,
 
 GPtrArray *gabble_g_ptr_array_copy (GPtrArray *source);
 
+WockyBareContact * ensure_bare_contact_from_jid (GabbleConnection *conn,
+    const gchar *jid);
+
+gboolean jingle_pick_best_resource (GabbleConnection *conn,
+    TpHandle peer,
+    gboolean want_audio,
+    gboolean want_video,
+    const char **transport_ns,
+    JingleDialect *dialect,
+    const gchar **resource_out);
+
+const gchar *jingle_pick_best_content_type (GabbleConnection *conn,
+    TpHandle peer,
+    const gchar *resource,
+    JingleMediaType type);
+
+GPtrArray *gabble_call_candidates_to_array (GList *candidates);
+
+gchar * gabble_peer_to_jid (GabbleConnection *conn,
+    TpHandle peer,
+    const gchar *resource);
+
 #endif /* __GABBLE_UTIL_H__ */
