@@ -1,6 +1,6 @@
 
 """
-Test tubes capabilities with Connection.Interface.ContactCapabilities.DRAFT
+Test tubes capabilities with Connection.Interface.ContactCapabilities
 
 1. Receive presence and caps from contacts and check that
 GetContactCapabilities works correctly and that ContactCapabilitiesChanged is
@@ -15,7 +15,7 @@ results.
 - 1 stream tube + 1 D-Bus tube caps, again, to test whether the caps cache
   works with tubes
 
-2. Test SetSelfCapabilities and test that a presence stanza is sent to the
+2. Test UpdateCapabilities and test that a presence stanza is sent to the
 contacts, test that the D-Bus signal ContactCapabilitiesChanged is fired for
 the self handle, ask Gabble for its caps with an iq request, check the reply
 is correct, and ask Gabble for its caps using D-Bus method
@@ -124,12 +124,20 @@ def receive_caps(q, conn, stream, contact, contact_handle, features,
     # check the Contacts interface give the same caps
     caps_via_contacts_iface = conn.Contacts.GetContactAttributes(
             [contact_handle], [cs.CONN_IFACE_CONTACT_CAPS], False) \
-            [contact_handle][cs.CONN_IFACE_CONTACT_CAPS + '/caps']
+            [contact_handle][cs.ATTR_CONTACT_CAPABILITIES]
     assertSameElements(caps[contact_handle], caps_via_contacts_iface)
 
 def test_tube_caps_from_contact(q, bus, conn, stream, contact):
     contact_handle = conn.RequestHandles(cs.HT_CONTACT, [contact])[0]
 
+    # Check that we don't crash if we haven't seen any caps/presence for this
+    # contact yet.
+    caps = conn.ContactCapabilities.GetContactCapabilities([contact_handle])
+
+    # Since we don't know their caps, they should be omitted from the dict,
+    # rather than present with no caps.
+    assertEquals({}, caps)
+
     # send presence with no tube cap
     basic_caps = dbus.Dictionary({contact_handle:
             [(text_fixed_properties, text_allowed_properties)]})
@@ -199,19 +207,12 @@ def test_tube_caps_from_contact(q, bus, conn, stream, contact):
          ns.TUBES + '/stream#daap',
         ], daap_xiangqi_caps, expect_disco=False)
 
-def filter_tube_caps(caps):
-    return [x
-            for x in caps
-            if x[0].get(cs.CHANNEL + '.ChannelType') in (
-                cs.CHANNEL_TYPE_STREAM_TUBE,
-                cs.CHANNEL_TYPE_DBUS_TUBE,
-                )
-            ]
 
 def advertise_caps(q, conn, stream, filters, expected_features, unexpected_features,
                    expected_caps):
     self_handle = conn.GetSelfHandle()
-    ret_caps = conn.ContactCapabilities.SetSelfCapabilities(filters)
+    ret_caps = conn.ContactCapabilities.UpdateCapabilities(
+            [(cs.CLIENT + '.Foo', filters, [])])
 
     # Expect Gabble to reply with the correct caps
     event, namespaces, signaled_caps = receive_presence_and_ask_caps(q, stream)
@@ -226,72 +227,76 @@ def advertise_caps(q, conn, stream, filters, expected_features, unexpected_featu
     for var in unexpected_features:
         assertDoesNotContain(var, namespaces)
 
-    # Check our own caps, ignoring non-Tube caps
+    # Check our own caps
     caps = conn.ContactCapabilities.GetContactCapabilities([self_handle])
-    caps[self_handle] = filter_tube_caps(caps[self_handle])
     assertSameElements(expected_caps, caps)
 
     # check the Contacts interface give the same caps
     caps_via_contacts_iface = conn.Contacts.GetContactAttributes(
             [self_handle], [cs.CONN_IFACE_CONTACT_CAPS], False) \
-            [self_handle][cs.CONN_IFACE_CONTACT_CAPS + '/caps']
-    caps_via_contacts_iface = filter_tube_caps(caps_via_contacts_iface)
+            [self_handle][cs.ATTR_CONTACT_CAPABILITIES]
     assertSameElements(caps[self_handle], caps_via_contacts_iface)
 
 def test_tube_caps_to_contact(q, bus, conn, stream):
     self_handle = conn.GetSelfHandle()
 
     basic_caps = dbus.Dictionary({self_handle:
-        [(stream_tube_fixed_properties, stream_tube_allowed_properties),
+        [(text_fixed_properties, text_allowed_properties),
+         (stream_tube_fixed_properties, stream_tube_allowed_properties),
          (dbus_tube_fixed_properties, dbus_tube_allowed_properties),
          ]})
     daap_caps = dbus.Dictionary({self_handle:
-        [(stream_tube_fixed_properties, stream_tube_allowed_properties),
-        (dbus_tube_fixed_properties, dbus_tube_allowed_properties),
-        (daap_fixed_properties, specialized_tube_allowed_properties)]})
-    xiangqi_caps = dbus.Dictionary({self_handle:
-        [(stream_tube_fixed_properties, stream_tube_allowed_properties),
-        (dbus_tube_fixed_properties, dbus_tube_allowed_properties),
-        (xiangqi_fixed_properties, specialized_tube_allowed_properties)]})
-    daap_xiangqi_caps = dbus.Dictionary({self_handle:
-        [(stream_tube_fixed_properties, stream_tube_allowed_properties),
+        [(text_fixed_properties, text_allowed_properties),
+        (stream_tube_fixed_properties, stream_tube_allowed_properties),
         (dbus_tube_fixed_properties, dbus_tube_allowed_properties),
         (daap_fixed_properties, specialized_tube_allowed_properties),
-        (xiangqi_fixed_properties, specialized_tube_allowed_properties)]})
+        (ft_fixed_properties, ft_allowed_properties)]})
+    xiangqi_caps = dbus.Dictionary({self_handle:
+        [(text_fixed_properties, text_allowed_properties),
+        (stream_tube_fixed_properties, stream_tube_allowed_properties),
+        (dbus_tube_fixed_properties, dbus_tube_allowed_properties),
+        (xiangqi_fixed_properties, specialized_tube_allowed_properties),
+        (ft_fixed_properties, ft_allowed_properties)]})
+    daap_xiangqi_caps = dbus.Dictionary({self_handle:
+        [(text_fixed_properties, text_allowed_properties),
+        (stream_tube_fixed_properties, stream_tube_allowed_properties),
+        (dbus_tube_fixed_properties, dbus_tube_allowed_properties),
+        (daap_fixed_properties, specialized_tube_allowed_properties),
+        (xiangqi_fixed_properties, specialized_tube_allowed_properties),
+        (ft_fixed_properties, ft_allowed_properties)]})
     all_tubes_caps = dbus.Dictionary({self_handle:
-        [(stream_tube_fixed_properties, stream_tube_allowed_properties),
+        [(text_fixed_properties, text_allowed_properties),
+        (stream_tube_fixed_properties, stream_tube_allowed_properties),
         (dbus_tube_fixed_properties, dbus_tube_allowed_properties),
         (daap_fixed_properties, specialized_tube_allowed_properties),
         (http_fixed_properties, specialized_tube_allowed_properties),
         (xiangqi_fixed_properties, specialized_tube_allowed_properties),
-        (go_fixed_properties, specialized_tube_allowed_properties)]})
+        (go_fixed_properties, specialized_tube_allowed_properties),
+        (ft_fixed_properties, ft_allowed_properties)]})
 
-    # Check our own caps, after removing non-tube capabilities
+    # Check our own caps
     caps = conn.ContactCapabilities.GetContactCapabilities([self_handle])
-    caps[self_handle] = filter_tube_caps(caps[self_handle])
     assertEquals(basic_caps, caps)
 
     # check the Contacts interface give the same caps
     caps_via_contacts_iface = conn.Contacts.GetContactAttributes(
             [self_handle], [cs.CONN_IFACE_CONTACT_CAPS], False) \
-            [self_handle][cs.CONN_IFACE_CONTACT_CAPS + '/caps']
-    caps_via_contacts_iface = filter_tube_caps(caps_via_contacts_iface)
+            [self_handle][cs.ATTR_CONTACT_CAPABILITIES]
     assertEquals(caps[self_handle], caps_via_contacts_iface)
 
     # Advertise nothing
-    conn.ContactCapabilities.SetSelfCapabilities([])
+    conn.ContactCapabilities.UpdateCapabilities(
+            [(cs.CLIENT + '.Foo', {}, [])])
 
     # Check our own caps
     caps = conn.ContactCapabilities.GetContactCapabilities([self_handle])
     assertLength(1, caps)
-    caps[self_handle] = filter_tube_caps(caps[self_handle])
     assertEquals(basic_caps, caps)
 
     # check the Contacts interface give the same caps
     caps_via_contacts_iface = conn.Contacts.GetContactAttributes(
             [self_handle], [cs.CONN_IFACE_CONTACT_CAPS], False) \
-            [self_handle][cs.CONN_IFACE_CONTACT_CAPS + '/caps']
-    caps_via_contacts_iface = filter_tube_caps(caps_via_contacts_iface)
+            [self_handle][cs.ATTR_CONTACT_CAPABILITIES]
     assertEquals(caps[self_handle], caps_via_contacts_iface)
 
     sync_stream(q, stream)
