@@ -9,25 +9,21 @@ import sys
 import dbus
 import servicetest
 
-import twisted
-from twisted.words.xish import domish, xpath
+from twisted.words.xish import domish
 from twisted.words.protocols.jabber import xmlstream
+import twisted.internet.protocol
+from twisted.internet import reactor
 
-from gabbletest import make_connection, make_stream, JabberAuthenticator, \
-                       XmppAuthenticator, \
-                       XmppXmlStream, JabberXmlStream
+from servicetest import (Event, unwrap)
+
+from gabbletest import (
+    make_connection, make_stream, XmppAuthenticator, XmppXmlStream,
+    disconnect_conn)
+import constants as cs
 
 NS_XMPP_TLS = 'urn:ietf:params:xml:ns:xmpp-tls'
 NS_XMPP_SASL = 'urn:ietf:params:xml:ns:xmpp-sasl'
 
-
-print "FIXME: test-nonblocking-tls.py disabled due to a bug in Loudmouth:"
-print "       http://loudmouth.lighthouseapp.com/projects/17276/tickets/5"
-print "       https://bugs.freedesktop.org/show_bug.cgi?id=14341"
-# exiting 77 causes automake to consider the test to have been skipped
-raise SystemExit(77)
-
-
 class BlockForeverTlsAuthenticator(xmlstream.Authenticator):
     """A TLS stream authenticator that is deliberately broken. It sends
     <proceed/> to the client but then do nothing, so the TLS handshake will
@@ -67,7 +63,8 @@ class BlockForeverTlsAuthenticator(xmlstream.Authenticator):
 def test(q, bus, conn1, conn2, stream1, stream2):
     # Connection 1
     conn1.Connect()
-    q.expect('dbus-signal', signal='StatusChanged', args=[1, 1])
+    q.expect('dbus-signal', signal='StatusChanged',
+            args=[cs.CONN_STATUS_CONNECTING, cs.CSR_REQUESTED])
 
     # Connection 1 blocks because the fake jabber server behind conn1 does not
     # proceed to the tls handshake. The second connection is independant and
@@ -75,17 +72,16 @@ def test(q, bus, conn1, conn2, stream1, stream2):
 
     # Connection 2
     conn2.Connect()
-    q.expect('dbus-signal', signal='StatusChanged', args=[1, 1])
+    q.expect('dbus-signal', signal='StatusChanged',
+            args=[cs.CONN_STATUS_CONNECTING, cs.CSR_REQUESTED])
     q.expect('stream-authenticated')
     q.expect('dbus-signal', signal='PresenceUpdate',
         args=[{1L: (0L, {u'available': {}})}])
-    q.expect('dbus-signal', signal='StatusChanged', args=[0, 1])
+    q.expect('dbus-signal', signal='StatusChanged',
+            args=[cs.CONN_STATUS_CONNECTED, cs.CSR_REQUESTED])
 
     # Disconnection 2
-    conn2.Disconnect()
-    q.expect('dbus-signal', signal='StatusChanged', args=[2, 1])
-
-    return True
+    disconnect_conn(q, conn2, stream2)
 
 if __name__ == '__main__':
     queue = servicetest.IteratingEventQueue(None)
@@ -102,10 +98,13 @@ if __name__ == '__main__':
         'server': 'localhost',
         'port': dbus.UInt32(4242),
         }
-    conn1 = make_connection(bus, queue.append, params)
+    conn1, jid1 = make_connection(bus, queue.append, params)
     authenticator = BlockForeverTlsAuthenticator('test1', 'pass')
-    stream1 = make_stream(queue.append, authenticator, protocol=XmppXmlStream,
-                          port=4242)
+    stream1 = make_stream(queue.append, authenticator, protocol=XmppXmlStream)
+
+    factory = twisted.internet.protocol.Factory()
+    factory.protocol = lambda:stream1
+    port1 = reactor.listenTCP(4242, factory)
 
     params = {
         'account': 'test2@localhost/Resource',
@@ -114,10 +113,29 @@ if __name__ == '__main__':
         'server': 'localhost',
         'port': dbus.UInt32(4343),
         }
-    conn2 = make_connection(bus, queue.append, params)
+    conn2, jid2 = make_connection(bus, queue.append, params)
     authenticator = XmppAuthenticator('test2', 'pass')
-    stream2 = make_stream(queue.append, authenticator, protocol=XmppXmlStream,
-                          port=4343)
+    stream2 = make_stream(queue.append, authenticator, protocol=XmppXmlStream)
+
+    factory = twisted.internet.protocol.Factory()
+    factory.protocol = lambda:stream2
+    port1 = reactor.listenTCP(4343, factory)
+
+
+    bus.add_signal_receiver(
+        lambda *args, **kw:
+            queue.append(Event('dbus-signal',
+                               path=unwrap(kw['path']),
+                               signal=kw['member'], args=map(unwrap, args),
+                               interface=kw['interface'])),
+        None,       # signal name
+        None,       # interface
+        None,
+        path_keyword='path',
+        member_keyword='member',
+        interface_keyword='interface',
+        byte_arrays=True
+        )
 
     try:
         test(queue, bus, conn1, conn2, stream1, stream2)
@@ -125,9 +143,6 @@ if __name__ == '__main__':
         try:
             conn1.Disconnect()
             conn2.Disconnect()
-            # second call destroys object
-            conn1.Disconnect()
-            conn2.Disconnect()
         except dbus.DBusException, e:
             pass
 
