@@ -52,7 +52,7 @@ def test_ft_caps_from_contact(q, bus, conn, stream, contact, contact_handle, cli
     # check the Contacts interface give the same caps
     caps_via_contacts_iface = conn_contacts_iface.GetContactAttributes(
             [contact_handle], [cs.CONN_IFACE_CONTACT_CAPS], False) \
-            [contact_handle][cs.CONN_IFACE_CONTACT_CAPS + '/caps']
+            [contact_handle][cs.ATTR_CONTACT_CAPABILITIES]
     assert caps_via_contacts_iface == caps[contact_handle], \
                                     caps_via_contacts_iface
 
@@ -94,7 +94,7 @@ def test_ft_caps_from_contact(q, bus, conn, stream, contact, contact_handle, cli
     # check the Contacts interface give the same caps
     caps_via_contacts_iface = conn_contacts_iface.GetContactAttributes(
             [contact_handle], [cs.CONN_IFACE_CONTACT_CAPS], False) \
-            [contact_handle][cs.CONN_IFACE_CONTACT_CAPS + '/caps']
+            [contact_handle][cs.ATTR_CONTACT_CAPABILITIES]
     assert caps_via_contacts_iface == caps[contact_handle], \
                                     caps_via_contacts_iface
 
@@ -109,7 +109,7 @@ def test(q, bus, conn, stream):
         2L, client)
 
     # our own capabilities, formerly tested here, are now in
-    # tests/twisted/caps/advertise-draft1.py
+    # tests/twisted/caps/advertise-contact-capabilities.py
 
 if __name__ == '__main__':
     exec_test(test)
