@@ -15,8 +15,8 @@ import ns
 import constants as cs
 import servicetest
 from servicetest import (
-    assertEquals, assertLength, assertContains, wrap_channel, EventPattern,
-    )
+    assertEquals, assertLength, assertContains, wrap_channel,
+    EventPattern, call_async, unwrap, Event)
 import twisted
 from twisted.words.xish import domish, xpath
 from twisted.words.protocols.jabber.client import IQ
@@ -28,7 +28,7 @@ import dbus
 NS_XMPP_SASL = 'urn:ietf:params:xml:ns:xmpp-sasl'
 NS_XMPP_BIND = 'urn:ietf:params:xml:ns:xmpp-bind'
 
-def make_result_iq(stream, iq):
+def make_result_iq(stream, iq, add_query_node=True):
     result = IQ(stream, "result")
     result["id"] = iq["id"]
     to = iq.getAttribute('to')
@@ -36,7 +36,7 @@ def make_result_iq(stream, iq):
         result["from"] = to
     query = iq.firstChildElement()
 
-    if query:
+    if query and add_query_node:
         result.addElement((query.uri, query.name))
 
     return result
@@ -87,15 +87,18 @@ def sync_stream(q, stream):
         predicate=(lambda event:
             event.stanza['id'] == id and event.iq_type == 'result'))
 
-class JabberAuthenticator(xmlstream.Authenticator):
-    "Trivial XML stream authenticator that accepts one username/digest pair."
-
+class GabbleAuthenticator(xmlstream.Authenticator):
     def __init__(self, username, password, resource=None):
         self.username = username
         self.password = password
         self.resource = resource
+        self.bare_jid = None
+        self.full_jid = None
         xmlstream.Authenticator.__init__(self)
 
+class JabberAuthenticator(GabbleAuthenticator):
+    "Trivial XML stream authenticator that accepts one username/digest pair."
+
     # Patch in fix from http://twistedmatrix.com/trac/changeset/23418.
     # This monkeypatch taken from Gadget source code
     from twisted.words.xish.utility import EventDispatcher
@@ -145,18 +148,18 @@ class JabberAuthenticator(xmlstream.Authenticator):
         if self.resource is not None:
             assertEquals(self.resource, str(resource[0]))
 
+        self.bare_jid = '%s@localhost' % self.username
+        self.full_jid = '%s/%s' % (self.bare_jid, resource)
+
         result = IQ(self.xmlstream, "result")
         result["id"] = iq["id"]
         self.xmlstream.send(result)
         self.xmlstream.dispatch(self.xmlstream, xmlstream.STREAM_AUTHD_EVENT)
 
 
-class XmppAuthenticator(xmlstream.Authenticator):
+class XmppAuthenticator(GabbleAuthenticator):
     def __init__(self, username, password, resource=None):
-        xmlstream.Authenticator.__init__(self)
-        self.username = username
-        self.password = password
-        self.resource = resource
+        GabbleAuthenticator.__init__(self, username, password, resource)
         self.authenticated = False
 
     def streamStarted(self, root=None):
@@ -201,18 +204,21 @@ class XmppAuthenticator(xmlstream.Authenticator):
         result = IQ(self.xmlstream, "result")
         result["id"] = iq["id"]
         bind = result.addElement((NS_XMPP_BIND, 'bind'))
-        jid = bind.addElement('jid', content=('test@localhost/%s' % resource))
+        self.bare_jid = '%s@localhost' % self.username
+        self.full_jid = '%s/%s' % (self.bare_jid, resource)
+        jid = bind.addElement('jid', content=self.full_jid)
         self.xmlstream.send(result)
 
         self.xmlstream.dispatch(self.xmlstream, xmlstream.STREAM_AUTHD_EVENT)
 
-def make_stream_event(type, stanza):
+def make_stream_event(type, stanza, stream):
     event = servicetest.Event(type, stanza=stanza)
+    event.stream = stream
     event.to = stanza.getAttribute("to")
     return event
 
-def make_iq_event(iq):
-    event = make_stream_event('stream-iq', iq)
+def make_iq_event(stream, iq):
+    event = make_stream_event('stream-iq', iq, stream)
     event.iq_type = iq.getAttribute("type")
     event.iq_id = iq.getAttribute("id")
     query = iq.firstChildElement()
@@ -229,16 +235,98 @@ def make_iq_event(iq):
 
     return event
 
-def make_presence_event(stanza):
-    event = make_stream_event('stream-presence', stanza)
+def make_presence_event(stream, stanza):
+    event = make_stream_event('stream-presence', stanza, stream)
     event.presence_type = stanza.getAttribute('type')
     return event
 
-def make_message_event(stanza):
-    event = make_stream_event('stream-message', stanza)
+def make_message_event(stream, stanza):
+    event = make_stream_event('stream-message', stanza, stream)
     event.message_type = stanza.getAttribute('type')
     return event
 
+class StreamFactory(twisted.internet.protocol.Factory):
+    def __init__(self, streams, jids):
+        self.streams = streams
+        self.jids = jids
+        self.presences = {}
+        self.mappings = dict(map (lambda jid, stream: (jid, stream),
+                                  jids, streams))
+
+        # Make a copy of the streams
+        self.factory_streams = list(streams)
+        self.factory_streams.reverse()
+
+        # Do not add observers for single instances because it's unnecessary and
+        # some unit tests need to respond to the roster request, and we shouldn't
+        # answer it for them otherwise we break compatibility
+        if len(streams) > 1:
+            # We need to have a function here because lambda keeps a reference on
+            # the stream and jid and in the for loop, there is no context
+            def addObservers(stream, jid):
+                stream.addObserver('/iq', lambda x: \
+                                       self.forward_iq(stream, jid, x))
+                stream.addObserver('/presence', lambda x: \
+                                       self.got_presence(stream, jid, x))
+
+            for (jid, stream) in self.mappings.items():
+                addObservers(stream, jid)
+
+    def protocol(self, *args):
+        return self.factory_streams.pop()
+
+
+    def got_presence (self, stream, jid, stanza):
+        stanza.attributes['from'] = jid
+        self.presences[jid] = stanza
+
+        for dest_jid  in self.presences.keys():
+            # Dispatch the new presence to other clients
+            stanza.attributes['to'] = dest_jid
+            self.mappings[dest_jid].send(stanza)
+
+            # Don't echo the presence twice
+            if dest_jid != jid:
+                # Dispatch other client's presence to this stream
+                presence = self.presences[dest_jid]
+                presence.attributes['to'] = jid
+                stream.send(presence)
+
+    def lost_presence(self, stream, jid):
+        if self.presences.has_key(jid):
+            del self.presences[jid]
+            for dest_jid  in self.presences.keys():
+                presence = domish.Element(('jabber:client', 'presence'))
+                presence['from'] = jid
+                presence['to'] = dest_jid
+                presence['type'] = 'unavailable'
+                self.mappings[dest_jid].send(presence)
+
+    def forward_iq(self, stream, jid, stanza):
+        stanza.attributes['from'] = jid
+
+        query = stanza.firstChildElement()
+
+        # Fake other accounts as being part of our roster
+        if query and query.uri == ns.ROSTER:
+            roster = make_result_iq(stream, stanza)
+            query = roster.firstChildElement()
+            for roster_jid in self.mappings.keys():
+                if jid != roster_jid:
+                    item = query.addElement('item')
+                    item['jid'] = roster_jid
+                    item['subscription'] = 'both'
+            stream.send(roster)
+            return
+
+        to = stanza.getAttribute('to')
+        dest = None
+        if to is not None:
+            dest = self.mappings.get(to)
+
+        if dest is not None:
+            dest.send(stanza)
+
 class BaseXmlStream(xmlstream.XmlStream):
     initiating = False
     namespace = 'jabber:client'
@@ -247,34 +335,51 @@ class BaseXmlStream(xmlstream.XmlStream):
         xmlstream.XmlStream.__init__(self, authenticator)
         self.event_func = event_func
         self.addObserver('//iq', lambda x: event_func(
-            make_iq_event(x)))
+            make_iq_event(self, x)))
         self.addObserver('//message', lambda x: event_func(
-            make_message_event(x)))
+            make_message_event(self, x)))
         self.addObserver('//presence', lambda x: event_func(
-            make_presence_event(x)))
+            make_presence_event(self, x)))
         self.addObserver('//event/stream/authd', self._cb_authd)
 
     def _cb_authd(self, _):
         # called when stream is authenticated
+        assert self.authenticator.full_jid is not None
+        assert self.authenticator.bare_jid is not None
+
         self.addObserver(
-            "/iq/query[@xmlns='http://jabber.org/protocol/disco#info']",
+            "/iq[@to='localhost']/query[@xmlns='http://jabber.org/protocol/disco#info']",
             self._cb_disco_iq)
+        self.addObserver(
+            "/iq[@to='%s']/query[@xmlns='http://jabber.org/protocol/disco#info']"
+                % self.authenticator.bare_jid,
+            self._cb_bare_jid_disco_iq)
         self.event_func(servicetest.Event('stream-authenticated'))
 
     def _cb_disco_iq(self, iq):
-        if iq.getAttribute('to') == 'localhost':
-            # add PEP support
-            nodes = xpath.queryForNodes(
-                "/iq/query[@xmlns='http://jabber.org/protocol/disco#info']",
-                iq)
-            query = nodes[0]
-            identity = query.addElement('identity')
-            identity['category'] = 'pubsub'
-            identity['type'] = 'pep'
+        iq['type'] = 'result'
+        iq['from'] = iq['to']
+        self.send(iq)
 
-            iq['type'] = 'result'
-            iq['from'] = iq['to']
-            self.send(iq)
+    def _cb_bare_jid_disco_iq(self, iq):
+        # advertise PEP support
+        nodes = xpath.queryForNodes(
+            "/iq/query[@xmlns='http://jabber.org/protocol/disco#info']",
+            iq)
+        query = nodes[0]
+        identity = query.addElement('identity')
+        identity['category'] = 'pubsub'
+        identity['type'] = 'pep'
+
+        iq['type'] = 'result'
+        iq['from'] = iq['to']
+        self.send(iq)
+
+    def onDocumentEnd(self):
+        self.event_func(servicetest.Event('stream-closed'))
+        # We don't chain up XmlStream.onDocumentEnd() because it will
+        # disconnect the TCP connection making tests as
+        # connect/disconnect-timeout.py not working
 
 class JabberXmlStream(BaseXmlStream):
     version = (0, 9)
@@ -295,15 +400,26 @@ class GoogleXmlStream(BaseXmlStream):
             feature['var'] = ns.GOOGLE_ROSTER
             feature = query.addElement('feature')
             feature['var'] = ns.GOOGLE_JINGLE_INFO
+            feature = query.addElement('feature')
+            feature['var'] = ns.GOOGLE_MAIL_NOTIFY
 
             iq['type'] = 'result'
             iq['from'] = 'localhost'
             self.send(iq)
 
-def make_connection(bus, event_func, params=None):
+    def _cb_bare_jid_disco_iq(self, iq):
+        # Google talk doesn't support PEP :(
+        iq['type'] = 'result'
+        iq['from'] = iq['to']
+        self.send(iq)
+
+
+def make_connection(bus, event_func, params=None, suffix=''):
     # Gabble accepts a resource in 'account', but the value of 'resource'
     # overrides it if there is one.
-    account = 'test@localhost/%s' % re.sub(r'.*tests/twisted/', '', sys.argv[0])
+
+    account = 'test%s@localhost/%s' % (suffix, re.sub(r'.*/', '', sys.argv[0]))
+
     default_params = {
         'account': account,
         'password': 'pass',
@@ -316,35 +432,49 @@ def make_connection(bus, event_func, params=None):
     if params:
         default_params.update(params)
 
-    return servicetest.make_connection(bus, event_func, 'gabble', 'jabber',
-        default_params)
+     # Allow omitting the 'password' param
+    if default_params['password'] is None:
+        del default_params['password']
 
-def make_stream(event_func, authenticator=None, protocol=None, port=4242, resource=None):
+     # Allow omitting the 'account' param
+    if default_params['account'] is None:
+        del default_params['account']
+
+    jid = default_params.get('account', None)
+    conn =  servicetest.make_connection(bus, event_func, 'gabble', 'jabber',
+                                        default_params)
+    return (conn, jid)
+
+def make_stream(event_func, authenticator=None, protocol=None,
+                resource=None, suffix=''):
     # set up Jabber server
-
     if authenticator is None:
-        authenticator = XmppAuthenticator('test', 'pass', resource=resource)
+        authenticator = XmppAuthenticator('test%s' % suffix, 'pass', resource=resource)
 
     if protocol is None:
         protocol = XmppXmlStream
 
     stream = protocol(event_func, authenticator)
-    factory = twisted.internet.protocol.Factory()
-    factory.protocol = lambda *args: stream
-    port = reactor.listenTCP(port, factory)
-    return (stream, port)
+    return stream
 
-def disconnect_conn(q, conn, stream, expected=[]):
-    conn.Disconnect()
+def disconnect_conn(q, conn, stream, expected_before=[], expected_after=[]):
+    call_async(q, conn, 'Disconnect')
 
-    tmp = expected + [EventPattern('dbus-signal', signal='StatusChanged',
-            args=[cs.CONN_STATUS_DISCONNECTED, cs.CSR_REQUESTED])]
+    tmp = expected_before + [
+        EventPattern('dbus-signal', signal='StatusChanged', args=[cs.CONN_STATUS_DISCONNECTED, cs.CSR_REQUESTED]),
+        EventPattern('stream-closed')]
 
-    events = q.expect_many(*tmp)
-    return events[:-1]
+    before_events = q.expect_many(*tmp)
+
+    stream.sendFooter()
+
+    tmp = expected_after + [EventPattern('dbus-return', method='Disconnect')]
+    after_events = q.expect_many(*tmp)
+
+    return before_events[:-2], after_events[:-1]
 
 def exec_test_deferred(fun, params, protocol=None, timeout=None,
-                        authenticator=None):
+                        authenticator=None, num_instances=1):
     # hack to ease debugging
     domish.Element.__repr__ = domish.Element.toXml
     colourer = None
@@ -352,21 +482,66 @@ def exec_test_deferred(fun, params, protocol=None, timeout=None,
     if sys.stdout.isatty() or 'CHECK_FORCE_COLOR' in os.environ:
         colourer = servicetest.install_colourer()
 
+    bus = dbus.SessionBus()
+
     queue = servicetest.IteratingEventQueue(timeout)
     queue.verbose = (
         os.environ.get('CHECK_TWISTED_VERBOSE', '') != ''
         or '-v' in sys.argv)
 
-    bus = dbus.SessionBus()
-    conn = make_connection(bus, queue.append, params)
+    conns = []
+    jids = []
+    streams = []
     resource = params.get('resource') if params is not None else None
-    (stream, port) = make_stream(queue.append, protocol=protocol,
-        authenticator=authenticator, resource=resource)
+    for i in range(0, num_instances):
+        if i == 0:
+            suffix = ''
+        else:
+            suffix = str(i)
+        (conn, jid) = make_connection(bus, queue.append, params, suffix)
+        conns.append(conn)
+        jids.append(jid)
+        streams.append(make_stream(queue.append, protocol=protocol,
+                                   authenticator=authenticator,
+                                   resource=resource, suffix=suffix))
+
+    factory = StreamFactory(streams, jids)
+    port = reactor.listenTCP(4242, factory)
+
+    def signal_receiver(*args, **kw):
+        if kw['path'] == '/org/freedesktop/DBus' and \
+                kw['member'] == 'NameOwnerChanged':
+            bus_name, old_name, new_name = args
+            if new_name == '':
+                for i, conn in enumerate(conns):
+                    stream = streams[i]
+                    jid = jids[i]
+                    if conn._requested_bus_name == bus_name:
+                        factory.lost_presence(stream, jid)
+                        break
+        queue.append(Event('dbus-signal',
+                           path=unwrap(kw['path']),
+                           signal=kw['member'], args=map(unwrap, args),
+                           interface=kw['interface']))
+
+    bus.add_signal_receiver(
+        signal_receiver,
+        None,       # signal name
+        None,       # interface
+        None,
+        path_keyword='path',
+        member_keyword='member',
+        interface_keyword='interface',
+        byte_arrays=True
+        )
 
     error = None
 
     try:
-        fun(queue, bus, conn, stream)
+        if len(conns) == 1:
+            fun(queue, bus, conns[0], streams[0])
+        else:
+            fun(queue, bus, conns, streams)
     except Exception, e:
         traceback.print_exc()
         error = e
@@ -376,22 +551,41 @@ def exec_test_deferred(fun, params, protocol=None, timeout=None,
 
     d = port.stopListening()
 
+    # Does the Connection object still exist?
+    for i, conn in enumerate(conns):
+        if not bus.name_has_owner(conn.object.bus_name):
+            # Connection has already been disconnected and destroyed
+            continue
+        try:
+            if conn.GetStatus() == cs.CONN_STATUS_CONNECTED:
+                # Connection is connected, properly disconnect it
+                disconnect_conn(queue, conn, streams[i])
+            else:
+                # Connection is not connected, call Disconnect() to destroy it
+                conn.Disconnect()
+        except dbus.DBusException, e:
+            pass
+
+        try:
+            conn.Disconnect()
+            raise AssertionError("Connection didn't disappear")
+        except dbus.DBusException, e:
+            pass
+        except Exception, e:
+            traceback.print_exc()
+            error = e
+
     if error is None:
         d.addBoth((lambda *args: reactor.crash()))
     else:
         # please ignore the POSIX behind the curtain
         d.addBoth((lambda *args: os._exit(1)))
 
-    try:
-        disconnect_conn(queue, conn, stream)
-    except dbus.DBusException, e:
-        # Connection has already been disconnected
-        pass
 
 def exec_test(fun, params=None, protocol=None, timeout=None,
-              authenticator=None):
+              authenticator=None, num_instances=1):
     reactor.callWhenRunning(
-        exec_test_deferred, fun, params, protocol, timeout, authenticator)
+        exec_test_deferred, fun, params, protocol, timeout, authenticator, num_instances)
     reactor.run()
 
 # Useful routines for server-side vCard handling
@@ -435,7 +629,7 @@ def _elem_add(elem, *children):
             raise ValueError(
                 'invalid child object %r (must be element or unicode)', child)
 
-def elem(a, b=None, **kw):
+def elem(a, b=None, attrs={}, **kw):
     r"""
     >>> elem('foo')().toXml()
     u'<foo/>'
@@ -446,6 +640,10 @@ def elem(a, b=None, **kw):
     >>> elem('foo', x='1')(u'hello',
     ...         elem('http://foo.org', 'bar', y='2')(u'bye')).toXml()
     u"<foo x='1'>hello<bar xmlns='http://foo.org' y='2'>bye</bar></foo>"
+    >>> elem('foo', attrs={'xmlns:bar': 'urn:bar', 'bar:cake': 'yum'})(
+    ...   elem('bar:e')(u'i')
+    ... ).toXml()
+    u"<foo xmlns:bar='urn:bar' bar:cake='yum'><bar:e>i</bar:e></foo>"
     """
 
     class _elem(domish.Element):
@@ -458,7 +656,22 @@ def elem(a, b=None, **kw):
     else:
         elem = _elem((None, a))
 
-    for k, v in kw.iteritems():
+    # Can't just update kw into attrs, because that *modifies the parameter's
+    # default*. Thanks python.
+    allattrs = {}
+    allattrs.update(kw)
+    allattrs.update(attrs)
+
+    # First, let's pull namespaces out
+    realattrs = {}
+    for k, v in allattrs.iteritems():
+        if k.startswith('xmlns:'):
+            abbr = k[len('xmlns:'):]
+            elem.localPrefixes[abbr] = v
+        else:
+            realattrs[k] = v
+
+    for k, v in realattrs.iteritems():
         if k == 'from_':
             elem['from'] = v
         else:
@@ -508,77 +721,3 @@ def make_presence(_from, to='test@localhost', type=None, show=None,
         x.addElement('photo').addContent(photo)
 
     return presence
-
-def expect_list_channel(q, bus, conn, name, contacts, lp_contacts=[],
-                        rp_contacts=[]):
-    return expect_contact_list_channel(q, bus, conn, cs.HT_LIST, name,
-        contacts, lp_contacts=lp_contacts, rp_contacts=rp_contacts)
-
-def expect_group_channel(q, bus, conn, name, contacts, lp_contacts=[],
-                         rp_contacts=[]):
-    return expect_contact_list_channel(q, bus, conn, cs.HT_GROUP, name,
-        contacts, lp_contacts=lp_contacts, rp_contacts=rp_contacts)
-
-def expect_contact_list_channel(q, bus, conn, ht, name, contacts,
-                                lp_contacts=[], rp_contacts=[]):
-    """
-    Expects NewChannel and NewChannels signals for the
-    contact list with handle type 'ht' and ID 'name', and checks that its
-    members, lp members and rp members are exactly 'contacts', 'lp_contacts'
-    and 'rp_contacts'.
-    Returns a proxy for the channel.
-    """
-
-    old_signal, new_signal = q.expect_many(
-            EventPattern('dbus-signal', signal='NewChannel'),
-            EventPattern('dbus-signal', signal='NewChannels'),
-            )
-
-    path, type, handle_type, handle, suppress_handler = old_signal.args
-
-    assertEquals(cs.CHANNEL_TYPE_CONTACT_LIST, type)
-    assertEquals(name, conn.InspectHandles(handle_type, [handle])[0])
-
-    chan = wrap_channel(bus.get_object(conn.bus_name, path),
-        cs.CHANNEL_TYPE_CONTACT_LIST)
-    members = chan.Group.GetMembers()
-
-    assertEquals(sorted(contacts),
-        sorted(conn.InspectHandles(cs.HT_CONTACT, members)))
-
-    lp_handles = conn.RequestHandles(cs.HT_CONTACT, lp_contacts)
-    rp_handles = conn.RequestHandles(cs.HT_CONTACT, rp_contacts)
-
-    # NB. comma: we're unpacking args. Thython!
-    info, = new_signal.args
-    assertLength(1, info) # one channel
-    path_, emitted_props = info[0]
-
-    assertEquals(path_, path)
-
-    assertEquals(cs.CHANNEL_TYPE_CONTACT_LIST, emitted_props[cs.CHANNEL_TYPE])
-    assertEquals(ht, emitted_props[cs.TARGET_HANDLE_TYPE])
-    assertEquals(handle, emitted_props[cs.TARGET_HANDLE])
-
-    channel_props = chan.Properties.GetAll(cs.CHANNEL)
-    assertEquals(handle, channel_props.get('TargetHandle'))
-    assertEquals(ht, channel_props.get('TargetHandleType'))
-    assertEquals(cs.CHANNEL_TYPE_CONTACT_LIST, channel_props.get('ChannelType'))
-    assertContains(cs.CHANNEL_IFACE_GROUP, channel_props.get('Interfaces'))
-    assertEquals(name, channel_props['TargetID'])
-    assertEquals(False, channel_props['Requested'])
-    assertEquals('', channel_props['InitiatorID'])
-    assertEquals(0, channel_props['InitiatorHandle'])
-
-    group_props = chan.Properties.GetAll(cs.CHANNEL_IFACE_GROUP)
-    assertContains('HandleOwners', group_props)
-    assertContains('Members', group_props)
-    assertEquals(members, group_props['Members'])
-    assertContains('LocalPendingMembers', group_props)
-    actual_lp_handles = [x[0] for x in group_props['LocalPendingMembers']]
-    assertEquals(sorted(lp_handles), sorted(actual_lp_handles))
-    assertContains('RemotePendingMembers', group_props)
-    assertEquals(sorted(rp_handles), sorted(group_props['RemotePendingMembers']))
-    assertContains('GroupFlags', group_props)
-
-    return chan
