@@ -176,8 +176,13 @@ def test(q, bus, conn, stream, incoming=True, too_slow=None):
                   })
         ret, old_sig, new_sig = q.expect_many(
             EventPattern('dbus-return', method='CreateChannel'),
-            EventPattern('dbus-signal', signal='NewChannel'),
-            EventPattern('dbus-signal', signal='NewChannels'),
+            EventPattern('dbus-signal', signal='NewChannel',
+                predicate=lambda e:
+                cs.CHANNEL_TYPE_CONTACT_LIST not in e.args),
+            EventPattern('dbus-signal', signal='NewChannels',
+                predicate=lambda e:
+                    cs.CHANNEL_TYPE_CONTACT_LIST not in
+                    e.args[0][0][1].values()),
             )
         path = ret.value[0]
         media_chan = make_channel_proxy(conn, path, 'Channel.Interface.Group')
