@@ -15,10 +15,6 @@ import gabbletest
 import constants as cs
 
 def test(q, bus, conn, stream):
-    jt = jingletest.JingleTest(stream, 'test@localhost', 'foo@bar.com/Foo')
-
-    # If we need to override remote caps, feats, codecs or caps,
-    # this is a good time to do it
 
     # Connecting
     conn.Connect()
@@ -32,6 +28,12 @@ def test(q, bus, conn, stream):
     q.expect('dbus-signal', signal='StatusChanged',
             args=[cs.CONN_STATUS_CONNECTED, cs.CSR_REQUESTED])
 
+    worker(q, bus, conn, stream, 'foo@bar.com/Foo')
+    worker(q, bus, conn, stream, 'foo@sip.bar.com')
+
+def worker(q, bus, conn, stream, peer):
+    jt = jingletest.JingleTest(stream, 'test@localhost', peer)
+
     self_handle = conn.GetSelfHandle()
 
     # We need remote end's presence for capabilities
@@ -39,7 +41,7 @@ def test(q, bus, conn, stream):
 
     # Gabble doesn't trust it, so makes a disco
     event = q.expect('stream-iq', query_ns='http://jabber.org/protocol/disco#info',
-             to='foo@bar.com/Foo')
+             to=jt.remote_jid)
 
     jt.send_remote_disco_reply(event.stanza)
 
@@ -56,8 +58,11 @@ def test(q, bus, conn, stream):
 
     ret, old_sig, new_sig = q.expect_many(
         EventPattern('dbus-return', method='CreateChannel'),
-        EventPattern('dbus-signal', signal='NewChannel'),
-        EventPattern('dbus-signal', signal='NewChannels'),
+        EventPattern('dbus-signal', signal='NewChannel',
+            predicate=lambda e: cs.CHANNEL_TYPE_CONTACT_LIST not in e.args),
+        EventPattern('dbus-signal', signal='NewChannels',
+            predicate=lambda e:
+                cs.CHANNEL_TYPE_CONTACT_LIST not in e.args[0][0][1].values()),
         )
 
     path = ret.value[0]
@@ -79,7 +84,7 @@ def test(q, bus, conn, stream):
     assert emitted_props[cs.CHANNEL_TYPE] == cs.CHANNEL_TYPE_STREAMED_MEDIA
     assert emitted_props[cs.TARGET_HANDLE_TYPE] == cs.HT_CONTACT
     assert emitted_props[cs.TARGET_HANDLE] == handle
-    assert emitted_props[cs.TARGET_ID] == 'foo@bar.com', emitted_props
+    assert emitted_props[cs.TARGET_ID] == jt.remote_bare_jid, emitted_props
     assert emitted_props[cs.REQUESTED] == True
     assert emitted_props[cs.INITIATOR_HANDLE] == self_handle
     assert emitted_props[cs.INITIATOR_ID]  == 'test@localhost'
@@ -105,7 +110,7 @@ def test(q, bus, conn, stream):
               cs.TP_AWKWARD_PROPERTIES, cs.CHANNEL_IFACE_HOLD]:
         assert i in interfaces, (i, interfaces)
 
-    assert channel_props['TargetID'] == 'foo@bar.com', channel_props
+    assert channel_props['TargetID'] == jt.remote_bare_jid, channel_props
     assert channel_props['Requested'] == True
     assert channel_props['InitiatorID'] == 'test@localhost'
     assert channel_props['InitiatorHandle'] == self_handle
