@@ -8,14 +8,18 @@ FEATURE_NEG = 'http://jabber.org/protocol/feature-neg'
 FILE_TRANSFER = 'http://jabber.org/protocol/si/profile/file-transfer'
 GEOLOC = 'http://jabber.org/protocol/geoloc'
 GOOGLE_FEAT_SESSION = 'http://www.google.com/xmpp/protocol/session'
+GOOGLE_FEAT_SHARE = 'http://google.com/xmpp/protocol/share/v1'
 GOOGLE_FEAT_VOICE = 'http://www.google.com/xmpp/protocol/voice/v1'
 GOOGLE_FEAT_VIDEO = 'http://www.google.com/xmpp/protocol/video/v1'
 GOOGLE_JINGLE_INFO = 'google:jingleinfo'
 GOOGLE_P2P = "http://www.google.com/transport/p2p"
+GOOGLE_QUEUE = 'google:queue'
 GOOGLE_ROSTER = 'google:roster'
 GOOGLE_SESSION = "http://www.google.com/session"
+GOOGLE_SESSION_SHARE = "http://www.google.com/session/share"
 GOOGLE_SESSION_PHONE = "http://www.google.com/session/phone"
 GOOGLE_SESSION_VIDEO = "http://www.google.com/session/video"
+GOOGLE_MAIL_NOTIFY = "google:mail:notify"
 IBB = 'http://jabber.org/protocol/ibb'
 JINGLE_015 = "http://jabber.org/protocol/jingle"
 JINGLE_015_AUDIO = "http://jabber.org/protocol/jingle/description/audio"
@@ -44,15 +48,21 @@ OLPC_BUDDY_PROPS_NOTIFY = "%s+notify" % OLPC_BUDDY_PROPS
 OLPC_CURRENT_ACTIVITY = "http://laptop.org/xmpp/current-activity"
 OLPC_CURRENT_ACTIVITY_NOTIFY = "%s+notify" % OLPC_CURRENT_ACTIVITY
 PUBSUB = "http://jabber.org/protocol/pubsub"
+PUBSUB_EVENT = "%s#event" % PUBSUB
 REGISTER = "jabber:iq:register"
 ROSTER = "jabber:iq:roster"
+SEARCH = 'jabber:iq:search'
 SI = 'http://jabber.org/protocol/si'
 SI_MULTIPLE = 'http://telepathy.freedesktop.org/xmpp/si-multiple'
 STANZA = "urn:ietf:params:xml:ns:xmpp-stanzas"
 STREAMS = "urn:ietf:params:xml:ns:xmpp-streams"
+TEMPPRES = "urn:xmpp:temppres:0"
 TUBES = 'http://telepathy.freedesktop.org/xmpp/tubes'
+MUJI = 'http://telepathy.freedesktop.org/xmpp/muji'
 VCARD_TEMP = 'vcard-temp'
 VCARD_TEMP_UPDATE = 'vcard-temp:x:update'
 X_DATA = 'jabber:x:data'
+X_DELAY = 'jabber:x:delay'
 XML = 'http://www.w3.org/XML/1998/namespace'
 X_OOB = 'jabber:x:oob'
+GABBLE_CAPS="http://telepathy.freedesktop.org/caps"
