@@ -4,13 +4,9 @@ test OLPC search activity
 
 import dbus
 
-from servicetest import call_async, EventPattern, tp_name_prefix
-from gabbletest import (exec_test, make_result_iq, acknowledge_iq, sync_stream,
-    elem)
-
-from twisted.words.xish import domish, xpath
-from twisted.words.protocols.jabber.client import IQ
-
+from servicetest import EventPattern
+from gabbletest import exec_test, acknowledge_iq
+import constants as cs
 from util import announce_gadget, gadget_publish
 import ns
 
@@ -18,7 +14,8 @@ def test(q, bus, conn, stream):
     conn.Connect()
 
     _, iq_event, disco_event = q.expect_many(
-        EventPattern('dbus-signal', signal='StatusChanged', args=[0, 1]),
+        EventPattern('dbus-signal', signal='StatusChanged',
+            args=[cs.CONN_STATUS_CONNECTED, cs.CSR_REQUESTED]),
         EventPattern('stream-iq', to=None, query_ns='vcard-temp',
             query_name='vCard'),
         EventPattern('stream-iq', to='localhost', query_ns=ns.DISCO_ITEMS))
