@@ -2,7 +2,7 @@ import dbus
 import time
 import datetime
 
-from gabbletest import exec_test, make_result_iq, elem
+from gabbletest import exec_test, make_result_iq, elem, acknowledge_iq, send_error_reply
 from servicetest import call_async, EventPattern, assertEquals, assertLength
 
 from twisted.words.xish import xpath
@@ -12,6 +12,10 @@ import ns
 Rich_Presence_Access_Control_Type_Publish_List = 1
 
 def test(q, bus, conn, stream):
+    # we don't yet know we have PEP
+    assertEquals(0, conn.Get(cs.CONN_IFACE_LOCATION,
+        "SupportedLocationFeatures", dbus_interface=cs.PROPERTIES_IFACE))
+
     conn.Connect()
 
     # discard activities request and status change
@@ -22,6 +26,10 @@ def test(q, bus, conn, stream):
             args=[cs.CONN_STATUS_CONNECTED, cs.CSR_REQUESTED]),
         )
 
+    # we now know we have PEP
+    assertEquals(cs.LOCATION_FEATURE_CAN_SET, conn.Get(cs.CONN_IFACE_LOCATION,
+        "SupportedLocationFeatures", dbus_interface=cs.PROPERTIES_IFACE))
+
     # check location properties
 
     access_control_types = conn.Get(
@@ -101,7 +109,7 @@ def test(q, bus, conn, stream):
     date_str = datetime.datetime.utcfromtimestamp(date).strftime('%FT%H:%M:%SZ')
 
     # set a Location
-    conn.Location.SetLocation({
+    call_async(q, conn.Location, 'SetLocation', {
         'lat': dbus.Double(0.0, variant_level=1),
         'lon': 0.0,
         'language': 'en',
@@ -128,6 +136,21 @@ def test(q, bus, conn, stream):
     lat = xpath.queryForNodes('/geoloc/accuracy', geoloc)[0]
     assertEquals(float(str(lat)), 1.4)
 
+    acknowledge_iq(stream, event.stanza)
+    q.expect('dbus-return', method='SetLocation')
+
+    # Server refuses to set Location
+    call_async(q, conn.Location, 'SetLocation', {
+        'lat': 0.0,
+        'lon': 0.0})
+
+    geoloc_iq_set_event = EventPattern('stream-iq', predicate=lambda x:
+        xpath.queryForNodes("/iq/pubsub/publish/item/geoloc", x.stanza))
+    event = q.expect_many(geoloc_iq_set_event)[0]
+
+    send_error_reply(stream, event.stanza)
+    q.expect('dbus-error', method='SetLocation')
+
     # Request Bob's location
     bob_handle = conn.RequestHandles(1, ['bob@foo.com'])[0]
     call_async(q, conn.Location, 'GetLocations', [bob_handle])
@@ -178,11 +201,17 @@ def test(q, bus, conn, stream):
     assertLength(1, locations)
     assertEquals(locations[bob_handle], location)
 
+    charles_handle = conn.RequestHandles(cs.HT_CONTACT, ['charles@foo.com'])[0]
+
     # check that Contacts interface supports location
-    assert conn.Contacts.GetContactAttributes([bob_handle],
-        [cs.CONN_IFACE_LOCATION], False) == { bob_handle:
-            { cs.CONN_IFACE_LOCATION + '/location': location,
-              'org.freedesktop.Telepathy.Connection/contact-id': 'bob@foo.com'}}
+    assert conn.Contacts.GetContactAttributes([bob_handle, charles_handle],
+        [cs.CONN_IFACE_LOCATION], False) == {
+            bob_handle:
+              { cs.CONN_IFACE_LOCATION + '/location': location,
+                'org.freedesktop.Telepathy.Connection/contact-id': 'bob@foo.com'},
+            charles_handle:
+              { cs.CONN_IFACE_LOCATION + '/location': {},
+                'org.freedesktop.Telepathy.Connection/contact-id': 'charles@foo.com'}}
 
     # Try to set our location by passing a valid with an invalid type (lat is
     # supposed to be a double)
@@ -195,7 +224,7 @@ def test(q, bus, conn, stream):
 
     # Bob updates his location
     message = elem('message', from_='bob@foo.com')(
-        elem((ns.PUBSUB + "#event"), 'event')(
+        elem((ns.PUBSUB_EVENT), 'event')(
             elem('items', node=ns.GEOLOC)(
                 elem('item', id='12345')(
                     elem(ns.GEOLOC, 'geoloc')(
