@@ -7,13 +7,15 @@ import dbus
 
 from servicetest import EventPattern
 from gabbletest import exec_test
-
-ispresence = u'org.freedesktop.Telepathy.Connection.Interface.SimplePresence'
+import constants as cs
 
 def test_presence(q, bus, conn, stream):
     conn.Connect()
 
-    q.expect('dbus-signal', signal='StatusChanged', args=[0, 1])
+    q.expect_many(
+        EventPattern('dbus-signal', signal='StatusChanged',
+            args=[cs.CONN_STATUS_CONNECTED, cs.CSR_REQUESTED]),
+        EventPattern('stream-presence'))
 
     iface = dbus.Interface (conn,
         u'org.freedesktop.Telepathy.Connection.Interface.Presence')
@@ -29,9 +31,12 @@ def test_presence(q, bus, conn, stream):
 def test_simple_presence(q, bus, conn, stream):
     conn.Connect()
 
-    q.expect('dbus-signal', signal='StatusChanged', args=[0, 1])
+    q.expect_many(
+        EventPattern('dbus-signal', signal='StatusChanged',
+            args=[cs.CONN_STATUS_CONNECTED, cs.CSR_REQUESTED]),
+        EventPattern('stream-presence'))
 
-    iface = dbus.Interface (conn, ispresence)
+    iface = dbus.Interface (conn, cs.CONN_IFACE_SIMPLE_PRESENCE)
     run_test(q, bus, conn, stream,
       (lambda status, message = "": iface.SetPresence (status, message)))
 
@@ -46,8 +51,8 @@ def run_test(q, bus, conn, stream, set_status_func):
         EventPattern('stream-presence'))
     assert signal.args == [{1L: (0L, {u'away': {u'message': u'gone'}})}]
     assert simple_signal.args == [{1L: (3L, u'away',  u'gone')}]
-    assert conn.Contacts.GetContactAttributes([1], [ispresence], False) == { 1L:
-      { ispresence + "/presence": (3L, u'away', u'gone'),
+    assert conn.Contacts.GetContactAttributes([1], [cs.CONN_IFACE_SIMPLE_PRESENCE], False) == { 1L:
+      { cs.CONN_IFACE_SIMPLE_PRESENCE + "/presence": (3L, u'away', u'gone'),
         'org.freedesktop.Telepathy.Connection/contact-id':
             'test@localhost'}}
 
@@ -60,8 +65,8 @@ def run_test(q, bus, conn, stream, set_status_func):
     # Set presence a second time. Since this call is redundant, there should
     # be no PresenceUpdate or <presence> sent to the server.
     set_status_func('away', 'gone')
-    assert conn.Contacts.GetContactAttributes([1], [ispresence], False) == { 1L:
-      { ispresence + "/presence": (3L, u'away', u'gone'),
+    assert conn.Contacts.GetContactAttributes([1], [cs.CONN_IFACE_SIMPLE_PRESENCE], False) == { 1L:
+      { cs.CONN_IFACE_SIMPLE_PRESENCE + "/presence": (3L, u'away', u'gone'),
         'org.freedesktop.Telepathy.Connection/contact-id':
             'test@localhost'}}
 
@@ -79,8 +84,8 @@ def run_test(q, bus, conn, stream, set_status_func):
     children = list(presence.stanza.elements())
     assert children[0].name == 'status'
     assert str(children[0]) == 'yo'
-    assert conn.Contacts.GetContactAttributes([1], [ispresence], False) == { 1L:
-      { ispresence + "/presence": (2L, u'available', u'yo'),
+    assert conn.Contacts.GetContactAttributes([1], [cs.CONN_IFACE_SIMPLE_PRESENCE], False) == { 1L:
+      { cs.CONN_IFACE_SIMPLE_PRESENCE + "/presence": (2L, u'available', u'yo'),
         'org.freedesktop.Telepathy.Connection/contact-id':
             'test@localhost'}}
 
@@ -94,14 +99,11 @@ def run_test(q, bus, conn, stream, set_status_func):
         EventPattern('stream-presence'))
     assert signal.args == [{1L: (0L, {u'available': {}})}]
     assert simple_signal.args == [{1L: (2L, u'available',  u'')}]
-    assert conn.Contacts.GetContactAttributes([1], [ispresence], False) == { 1L:
-      { ispresence + "/presence": (2L, u'available', u''),
+    assert conn.Contacts.GetContactAttributes([1], [cs.CONN_IFACE_SIMPLE_PRESENCE], False) == { 1L:
+      { cs.CONN_IFACE_SIMPLE_PRESENCE + "/presence": (2L, u'available', u''),
         'org.freedesktop.Telepathy.Connection/contact-id':
             'test@localhost'}}
 
-    conn.Disconnect()
-    q.expect('dbus-signal', signal='StatusChanged', args=[2, 1])
-
 if __name__ == '__main__':
     exec_test(test_simple_presence)
     exec_test(test_presence)
