@@ -4,14 +4,14 @@ Test text channel initiated by me, using Requests.EnsureChannel
 
 import dbus
 
-from twisted.words.xish import domish
-
 from gabbletest import exec_test
 from servicetest import call_async, EventPattern
+import constants as cs
 
 def test(q, bus, conn, stream):
     conn.Connect()
-    q.expect('dbus-signal', signal='StatusChanged', args=[0, 1])
+    q.expect('dbus-signal', signal='StatusChanged',
+            args=[cs.CONN_STATUS_CONNECTED, cs.CSR_REQUESTED])
 
     self_handle = conn.GetSelfHandle()
 
@@ -22,37 +22,26 @@ def test(q, bus, conn, stream):
     handles = event.value[0]
 
     properties = conn.GetAll(
-            'org.freedesktop.Telepathy.Connection.Interface.Requests',
-            dbus_interface=dbus.PROPERTIES_IFACE)
+        cs.CONN_IFACE_REQUESTS, dbus_interface=cs.PROPERTIES_IFACE)
     assert properties.get('Channels') == [], properties['Channels']
-    assert ({'org.freedesktop.Telepathy.Channel.ChannelType':
-                'org.freedesktop.Telepathy.Channel.Type.Text',
-             'org.freedesktop.Telepathy.Channel.TargetHandleType': 1,
+    assert ({cs.CHANNEL_TYPE: cs.CHANNEL_TYPE_TEXT,
+             cs.TARGET_HANDLE_TYPE: cs.HT_CONTACT,
              },
-             ['org.freedesktop.Telepathy.Channel.TargetHandle',
-              'org.freedesktop.Telepathy.Channel.TargetID'
-             ],
+             [cs.TARGET_HANDLE, cs.TARGET_ID],
              ) in properties.get('RequestableChannelClasses'),\
                      properties['RequestableChannelClasses']
 
-    requestotron = dbus.Interface(conn,
-            'org.freedesktop.Telepathy.Connection.Interface.Requests')
+    test_ensure_ensure(q, conn, self_handle, jids[0], handles[0])
+    test_request_ensure(q, conn, self_handle, jids[1], handles[1])
 
-    test_ensure_ensure(q, requestotron, conn, self_handle, jids[0], handles[0])
-    test_request_ensure(q, requestotron, conn, self_handle, jids[1], handles[1])
-
-    conn.Disconnect()
-    q.expect('dbus-signal', signal='StatusChanged', args=[2, 1])
-
-
-def test_ensure_ensure(q, requestotron, conn, self_handle, jid, handle):
+def test_ensure_ensure(q, conn, self_handle, jid, handle):
     """
     Test ensuring a non-existant channel twice.  The first call should succeed
     with Yours=True; the subsequent call should succeed with Yours=False
     """
 
     # Check that Ensuring a channel that doesn't exist succeeds
-    call_async(q, requestotron, 'EnsureChannel', request_props (handle))
+    call_async(q, conn.Requests, 'EnsureChannel', request_props (handle))
 
     ret, old_sig, new_sig = q.expect_many(
         EventPattern('dbus-return', method='EnsureChannel'),
@@ -73,9 +62,8 @@ def test_ensure_ensure(q, requestotron, conn, self_handle, jid, handle):
     old_path, old_ct, old_ht, old_h, old_sh = old_sig.args
 
     assert old_path == path
-    assert old_ct == u'org.freedesktop.Telepathy.Channel.Type.Text'
-    # check that handle type == contact handle
-    assert old_ht == 1
+    assert old_ct == cs.CHANNEL_TYPE_TEXT
+    assert old_ht == cs.HT_CONTACT
     assert old_h == handle
     assert old_sh == True      # suppress handler
 
@@ -86,15 +74,14 @@ def test_ensure_ensure(q, requestotron, conn, self_handle, jid, handle):
     assert new_sig.args[0][0][1] == emitted_props
 
     properties = conn.GetAll(
-            'org.freedesktop.Telepathy.Connection.Interface.Requests',
-            dbus_interface=dbus.PROPERTIES_IFACE)
+        cs.CONN_IFACE_REQUESTS, dbus_interface=dbus.PROPERTIES_IFACE)
 
     assert new_sig.args[0][0] in properties['Channels'], \
             (new_sig.args[0][0], properties['Channels'])
 
 
     # Now try Ensuring a channel which already exists
-    call_async(q, requestotron, 'EnsureChannel', request_props (handle))
+    call_async(q, conn.Requests, 'EnsureChannel', request_props(handle))
     ret_ = q.expect('dbus-return', method='EnsureChannel')
 
     assert len(ret_.value) == 3
@@ -107,13 +94,13 @@ def test_ensure_ensure(q, requestotron, conn, self_handle, jid, handle):
     assert emitted_props == emitted_props_, (emitted_props, emitted_props_)
 
 
-def test_request_ensure(q, requestotron, conn, self_handle, jid, handle):
+def test_request_ensure(q, conn, self_handle, jid, handle):
     """
     Test Creating a non-existant channel, then Ensuring the same channel.
     The call to Ensure should succeed with Yours=False.
     """
 
-    call_async(q, requestotron, 'CreateChannel', request_props (handle))
+    call_async(q, conn.Requests, 'CreateChannel', request_props(handle))
 
     ret, old_sig, new_sig = q.expect_many(
         EventPattern('dbus-return', method='CreateChannel'),
@@ -130,9 +117,8 @@ def test_request_ensure(q, requestotron, conn, self_handle, jid, handle):
     old_path, old_ct, old_ht, old_h, old_sh = old_sig.args
 
     assert old_path == path
-    assert old_ct == u'org.freedesktop.Telepathy.Channel.Type.Text'
-    # check that handle type == contact handle
-    assert old_ht == 1
+    assert old_ct == cs.CHANNEL_TYPE_TEXT
+    assert old_ht == cs.HT_CONTACT
     assert old_h == handle
     assert old_sh == True      # suppress handler
 
@@ -143,15 +129,14 @@ def test_request_ensure(q, requestotron, conn, self_handle, jid, handle):
     assert new_sig.args[0][0][1] == emitted_props
 
     properties = conn.GetAll(
-            'org.freedesktop.Telepathy.Connection.Interface.Requests',
-            dbus_interface=dbus.PROPERTIES_IFACE)
+        cs.CONN_IFACE_REQUESTS, dbus_interface=dbus.PROPERTIES_IFACE)
 
     assert new_sig.args[0][0] in properties['Channels'], \
             (new_sig.args[0][0], properties['Channels'])
 
 
     # Now try Ensuring that same channel.
-    call_async(q, requestotron, 'EnsureChannel', request_props (handle))
+    call_async(q, conn.Requests, 'EnsureChannel', request_props(handle))
     ret_ = q.expect('dbus-return', method='EnsureChannel')
 
     assert len(ret_.value) == 3
@@ -165,26 +150,19 @@ def test_request_ensure(q, requestotron, conn, self_handle, jid, handle):
 
 
 def check_props(props, self_handle, handle, jid):
-    assert props['org.freedesktop.Telepathy.Channel.ChannelType'] ==\
-            'org.freedesktop.Telepathy.Channel.Type.Text'
-    assert props['org.freedesktop.Telepathy.Channel.'
-            'TargetHandleType'] == 1
-    assert props['org.freedesktop.Telepathy.Channel.TargetHandle'] ==\
-            handle
-    assert props['org.freedesktop.Telepathy.Channel.TargetID'] == jid
-    assert props['org.freedesktop.Telepathy.Channel.'
-            'Requested'] == True
-    assert props['org.freedesktop.Telepathy.Channel.'
-            'InitiatorHandle'] == self_handle
-    assert props['org.freedesktop.Telepathy.Channel.'
-            'InitiatorID'] == 'test@localhost'
+    assert props[cs.CHANNEL_TYPE] == cs.CHANNEL_TYPE_TEXT
+    assert props[cs.TARGET_HANDLE_TYPE] == cs.HT_CONTACT
+    assert props[cs.TARGET_HANDLE] == handle
+    assert props[cs.TARGET_ID] == jid
+    assert props[cs.REQUESTED] == True
+    assert props[cs.INITIATOR_HANDLE] == self_handle
+    assert props[cs.INITIATOR_ID] == 'test@localhost'
 
 
 def request_props(handle):
-    return { 'org.freedesktop.Telepathy.Channel.ChannelType':
-                'org.freedesktop.Telepathy.Channel.Type.Text',
-             'org.freedesktop.Telepathy.Channel.TargetHandleType': 1,
-             'org.freedesktop.Telepathy.Channel.TargetHandle': handle,
+    return { cs.CHANNEL_TYPE: cs.CHANNEL_TYPE_TEXT,
+             cs.TARGET_HANDLE_TYPE: cs.HT_CONTACT,
+             cs.TARGET_HANDLE: handle,
            }
 
 
