@@ -1,5 +1,7 @@
 """Test IBB stream tube support in the context of a MUC."""
 
+import sys
+
 import dbus
 
 from servicetest import call_async, EventPattern, EventProtocolClientFactory, unwrap, assertEquals
@@ -42,13 +44,6 @@ def test(q, bus, conn, stream, bytestream_cls,
     call_async(q, conn, 'RequestHandles', 2,
         ['chat@conf.localhost'])
 
-    event = q.expect('stream-iq', to='conf.localhost',
-            query_ns='http://jabber.org/protocol/disco#info')
-    result = make_result_iq(stream, event.stanza)
-    feature = result.firstChildElement().addElement('feature')
-    feature['var'] = 'http://jabber.org/protocol/muc'
-    stream.send(result)
-
     event = q.expect('dbus-return', method='RequestHandles')
     handles = event.value[0]
     room_handle = handles[0]
@@ -175,6 +170,8 @@ def test(q, bus, conn, stream, bytestream_cls,
     assert props[cs.TARGET_ID] == 'chat@conf.localhost'
     assert props[cs.STREAM_TUBE_SERVICE] == 'echo'
     assert props[cs.TUBE_PARAMETERS] == {'s': 'hello', 'ay': 'hello', 'u': 123, 'i': -123}
+    assert access_control in \
+            props[cs.STREAM_TUBE_SUPPORTED_SOCKET_TYPES][address_type]
 
     tube_chan = bus.get_object(conn.bus_name, path)
     tube_props = tube_chan.GetAll(cs.CHANNEL_IFACE_TUBE, dbus_interface=cs.PROPERTIES_IFACE,
