@@ -81,10 +81,10 @@ def test(q, bus, conn, stream):
     e = q.expect('dbus-error', method='OfferDBusTube').error
     assert e.get_dbus_name() == cs.NOT_AVAILABLE, e.get_dbus_name()
 
-    address = t.create_server(q, cs.SOCKET_ADDRESS_TYPE_UNIX)
+    address = t.create_server(q, cs.SOCKET_ADDRESS_TYPE_IPV4)
 
     call_async(q, tubes, 'OfferStreamTube', 'echo', {},
-        cs.SOCKET_ADDRESS_TYPE_UNIX, address,
+        cs.SOCKET_ADDRESS_TYPE_IPV4, address,
         cs.SOCKET_ACCESS_CONTROL_LOCALHOST, "")
     e = q.expect('dbus-error', method='OfferStreamTube').error
     assert e.get_dbus_name() == cs.NOT_AVAILABLE, e.get_dbus_name()
@@ -96,7 +96,7 @@ def test(q, bus, conn, stream):
         {cs.STREAM_TUBE_SERVICE: "newecho"}))
     st_chan = bus.get_object(conn.bus_name, st_path)
     st = dbus.Interface(st_chan, cs.CHANNEL_TYPE_STREAM_TUBE)
-    call_async(q, st, 'Offer', cs.SOCKET_ADDRESS_TYPE_UNIX,
+    call_async(q, st, 'Offer', cs.SOCKET_ADDRESS_TYPE_IPV4,
         address, cs.SOCKET_ACCESS_CONTROL_LOCALHOST, {})
     e = q.expect('dbus-error', method='Offer').error
     assert e.get_dbus_name() == cs.NOT_AVAILABLE, e.get_dbus_name()
