@@ -5,15 +5,16 @@ RequestAvatar calls for the same contact.
 """
 
 import base64
-import sha
 
 from servicetest import EventPattern
-from gabbletest import exec_test, acknowledge_iq
+from gabbletest import exec_test, acknowledge_iq, make_result_iq
+import constants as cs
 
 def test(q, bus, conn, stream):
     conn.Connect()
     _, iq_event = q.expect_many(
-        EventPattern('dbus-signal', signal='StatusChanged', args=[0, 1]),
+        EventPattern('dbus-signal', signal='StatusChanged',
+            args=[cs.CONN_STATUS_CONNECTED, cs.CSR_REQUESTED]),
         EventPattern('stream-iq', to=None, query_ns='vcard-temp',
             query_name='vCard'))
 
@@ -27,26 +28,16 @@ def test(q, bus, conn, stream):
 
     iq_event = q.expect('stream-iq', to='bob@foo.com', query_ns='vcard-temp',
         query_name='vCard')
-    iq = iq_event.stanza
-    vcard = iq_event.query
+    iq = make_result_iq(stream, iq_event.stanza)
+    vcard = iq.firstChildElement()
     photo = vcard.addElement('PHOTO')
     photo.addElement('TYPE', content='image/png')
     photo.addElement('BINVAL', content=base64.b64encode('hello'))
-    iq['type'] = 'result'
     stream.send(iq)
 
     event = q.expect('dbus-signal', signal='AvatarRetrieved')
 
-    conn.Disconnect()
-
-    e = q.expect('dbus-signal')
-
-    # We really need q.forbid(...) somehow
-    assert e.signal != 'AvatarRetrieved'
-
-    assert e.signal == 'StatusChanged'
-    assert e.args == [2, 1]
-
+    q.forbid_events([EventPattern('dbus-signal', signal='AvatarRetrieved')])
 
 if __name__ == '__main__':
     exec_test(test)
