@@ -7,11 +7,13 @@ import base64
 
 from servicetest import call_async, EventPattern
 from gabbletest import exec_test, acknowledge_iq, make_result_iq
+import constants as cs
 
 def test(q, bus, conn, stream):
     conn.Connect()
     _, event = q.expect_many(
-        EventPattern('dbus-signal', signal='StatusChanged', args=[0, 1]),
+        EventPattern('dbus-signal', signal='StatusChanged',
+            args=[cs.CONN_STATUS_CONNECTED, cs.CSR_REQUESTED]),
         EventPattern('stream-iq', to=None, query_ns='vcard-temp',
             query_name='vCard'))
 
@@ -31,9 +33,5 @@ def test(q, bus, conn, stream):
     q.expect('dbus-return', method='RequestAvatar',
         value=('hello', 'image/png'))
 
-    conn.Disconnect()
-    q.expect('dbus-signal', signal='StatusChanged', args=[2, 1])
-
 if __name__ == '__main__':
     exec_test(test)
-
