@@ -5,11 +5,13 @@ Test alias setting support.
 
 from servicetest import EventPattern
 from gabbletest import exec_test, acknowledge_iq
+import constants as cs
 
 def test(q, bus, conn, stream):
     conn.Connect()
     _, iq_event = q.expect_many(
-        EventPattern('dbus-signal', signal='StatusChanged', args=[0, 1]),
+        EventPattern('dbus-signal', signal='StatusChanged',
+            args=[cs.CONN_STATUS_CONNECTED, cs.CSR_REQUESTED]),
         EventPattern('stream-iq', to=None, query_ns='vcard-temp',
             query_name='vCard'))
 
@@ -24,9 +26,5 @@ def test(q, bus, conn, stream):
     event = q.expect('dbus-signal', signal='AliasesChanged',
         args=[[(1, u'lala')]])
 
-    conn.Disconnect()
-    q.expect('dbus-signal', signal='StatusChanged', args=[2, 1])
-
 if __name__ == '__main__':
     exec_test(test)
-
